@@ -59,6 +59,7 @@ attempt to call the Prolog defined trace interceptor.
 :- meta_predicate
 	dynamic(:),
 	multifile(:),
+	public(:),
 	module_transparent(:),
 	discontiguous(:),
 	volatile(:),
@@ -87,6 +88,7 @@ discontiguous(Spec)	 :- '$set_pattr'(Spec, (discontiguous)).
 volatile(Spec)		 :- '$set_pattr'(Spec, (volatile)).
 thread_local(Spec)	 :- '$set_pattr'(Spec, (thread_local)).
 noprofile(Spec)		 :- '$set_pattr'(Spec, (noprofile)).
+public(Spec)		 :- '$set_pattr'(Spec, (public)).
 '$iso'(Spec)		 :- '$set_pattr'(Spec, (iso)).
 
 %%	'$hide'(:PI)
@@ -168,26 +170,26 @@ noprofile(Spec)		 :- '$set_pattr'(Spec, (noprofile)).
 
 call(Goal) :-				% make these available as predicates
 	Goal.
-call(G, A) :-
-	call(G, A).
-call(G, A, B) :-
-	call(G, A, B).
-call(G, A, B, C) :-
-	call(G, A, B, C).
-call(G, A, B, C, D) :-
-	call(G, A, B, C, D).
-call(G, A, B, C, D, E) :-
-	call(G, A, B, C, D, E).
-call(G, A, B, C, D, E, F) :-
-	call(G, A, B, C, D, E, F).
-call(G, A, B, C, D, E, F, G) :-
-	call(G, A, B, C, D, E, F, G).
-call(G, A, B, C, D, E, F, G, H) :-
-	call(G, A, B, C, D, E, F, G, H).
-call(G, A, B, C, D, E, F, G, H, I) :-
-	call(G, A, B, C, D, E, F, G, H, I).
-call(G, A, B, C, D, E, F, G, H, I, J) :-
-	call(G, A, B, C, D, E, F, G, H, I, J).
+call(Goal, A) :-
+	call(Goal, A).
+call(Goal, A, B) :-
+	call(Goal, A, B).
+call(Goal, A, B, C) :-
+	call(Goal, A, B, C).
+call(Goal, A, B, C, D) :-
+	call(Goal, A, B, C, D).
+call(Goal, A, B, C, D, E) :-
+	call(Goal, A, B, C, D, E).
+call(Goal, A, B, C, D, E, F) :-
+	call(Goal, A, B, C, D, E, F).
+call(Goal, A, B, C, D, E, F, G) :-
+	call(Goal, A, B, C, D, E, F, G).
+call(Goal, A, B, C, D, E, F, G, H) :-
+	call(Goal, A, B, C, D, E, F, G, H).
+call(Goal, A, B, C, D, E, F, G, H, I) :-
+	call(Goal, A, B, C, D, E, F, G, H, I).
+call(Goal, A, B, C, D, E, F, G, H, I, J) :-
+	call(Goal, A, B, C, D, E, F, G, H, I, J).
 
 not(Goal) :-
 	\+ Goal.
@@ -234,8 +236,12 @@ catch(_Goal, _Catcher, _Recover) :-
 %%	call_cleanup(:Goal, :Cleanup)
 %%	call_cleanup(:Goal, +Catcher, :Cleanup)
 %
-%	Call Cleanup once after Goal is finished (deterministic success, failure,
-%	exception or cut).  '$call_cleanup' translated to I_CALLCLEANUP.
+%	Call Cleanup once after Goal is finished (deterministic success,
+%	failure, exception or  cut).  The   call  to  '$call_cleanup' is
+%	translated to I_CALLCLEANUP. This  instruction   relies  on  the
+%	exact stack layout left   by  setup_call_catcher_cleanup/4. Also
+%	the predicate name is used by   the kernel cleanup mechanism and
+%	can only be changed together with the kernel.
 
 setup_call_catcher_cleanup(Setup, _Goal, _Catcher, _Cleanup) :-
 	'$sig_atomic'(Setup),
@@ -553,10 +559,14 @@ absolute_file_name(Spec, Args, Path) :-
 	    Exts = ['']
 	),
 	'$canonise_extensions'(Exts, Extensions),
-	(   '$select'(solutions(Sols), Conditions, C1)
+	(   nonvar(Type)
+	->  C0 = Conditions
+	;   C0 = [file_type(regular)|Conditions] % ask for a regular file
+	),
+	(   '$select'(solutions(Sols), C0, C1)
 	->  true
 	;   Sols = first,
-	    C1 = Conditions
+	    C1 = C0
 	),
 	(   '$select'(file_errors(FileErrors), C1, C2)
 	->  true
@@ -607,6 +617,8 @@ user:prolog_file_type(qlf,	qlf).
 user:prolog_file_type(Ext,	executable) :-
 	current_prolog_flag(shared_object_extension, Ext).
 
+%%	'$chk_file'(+Spec, +Extensions, +Cond, +UseCache, -FullName)
+%
 %	File is a specification of a Prolog source file. Return the full
 %	path of the file.
 
@@ -1500,6 +1512,11 @@ load_files(Module:Files, Options) :-
 '$load_module'(Reserved, _, _, _, _, _) :-
 	'$reserved_module'(Reserved), !,
 	throw(error(permission_error(load, module, Reserved), _)).
+'$load_module'(Module, _Public, _Cls, _In, File, _Options) :-
+	'$current_module'(Module, OldFile),
+	source_location(File, _Line),
+	OldFile \== File,
+	same_file(OldFile, File), !.
 '$load_module'(Module, Public, Cls, In, File, Options) :-
 	'$set_source_module'(OldModule, OldModule),
 	source_location(_File, Line),
@@ -2082,6 +2099,13 @@ saved state.
 '$delete'([Head|Tail], Elem, [Head|Rest]) :-
 	'$delete'(Tail, Elem, Rest).
 
+'$last'([H|T], Last) :-
+	'$last'(T, H, Last).
+
+'$last'([], Last, Last).
+'$last'([H|T], _, Last) :-
+	'$last'(T, H, Last).
+
 
 		 /*******************************
 		 *   HANDLE TRACER 'L'-COMMAND	*
