@@ -207,8 +207,8 @@ is_not(chars, X) :- !,
 	not_a_list(chars, X).
 is_not(codes, X) :- !,
 	not_a_list(codes, X).
-is_not(var,_X) :- !,
-	representation_error(variable).
+is_not(var,X) :- !,
+	throw(error(uninstantiation_error(X), _)).
 is_not(rational, X) :- !,
 	not_a_rational(X).
 is_not(Type, X) :-
