@@ -36,7 +36,7 @@
 	  portray_clause/2		% +Stream, +Clause
 	]).
 :- use_module(library(lists), [member/2]).
-%:- set_prolog_flag(generate_debug_info, false).
+:- set_prolog_flag(generate_debug_info, false).
 
 :- module_transparent
 	listing/0.
