@@ -72,7 +72,7 @@ clause_info(ClauseRef, File, TermPos, NameOffset) :-
 	debug(clause_info, 'from ~w:~d ... ', [File, LineNo]),
 	read_term_at_line(File, LineNo, Module, Clause, TermPos0, VarNames),
 	debug(clause_info, 'read ...', []),
-	unify_clause(Clause, DecompiledClause, TermPos0, TermPos),
+	unify_clause(Clause, DecompiledClause, Module, TermPos0, TermPos),
 	debug(clause_info, 'unified ...', []),
 	make_varnames(Clause, VarOffset, VarNames, NameOffset),
 	debug(clause_info, 'got names~n', []), !.
@@ -88,6 +88,9 @@ clause_info(ClauseRef, File, TermPos, NameOffset) :-
 %	NOTE: Called directly from  library(trace/clause)   for  the GUI
 %	tracer.
 
+:- public
+	unify_term/2.
+
 unify_term(X, X) :- !.
 unify_term(X1, X2) :-
 	compound(X1),
@@ -218,7 +221,8 @@ find_varname(Var, [Name = TheVar|_], Name) :-
 find_varname(Var, [_|T], Name) :-
 	find_varname(Var, T, Name).
 
-%%	unify_clause(+Read, +Decompiled, +ReadTermPos, -RecompiledTermPos).
+%%	unify_clause(+Read, +Decompiled, +Module, +ReadTermPos,
+%%		     -RecompiledTermPos).
 %
 %	What you read isn't always what goes into the database. The task
 %	of this predicate is to establish  the relation between the term
@@ -228,16 +232,16 @@ find_varname(Var, [_|T], Name) :-
 %	complex source-translations, falling back to  a heuristic method
 %	locating as much as possible.
 
-unify_clause(Read, Read, TermPos, TermPos) :- !.
+unify_clause(Read, Read, _, TermPos, TermPos) :- !.
 					% XPCE send-methods
-unify_clause(:->(Head, Body), (PlHead :- PlBody), TermPos0, TermPos) :- !,
+unify_clause(:->(Head, Body), (PlHead :- PlBody), _, TermPos0, TermPos) :- !,
 	pce_method_clause(Head, Body, PlHead, PlBody, TermPos0, TermPos).
 					% XPCE get-methods
-unify_clause(:<-(Head, Body), (PlHead :- PlBody), TermPos0, TermPos) :- !,
+unify_clause(:<-(Head, Body), (PlHead :- PlBody), _, TermPos0, TermPos) :- !,
 	pce_method_clause(Head, Body, PlHead, PlBody, TermPos0, TermPos).
 					% Unit test clauses
 unify_clause((TH :- Body),
-	     (_:'unit body'(_, _) :- !, Body),
+	     (_:'unit body'(_, _) :- !, Body), _,
 	     TP0, TP) :-
 	(   TH = test(_,_)
 	;   TH = test(_)
@@ -246,17 +250,17 @@ unify_clause((TH :- Body),
 	TP  = term_position(F,T,FF,FT,[HP,term_position(0,0,0,0,[FF-FT,BP])]).
 					% module:head :- body
 unify_clause((Head :- Read),
-	     (Head :- _M:Compiled), TermPos0, TermPos) :-
-	unify_clause((Head :- Read), (Head :- Compiled), TermPos0, TermPos1),
+	     (Head :- _M:Compiled), Module, TermPos0, TermPos) :-
+	unify_clause((Head :- Read), (Head :- Compiled), Module, TermPos0, TermPos1),
 	TermPos1 = term_position(TA,TZ,FA,FZ,[PH,PB]),
 	TermPos  = term_position(TA,TZ,FA,FZ,
 				 [ PH,
 				   term_position(0,0,0,0,[0-0,PB])
 				 ]).
-unify_clause(Read, Compiled1, TermPos0, TermPos) :-
+unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
 	Read = (_ --> List, _),
 	is_list(List),
-	catch(expand_term(Read, Compiled2), E, expand_failed(E, Read)),
+	ci_expand(Read, Compiled2, Module),
 	Compiled2 = (DH :- _),
 	functor(DH, _, Arity),
 	DArg is Arity - 1,
@@ -268,11 +272,11 @@ unify_clause(Read, Compiled1, TermPos0, TermPos) :-
 	TermPos1 = term_position(F,T,FF,FT,[ HP, BP ]),
 	match_module(Compiled2, Compiled1, TermPos1, TermPos).
 					% general term-expansion
-unify_clause(Read, Compiled1, TermPos0, TermPos) :-
-	catch(expand_term(Read, Compiled2), E, expand_failed(E, Read)),
+unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
+	ci_expand(Read, Compiled2, Module),
 	match_module(Compiled2, Compiled1, TermPos0, TermPos).
 					% I don't know ...
-unify_clause(_, _, _, _) :-
+unify_clause(_, _, _, _, _) :-
 	debug(clause_info, 'Could not unify clause', []),
 	fail.
 
@@ -280,6 +284,13 @@ unify_clause_head(H1, H2) :-
 	strip_module(H1, _, H),
 	strip_module(H2, _, H).
 
+ci_expand(Read, Compiled, Module) :-
+	catch(setup_call_cleanup('$set_source_module'(Old, Module),
+				 expand_term(Read, Compiled),
+				 '$set_source_module'(_, Old)),
+	      E,
+	      expand_failed(E, Read)).
+
 match_module((H1 :- B1), (H2 :- B2), Pos0, Pos) :- !,
 	unify_clause_head(H1, H2),
 	unify_body(B1, B2, Pos0, Pos).
@@ -304,13 +315,26 @@ expand_failed(E, Read) :-
 %	Pos0 and Pos still include the term-position of the head.
 
 unify_body(B, B, Pos, Pos) :-
-	acyclic_term(B),		% X = call(X)
-	\+ sub_term(brace_term_position(_,_,_), Pos), !.
+	does_not_dcg_after_binding(B, Pos), !.
 unify_body(R, D,
 	   term_position(F,T,FF,FT,[HP,BP0]),
 	   term_position(F,T,FF,FT,[HP,BP])) :-
 	ubody(R, D, BP0, BP).
 
+%%	does_not_dcg_after_binding(+ReadBody, +ReadPos) is semidet.
+%
+%	True  if  ReadPos/ReadPos  does   not    contain   DCG   delayed
+%	unifications.
+%
+%	@tbd	We should pass that we are in a DCG; if we are not there
+%		is no reason for this test.
+
+does_not_dcg_after_binding(B, Pos) :-
+	acyclic_term(B),		% X = call(X)
+	\+ sub_term(brace_term_position(_,_,_), Pos),
+	\+ (sub_term((Cut,_=_), B), Cut == !), !.
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Some remarks.
 
@@ -326,8 +350,7 @@ a --> { x, y, z }.
 %	@param TermPosRead	Sub-term positions of source
 
 ubody(B, B, P, P) :-
-	acyclic_term(B),		% X = call(X)
-	\+ sub_term(brace_term_position(_,_,_), P), !.
+	does_not_dcg_after_binding(B, P), !.
 ubody(X, call(X),			% X = call(X)
       From-To,
       term_position(From, To, From, To, [From-To])) :- !.
@@ -395,6 +418,9 @@ conj((A,B), brace_term_position(_,T,PA), GL, TG, PL, TP) :-
 	conj(A, PA, GL, TGA, PL, TPA),
 	T1 is T - 1,
 	conj(B, T1-T, TGA, TG, TPA, TP).
+conj((!,(S=SR)), F-T, [!,S=SR|TG], TG, [F-T,F1-T1|TP], TP) :-
+	F1 is F+1,
+	T1 is T+1.
 conj(A, P, [A|TG], TG, [P|TP], TP).
 
 
