@@ -25,11 +25,16 @@
 #include <SWI-Prolog.h>
 #include <string.h>
 #include <stdlib.h>
+#include <assert.h>
 #include "clib.h"
 #include "form.h"
 
+#ifdef __WINDOWS__		/* strtoll is C99, but it is only 2011 ... */
+#define strtoll(s,e,b) _strtoi64(s,e,b)
+#endif
+
 static int
-isinteger(const char *s, long *val, size_t len)
+isinteger(const char *s, long long *val, size_t len)
 { char *e;
 
   if ( len == (size_t)-1 )
@@ -37,7 +42,7 @@ isinteger(const char *s, long *val, size_t len)
   if ( len == 0 )
     return FALSE;
 
-  *val = strtol(s, &e, 10);
+  *val = strtoll(s, &e, 10);
   if ( e == s+len )
     return TRUE;
 
@@ -63,81 +68,117 @@ isfloat(const char *s, double *val, size_t len)
 
 
 static int
-add_to_form(const char *name, const char *value, void *closure)
+add_to_form(const char *name, size_t nlen,
+	    const char *value, size_t len,
+	    void *closure)
 { term_t head = PL_new_term_ref();
   term_t tail = (term_t) closure;
   term_t val  = PL_new_term_ref();
-  long vl;
+  long long vl;
   double vf;
-  size_t len = strlen(value);		/* TBD: pass this in */
+  int rc;
+  atom_t aname = 0;
 
   if ( isinteger(value, &vl, len) )
-    PL_put_integer(val, vl);
+    rc = PL_put_int64(val, vl);
   else if ( isfloat(value, &vf, len) )
-    PL_put_float(val, vf);
+    rc = PL_put_float(val, vf);
   else
-    PL_unify_chars(val, PL_ATOM|REP_UTF8, len, value);
+    rc = PL_unify_chars(val, PL_ATOM|REP_UTF8, len, value);
 
-  if ( !PL_unify_list(tail, head, tail) ||
-       !PL_unify_term(head,
-		      PL_FUNCTOR, PL_new_functor(PL_new_atom(name), 1),
-		      PL_TERM, val) )
-    return FALSE;
+  rc = ( rc &&
+	 PL_unify_list(tail, head, tail) &&
+	 (aname = PL_new_atom_nchars(nlen, name)) &&
+	 PL_unify_term(head,
+		       PL_FUNCTOR, PL_new_functor(aname, 1),
+		       PL_TERM, val) );
 
-  return TRUE;
+  if ( aname )
+    PL_unregister_atom(aname);
+
+  return rc;
 }
 
 
 static int
-mp_add_to_form(const char *name, const char *value, size_t len,
+mp_add_to_form(const char *name, size_t nlen,
+	       const char *value, size_t len,
 	       const char *file, void *closure)
 { term_t head = PL_new_term_ref();
   term_t tail = (term_t) closure;
   term_t val  = PL_new_term_ref();
-  long vl;
+  long long vl;
   double vf;
+  int rc;
+  atom_t aname = 0;
 
   if ( isinteger(value, &vl, len) )
-    PL_put_integer(val, vl);
+    rc = PL_put_int64(val, vl);
   else if ( isfloat(value, &vf, len) )
-    PL_put_float(val, vf);
+    rc = PL_put_float(val, vf);
   else
-    PL_unify_chars(val, PL_ATOM|REP_UTF8, len, value);
+    rc = PL_unify_chars(val, PL_ATOM|REP_UTF8, len, value);
 
-  if ( !PL_unify_list(tail, head, tail) ||
-       !PL_unify_term(head,
-		      PL_FUNCTOR, PL_new_functor(PL_new_atom(name), 1),
-		      PL_TERM, val) )
-    return FALSE;
+  rc = ( rc &&
+	 PL_unify_list(tail, head, tail) &&
+	 (aname = PL_new_atom_nchars(nlen, name)) &&
+	 PL_unify_term(head,
+			PL_FUNCTOR, PL_new_functor(aname, 1),
+			PL_TERM, val) );
 
-  return TRUE;
+  if ( aname )
+    PL_unregister_atom(aname);
+
+  return rc;
 }
 
 
 static foreign_t
 pl_cgi_get_form(term_t form)
 { size_t len = 0;
-  char *data = get_raw_form_data(&len);
+  char *data;
+  int must_free = FALSE;
   term_t list = PL_copy_term_ref(form);
   char *ct, *boundary;
 
-  if ( !data )
-  { term_t ctx = PL_new_term_ref();
-
-    PL_put_nil(ctx);
-    return pl_error("cgi_get_form", 1, "no data?",
-		    ERR_EXISTENCE, "cgi_form", ctx);
-  }
+  if ( !get_raw_form_data(&data, &len, &must_free) )
+    return FALSE;
 
   if ( (ct = getenv("CONTENT_TYPE")) &&
        (boundary = strstr(ct, "boundary=")) )
   { boundary = strchr(boundary, '=')+1;
 
-    break_multipart(data, len, boundary, mp_add_to_form, (void *)list);
+    switch( break_multipart(data, len, boundary,
+			    mp_add_to_form, (void *)list) )
+    { case FALSE:
+	return FALSE;
+      case TRUE:
+	break;
+      default:
+	assert(0);
+        return FALSE;
+    }
   } else
-  { break_form_argument(data, add_to_form, (void *)list);
+  { switch( break_form_argument(data, add_to_form, (void *)list) )
+    { case FALSE:
+	return FALSE;
+      case TRUE:
+	break;
+      case ERROR_NOMEM:
+	return pl_error("cgi_get_form", 1, NULL,
+			ERR_RESOURCE, "memory");
+      case ERROR_SYNTAX_ERROR:
+	return pl_error("cgi_get_form", 1, NULL,
+			ERR_SYNTAX, "cgi_value");
+      default:
+	assert(0);
+        return FALSE;
+    }
   }
 
+  if ( must_free )
+    free(data);
+
   return PL_unify_nil(list);
 }
 
