@@ -50,7 +50,7 @@ by Double Precision, Inc, part of the maildrop system.
 Parsing MIME messages is accomplished  using   a  single predicate. This
 predicate parses the input  and  returns   a  complex  term  holding the
 various MIME message  parts.  The  mime   message  is  encoded  into the
-following structure: 
+following structure:
 
 	mime(Attributes, Data, SubMimeList)
 
@@ -86,8 +86,8 @@ static functor_t FUNCTOR_filename1;
 
 struct dbuf
 { char *buf;
-  int size;
-  int allocated;
+  size_t size;
+  size_t allocated;
 };
 
 static int
@@ -167,7 +167,7 @@ mime_unify(term_t result, struct rfc2045 *rfc, const char *buffer)
 { term_t data = PL_new_term_ref();
   term_t subs = PL_new_term_ref();
   term_t atts = PL_new_term_ref();
-  
+
   if ( !PL_unify_term(result,
 		      PL_FUNCTOR, FUNCTOR_mime3,
 		        PL_TERM, atts,
@@ -176,8 +176,8 @@ mime_unify(term_t result, struct rfc2045 *rfc, const char *buffer)
     return FALSE;
 
   if ( rfc->isdummy )
-  { PL_unify_nil(data);			/* dubious */
-    if ( !PL_unify_nil(atts) )
+  { if ( !PL_unify_nil(data) ||
+	 !PL_unify_nil(atts) )
       return FALSE;
   } else
   { term_t at = PL_copy_term_ref(atts);
@@ -188,7 +188,7 @@ mime_unify(term_t result, struct rfc2045 *rfc, const char *buffer)
     const char *desc = rfc2045_content_description(rfc);
     const char *lang = rfc2045_content_language(rfc);
     const char *md5  = rfc2045_content_md5(rfc);
-    
+
     rfc2045_mimeinfo(rfc, &type, &enc, &cset);
     rfc2045_dispositioninfo(rfc, &disp, &name, &fnam);
 
@@ -256,7 +256,7 @@ get_character_data(term_t from, char **data, size_t *len, int *malloced)
     { IOSTREAM *stream;
       term_t arg = PL_new_term_ref();
 
-      PL_get_arg(1, from, arg);
+      _PL_get_arg(1, from, arg);
       if ( !PL_get_stream_handle(arg, &stream) )
 	return pl_error(NULL, 0, NULL, ERR_ARGTYPE, 1, from, "stream");
 
@@ -266,7 +266,7 @@ get_character_data(term_t from, char **data, size_t *len, int *malloced)
 
 	if ( !(buf = malloc(allocated)) )
 	  return pl_error(NULL, 0, NULL, ERR_ERRNO, errno, "allocate", "memory", 0);
-    
+
 	for( done=0; (c=Sgetcode(stream)) != EOF; )
 	{ if ( done >= allocated )
 	  { allocated *= 2;
@@ -288,13 +288,13 @@ get_character_data(term_t from, char **data, size_t *len, int *malloced)
 	long done;
 	int c;
 
-	PL_get_arg(2, from, arg);
+	_PL_get_arg(2, from, arg);
 	if ( !PL_get_long(arg, &size) || size < 0 )
 	  return pl_error(NULL, 0, NULL, ERR_ARGTYPE, 1, arg, "natural");
-	
+
 	if ( !(buf = malloc(size)) )
 	  return pl_error(NULL, 0, NULL, ERR_ERRNO, errno, "allocate", "memory", 0);
-    
+
 	for( done=0; (c=Sgetcode(stream)) != EOF && done < size; )
 	  buf[done++] = c;
 
@@ -352,13 +352,15 @@ void
 rfc2045_error(const char *errmsg)
 { term_t e = PL_new_term_ref();
 
-  PL_unify_term(e,
-		PL_FUNCTOR_CHARS, "error", 2,
-		  PL_FUNCTOR_CHARS, "mime", 1,
-		    PL_CHARS, errmsg,
-		  PL_VARIABLE);
+  if ( (e=PL_new_term_ref()) &&
+       PL_unify_term(e,
+		     PL_FUNCTOR_CHARS, "error", 2,
+		       PL_FUNCTOR_CHARS, "mime", 1,
+		         PL_CHARS, errmsg,
+		       PL_VARIABLE) )
+    PL_throw(e);
 
-  PL_throw(e);
+  PL_fatal_error("Could not recover from rfc2045 error");
 }
 
 		 /*******************************
