@@ -127,6 +127,13 @@ leave the details to this function.
 #include <malloc.h>
 #endif
 
+#ifdef __WINDOWS__
+#define GET_ERRNO WSAGetLastError()
+#define GET_H_ERRNO WSAGetLastError()
+#else
+#define GET_ERRNO errno
+#define GET_H_ERRNO h_errno
+#endif
 
 #ifndef __WINDOWS__
 #define closesocket(n) close((n))	/* same on Unix */
@@ -196,6 +203,7 @@ typedef struct _plsocket
   nbio_request	    request;		/* our request */
   DWORD		    thread;		/* waiting thread */
   DWORD		    error;		/* error while executing request */
+  DWORD             close_timeout;      /* Time to reap socket if FD_CLOSE is not received */
   int		    done;		/* request completed */
   int		    w32_flags;		/* or of received FD_* */
   union
@@ -242,6 +250,7 @@ typedef struct _plsocket
 static plsocket *allocSocket(SOCKET socket);
 #ifdef __WINDOWS__
 static plsocket *lookupOSSocket(SOCKET socket);
+static int socketIsPendingClose(plsocket *s);
 static const char *WinSockError(unsigned long eno);
 #endif
 
@@ -380,6 +389,7 @@ doneRequest(plsocket *s)
   s->done = TRUE;
   s->request = REQ_NONE;
 
+  /* If we have FD_CLOSE, then we know we can release the socket to the OS */
   if ( (s->w32_flags & FD_CLOSE) && (sock=s->socket) >= 0 )
   { s->socket = -1;
     closesocket(sock);
@@ -884,8 +894,13 @@ socket_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 
 	s->error = err;
 	if ( sock )
-	{ s->socket = -1;
-	  closesocket(sock);
+	{ /* We cannot close the socket yet, since the late arrival
+             of FD_CLOSE might be delivered to this socket even after
+	     it has been reallocated. Instead, calculate a timeout to
+             allow for the case when FD_CLOSE never comes, and then
+	     continue as normal
+          */
+          socketIsPendingClose(s);
 	}
 
 	switch(s->request)
@@ -1080,7 +1095,7 @@ nbio_fcntl(nbio_sock_t socket, int op, int arg)
   { if ( op == F_SETFL && arg == O_NONBLOCK )
       s->flags |= PLSOCK_NONBLOCK;
   } else
-    nbio_error(errno, TCP_ERRNO);
+    nbio_error(GET_ERRNO, TCP_ERRNO);
 
   return rc;
 }
@@ -1228,6 +1243,8 @@ allocSocket(SOCKET socket)
   size_t i;
 
 #ifdef __WINDOWS__
+  DWORD now = GetTickCount();
+
   if ( (p=lookupOSSocket(socket)) )
   { DEBUG(1, Sdprintf("WinSock %d already registered on %d\n",
 		      (int)socket, p->id));
@@ -1252,10 +1269,25 @@ allocSocket(SOCKET socket)
   }
 
   for(i=0; i<socks_allocated; i++)
-  { if ( sockets[i] == NULL )
+  { if ( (p=sockets[i]) == NULL )
     { sockets[i] = p = PL_malloc(sizeof(*p));
       socks_count++;
       break;
+#ifdef __WINDOWS__
+    } else
+    { if ( p->close_timeout && p->close_timeout < now )
+      { SOCKET sock;
+
+	if ( (sock=p->socket) )		/* is this ever the case? */
+	{ int rval;
+
+	  do
+	  { rval = closesocket(sock);
+	  } while(rval == SOCKET_ERROR && errno == EINTR);
+	}
+	break;
+      }
+#endif
     }
   }
   UNLOCK();
@@ -1325,13 +1357,13 @@ freeSocket(plsocket *s)
 		 *******************************/
 
 #ifdef __WINDOWS__
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The code in BILLY_GETS_BETTER is, according to various documents the
 right code, but it doesn't work, so we do it by hand.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #ifdef BILLY_GETS_BETTER
-
 static const char *
 WinSockError(unsigned long eno)
 { char buf[1024];
@@ -1413,10 +1445,10 @@ WinSockError(unsigned long error)
     { WSANOTINITIALISED, "Socket layer not initialised" },
 					/* WinSock 2 errors */
     { WSAHOST_NOT_FOUND, "Host not found" },
+    { WSANO_DATA, "Valid name, no data record of requested type" },
     { 0, NULL }
   };
   char tmp[100];
-
   for(ep=edefs; ep->string; ep++)
   { if ( ep->index == (int)error )
       return ep->string;
@@ -1467,7 +1499,6 @@ nbio_error(int code, nbio_error_map mapid)
 
   if ( code == EPLEXCEPTION )
     return FALSE;
-
   switch( mapid )
   { case TCP_HERRNO:
       map = h_errno_codes;
@@ -1475,7 +1506,6 @@ nbio_error(int code, nbio_error_map mapid)
     default:
       map = NULL;
   }
-
   {
 #ifdef __WINDOWS__
   msg = WinSockError(code);
@@ -1586,6 +1616,25 @@ nbio_cleanup(void)
   return 0;
 }
 
+#ifdef __WINDOWS__
+
+/* socketIsPendingClose() is called if a Windows socked is closed, but
+   we did not yet see an FD_CLOSE message for it.  We will reuse this
+   socket using lookupTimedOutSocket() if we have not seen the FD_CLOSE
+   after 30 seconds.  This is a random time.  Are we guaranteed to get
+   an FD_CLOSE?
+*/
+
+static int
+socketIsPendingClose(plsocket *s)
+{ s->close_timeout = GetTickCount() + 30 * 1000;
+  shutdown(s->socket, SD_BOTH);
+  DEBUG(3, Sdprintf("Setting timeout for FD_CLOSE on socket %d...\n", s->id));
+
+  return 0;
+}
+
+#endif /*__WINDOWS__*/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 socket(-Socket)
@@ -1601,7 +1650,7 @@ nbio_socket(int domain, int type, int protocol)
   assert(initialised);
 
   if ( (sock = socket(domain, type , protocol)) < 0)
-  { nbio_error(errno, TCP_ERRNO);
+  { nbio_error(GET_ERRNO, TCP_ERRNO);
     return -1;
   }
   if ( !(s=allocSocket(sock)) )		/* register it */
@@ -1640,14 +1689,12 @@ nbio_closesocket(nbio_sock_t socket)
     }
   } else
   {
+  // We cannot free the socket in Windows since we might subsequently get an FD_CLOSE. Instead set the timeout
 #ifdef __WINDOWS__
-    if ( true(s, PLSOCK_CONNECT) )
-    { if ( s->socket >= 0 )
-	shutdown(s->socket, SD_SEND);
-    }
+  socketIsPendingClose(s);
+#else
+  freeSocket(s);
 #endif
-
-    freeSocket(s);
   }
 
   return 0;
@@ -1673,7 +1720,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 
       if( setsockopt(s->socket, SOL_SOCKET, SO_REUSEADDR,
 		     (const char *)&val, sizeof(val)) == -1 )
-      { nbio_error(h_errno, TCP_HERRNO);
+      { nbio_error(GET_H_ERRNO, TCP_HERRNO);
 	rc = -1;
       } else
 	rc = 0;
@@ -1689,7 +1736,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 #endif
       if ( setsockopt(s->socket, IPPROTO_TCP, TCP_NODELAY,
 		      (const char *)&val, sizeof(val)) == -1 )
-      { nbio_error(h_errno, TCP_HERRNO);
+      { nbio_error(GET_H_ERRNO, TCP_HERRNO);
 	rc = -1;
       } else
       { rc = 0;
@@ -1705,7 +1752,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 
       if ( setsockopt(s->socket, SOL_SOCKET, SO_BROADCAST,
 		     (const char *)&val, sizeof(val)) == -1 )
-      { nbio_error(h_errno, TCP_HERRNO);
+      { nbio_error(GET_H_ERRNO, TCP_HERRNO);
 	rc = -1;
       } else
 	rc = 0;
@@ -1795,11 +1842,13 @@ nbio_get_port(term_t Port, int *port)
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Convert a term Host:Port to a socket address.  Port is either an integer
 or the name of a registered port (e.g. 'smtp').
+
+(*) TBD: Supply the port/service here too, simplifying the rest
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 int
 nbio_get_sockaddr(term_t Address, struct sockaddr_in *addr)
-{ int		  port;
+{ int port;
 
   addr->sin_family = AF_INET;
   addr->sin_addr.s_addr = INADDR_ANY;
@@ -1810,13 +1859,18 @@ nbio_get_sockaddr(term_t Address, struct sockaddr_in *addr)
 
     _PL_get_arg(1, Address, arg);
     if ( PL_get_atom_chars(arg, &hostName) )
-    { struct hostent *host;
+    { struct addrinfo hints;
+      struct addrinfo *res;
 
-      if( !(host = gethostbyname(hostName)) )
-	return nbio_error(h_errno, TCP_HERRNO);
-      if ( (int)sizeof(addr->sin_addr) < host->h_length )
-	return PL_warning("Oops, host address too long!");
-      memcpy(&addr->sin_addr, host->h_addr, host->h_length);
+      memset(&hints, 0, sizeof(hints));
+      hints.ai_family = AF_INET;
+      if ( getaddrinfo(hostName, NULL, &hints, &res) !=  0) /* see (*) */
+	return nbio_error(GET_H_ERRNO, TCP_HERRNO);
+      assert(res->ai_family == AF_INET);
+      memcpy(&addr->sin_addr,
+	     &((struct sockaddr_in*)res->ai_addr)->sin_addr,
+	     sizeof(addr->sin_addr));
+      freeaddrinfo(res);
     } else if ( !nbio_get_ip(arg, &addr->sin_addr) )
     { return pl_error(NULL, 0, NULL, ERR_ARGTYPE, 1, arg, "atom|ip/4");
     }
@@ -1900,7 +1954,8 @@ nbio_bind(nbio_sock_t socket, struct sockaddr *my_addr, size_t addrlen)
 #else
   if ( bind(s->socket, my_addr, addrlen) )
 #endif
-  { nbio_error(errno, TCP_ERRNO);
+  {
+    nbio_error(GET_ERRNO, TCP_ERRNO);
     return -1;
   }
 
@@ -1938,12 +1993,12 @@ nbio_connect(nbio_sock_t socket,
 #else /*!__WINDOWS__*/
   for(;;)
   { if ( connect(s->socket, serv_addr, addrlen) )
-    { if ( need_retry(errno) )
+    { if ( need_retry(GET_ERRNO) )
       { if ( PL_handle_signals() < 0 )
 	  return -1;
 	continue;
       }
-      nbio_error(errno, TCP_ERRNO);
+      nbio_error(GET_ERRNO, TCP_ERRNO);
       return -1;
     } else
       break;
@@ -1999,13 +2054,13 @@ nbio_accept(nbio_sock_t master, struct sockaddr *addr, socklen_t *addrlen)
     slave = accept(m->socket, addr, addrlen);
 
     if ( slave == SOCKET_ERROR )
-    { if ( need_retry(errno) )
+    { if ( need_retry(GET_ERRNO) )
       { if ( PL_handle_signals() < 0 )
 	  return -1;
 
 	continue;
       } else
-      { nbio_error(errno, TCP_ERRNO);
+      { nbio_error(GET_ERRNO, TCP_ERRNO);
 	return -1;
       }
     } else
@@ -2033,7 +2088,7 @@ nbio_listen(nbio_sock_t socket, int backlog)
     return -1;
 
   if( listen(s->socket, backlog) == -1 )
-  { nbio_error(errno, TCP_ERRNO);
+  { nbio_error(GET_ERRNO, TCP_ERRNO);
     return -1;
   }
 
@@ -2095,7 +2150,7 @@ nbio_read(int socket, char *buf, size_t bufSize)
 
     n = recv(s->socket, buf, bufSize, 0);
 
-    if ( n == -1 && need_retry(errno) )
+    if ( n == -1 && need_retry(GET_ERRNO) )
     { if ( PL_handle_signals() < 0 )
       { errno = EPLEXCEPTION;
 	return -1;
@@ -2167,7 +2222,7 @@ nbio_write(nbio_sock_t socket, char *buf, size_t bufSize)
 
     n = send(s->socket, str, len, 0);
     if ( n < 0 )
-    { if ( need_retry(errno) )
+    { if ( need_retry(GET_ERRNO) )
       { if ( PL_handle_signals() < 0 )
 	{ errno = EPLEXCEPTION;
 	  return -1;
@@ -2205,19 +2260,17 @@ nbio_close_input(nbio_sock_t socket)
     if ( (sock=s->socket) < 0 )
     { s->error = WSAECONNRESET;
       rc = -1;
-    } else if ( shutdown(sock, SD_RECEIVE) == SOCKET_ERROR )
-    { DEBUG(1, Sdprintf("shutdown(%d=%d, SD_RECEIVE) failed: %s\n",
-			socket, s->socket,
-			WinSockError(WSAGetLastError())));
-      Sseterr(s->input, SIO_FERR, WinSockError(WSAGetLastError()));
-      rc = -1;
     }
   }
 #endif
 
   s->input = NULL;
   if ( !(s->flags & (PLSOCK_INSTREAM|PLSOCK_OUTSTREAM)) )
+#ifdef __WINDOWS__
+    return socketIsPendingClose(s);
+#else
     return freeSocket(s);
+#endif
 
   return rc;
 }
@@ -2244,14 +2297,6 @@ nbio_close_output(nbio_sock_t socket)
     if ( (sock=s->socket) < 0 )
     { s->error = WSAECONNRESET;
       rc = -1;
-    } else if ( shutdown(sock, SD_SEND) == SOCKET_ERROR )
-    { const char *msg;
-
-      msg = WinSockError(WSAGetLastError());
-      Sseterr(s->output, SIO_FERR, msg);
-      DEBUG(1, Sdprintf("shutdown(%d=%d, SD_SEND) failed: %s\n",
-			socket, s->socket, msg));
-      rc = -1;
     }
 #endif
   }
@@ -2259,7 +2304,11 @@ nbio_close_output(nbio_sock_t socket)
   DEBUG(3, Sdprintf("%d->flags = 0x%x\n", socket, s->flags));
   s->output = NULL;
   if ( !(s->flags & (PLSOCK_INSTREAM|PLSOCK_OUTSTREAM)) )
+#ifdef __WINDOWS__
+    return socketIsPendingClose(s);
+#else
     return freeSocket(s);
+#endif
 
   return rc;
 }
@@ -2322,7 +2371,7 @@ nbio_recvfrom(int socket, void *buf, size_t bufSize, int flags,
 
     n = recvfrom(s->socket, buf, bufSize, flags, from, fromlen);
 
-    if ( n == -1 && need_retry(errno) )
+    if ( n == -1 && need_retry(GET_ERRNO) )
     { if ( PL_handle_signals() < 0 )
       { errno = EPLEXCEPTION;
 	return -1;
