@@ -121,7 +121,7 @@ domain_error(const char *expected, term_t found)
 		 *******************************/
 
 #define	ESC_PATH       (CH_PCHAR|CH_EX_PATH)
-#define	ESC_QUERY      (CH_PCHAR|CH_EX_QF)
+#define	ESC_QUERY      (CH_UNRESERVED|CH_PSUBDELIM|CH_EX_PCHAR|CH_EX_QF)
 #define	ESC_QVALUE     (CH_UNRESERVED|CH_QSUBDELIM|CH_EX_PCHAR|CH_EX_QF)
 #define	ESC_QNAME      (CH_PCHAR)
 #define	ESC_FRAGMENT   (CH_PCHAR|CH_EX_QF)
@@ -142,7 +142,8 @@ domain_error(const char *expected, term_t found)
 #define CH_EX_QF      0x0080		/* Extra query and fragment chars */
 #define CH_EX_SCHEME  0x0100
 #define CH_QSUBDELIM  0x0200
-#define CH_EX_PATH    0x0400
+#define CH_PSUBDELIM  0x0400
+#define CH_EX_PATH    0x0800
 
 #define CH_SCHEME	(CH_ALPHA|CH_DIGIT|CH_EX_SCHEME)
 #define CH_UNRESERVED	(CH_ALPHA|CH_DIGIT|CH_EX_UNRES)
@@ -172,7 +173,8 @@ fill_flags()
     set_flags("-._~",        CH_EX_UNRES);
     set_flags(":/?#[]@",     CH_GENDELIM);
     set_flags("!$&'()+*,;=", CH_SUBDELIM);
-    set_flags("!$'()*,;",    CH_QSUBDELIM); /* = CH_SUBDELIM - "&=+" */
+    set_flags("!$&'()*,;=",  CH_PSUBDELIM); /* = CH_SUBDELIM - "+"  */
+    set_flags("!$'()*,",     CH_QSUBDELIM); /* = CH_SUBDELIM - "&=+" */
     set_flags(":@",          CH_EX_PCHAR);
     set_flags("/",           CH_EX_PATH);
     set_flags("/?",          CH_EX_QF);
@@ -798,11 +800,13 @@ uri_is_global(term_t URI)
     const pl_wchar_t *end = &s[len];
     range r;
 
+    fill_flags();
+
     e = skip_not(s, end, L":/?#");
     if ( e > s && e[0] == ':' )
     { r.start = s;
       r.end = e;
-      if ( range_is_unreserved(&r, TRUE, CH_SCHEME) )
+      if ( range_is_unreserved(&r, FALSE, CH_SCHEME) )
 	return TRUE;
     }
   }
@@ -850,7 +854,7 @@ unify_query_string_components(term_t list, size_t len, const pl_wchar_t *qs)
       name.end   = skip_not(qs, end, L"=");
       if ( name.end < end )
       { value.start = name.end+1;
-	value.end   = skip_not(value.start, end, L"&");
+	value.end   = skip_not(value.start, end, L"&;");
 
 	qs = value.end+1;
       } else
@@ -1153,7 +1157,7 @@ normalize_in_charbuf(charbuf *cb, uri_component_ranges *ranges, int iri)
   }
   if ( ranges->fragment.start )
   { add_charbuf(cb, '#');
-    add_range_charbuf(cb, &ranges->fragment, iri, ESC_FRAGMENT);
+    add_range_charbuf(cb, &ranges->fragment, iri, ESC_QVALUE);
   }
 
   return TRUE;
@@ -1597,3 +1601,9 @@ install_uri()
   PL_register_foreign("uri_encoded",	      3, uri_encoded,	       0);
   PL_register_foreign("uri_iri",	      2, uri_iri,	       0);
 }
+
+
+install_t
+uninstall_uri()
+{ pthread_key_delete(base_key);
+}
