@@ -58,14 +58,19 @@
 					% Emitting the HTML code
 	    print_html/1,		% +List
 	    print_html/2,		% +Stream, +List
-	    html_print_length/2		% +List, -Length
+	    html_print_length/2,	% +List, -Length
+
+					% Extension support
+	    (html_meta)/1,		% +Spec
+	    op(1150, fx, html_meta)
 	  ]).
 :- use_module(library(error)).
+:- use_module(library(apply)).
 :- use_module(library(lists)).
 :- use_module(library(option)).
 :- use_module(library(pairs)).
 :- use_module(library(sgml)).		% Quote output
-:- use_module(library(url)).
+:- use_module(library(uri)).
 :- use_module(library(quintus)).	% for meta_predicate/1
 :- set_prolog_flag(generate_debug_info, false).
 
@@ -577,6 +582,8 @@ name(Name) -->
 %	  Emit URL-encoded version of V.  See www_form_encode/2.
 %	  * An option list
 %	  Emit ?Name1=encode(Value1)&Name2=encode(Value2) ...
+%	  * A term Format-Arguments
+%	  Use format/3 and emit the result as quoted value.
 %
 %	The hook html_write:expand_attribute_value//1 can  be defined to
 %	provide additional `function like'   translations.  For example,
@@ -587,24 +594,36 @@ name(Name) -->
 :- multifile
 	expand_attribute_value//1.
 
-attribute_value(Var) -->
+attribute_value(List) -->
+	{ is_list(List) },
+	attribute_value_m(List).
+attribute_value(Value) -->
+	attribute_value_s(Value).
+
+% emit a single attribute value
+
+attribute_value_s(Var) -->
 	{ var(Var), !,
 	  instantiation_error(Var)
 	}.
-attribute_value(A+B) --> !,
+attribute_value_s(A+B) --> !,
 	attribute_value(A),
-	attribute_value(B).
-attribute_value([]) --> !.
-attribute_value(List) -->
-	{ is_list(List) }, !,
-	[ ? ],
-	search_parameters(List).
-attribute_value(encode(Value)) --> !,
-	{ www_form_encode(Value, Encoded) },
+	(   { is_list(B) }
+	->  (   { B == [] }
+	    ->	[]
+	    ;	[?], search_parameters(B)
+	    )
+	;   attribute_value(B)
+	).
+attribute_value_s(encode(Value)) --> !,
+	{ uri_encoded(query_value, Value, Encoded) },
 	[ Encoded ].
-attribute_value(Value) -->
+attribute_value_s(Value) -->
 	expand_attribute_value(Value), !.
-attribute_value(Value) -->
+attribute_value_s(Fmt-Args) --> !,
+	{ format(string(Value), Fmt, Args) },
+	html_quoted_attribute(Value).
+attribute_value_s(Value) -->
 	html_quoted_attribute(Value).
 
 search_parameters([H|T]) -->
@@ -631,6 +650,26 @@ search_parameter(Term) -->
 	{ domain_error(search_parameter, Term)
 	}.
 
+%%	attribute_value_m(+List)//
+%
+%	Used for multi-valued attributes, such as class-lists.  E.g.,
+%
+%	  ==
+%	  	body(class([c1, c2]), Body)
+%	  ==
+%
+%	  Emits =|<body class="c1 c2"> ...|=
+
+attribute_value_m([]) -->
+	[].
+attribute_value_m([H|T]) -->
+	attribute_value_s(H),
+	(   { T == [] }
+	->  []
+	;   [' '],
+	    attribute_value_m(T)
+	).
+
 
 		 /*******************************
 		 *	   QUOTING RULES	*
@@ -1050,6 +1089,85 @@ reply_html_page(Style, Head, Body) :-
 	print_html(HTML).
 
 
+		 /*******************************
+		 *     META-PREDICATE SUPPORT	*
+		 *******************************/
+
+%%	html_meta(+Heads) is det.
+%
+%	This directive can be used  to   declare  that an HTML rendering
+%	rule takes HTML content as  argument.   It  has  two effects. It
+%	emits  the  appropriate  meta_predicate/1    and  instructs  the
+%	built-in editor (PceEmacs) to provide   proper colouring for the
+%	arguments.
+
+html_meta(Spec) :-
+	throw(error(context_error(nodirective, html_meta(Spec)), _)).
+
+html_meta_decls(Var, _, _) :-
+	var(Var), !,
+	instantiation_error(Var).
+html_meta_decls((A,B), (MA,MB), [MH|T]) :- !,
+	html_meta_decl(A, MA, MH),
+	html_meta_decls(B, MB, T).
+html_meta_decls(A, MA, MH) :-
+	html_meta_decl(A, MA, MH).
+
+html_meta_decl(Head, MetaHead,
+	       html_write:html_meta_head(GenHead, Module, Head)) :-
+	functor(Head, Name, Arity),
+	functor(GenHead, Name, Arity),
+	prolog_load_context(module, Module),
+	Head =.. [Name|HArgs],
+	maplist(html_meta_decl, HArgs, MArgs),
+	MetaHead =.. [Name|MArgs].
+
+html_meta_decl(html, :) :- !.
+html_meta_decl(Meta, Meta).
+
+system:term_expansion((:- html_meta(Heads)),
+		      [ (:- meta_predicate(Meta))
+		      | MetaHeads
+		      ]) :-
+	html_meta_decls(Heads, Meta, MetaHeads).
+
+:- multifile
+	html_meta_head/3.
+
+html_meta_colours(Head, Goal, built_in-Colours) :-
+	Head =.. [_|MArgs],
+	Goal =.. [_|Args],
+	maplist(meta_colours, MArgs, Args, Colours).
+
+meta_colours(html, HTML, Colours) :- !,
+	html_colours(HTML, Colours).
+meta_colours(_, _, classify).
+
+html_meta_called(Head, Goal, Called) :-
+	Head =.. [_|MArgs],
+	Goal =.. [_|Args],
+	meta_called(MArgs, Args, Called, []).
+
+meta_called([], [], Called, Called).
+meta_called([html|MT], [A|AT], Called, Tail) :- !,
+	phrase(called_by(A), Called, Tail1),
+	meta_called(MT, AT, Tail1, Tail).
+meta_called([_|MT], [_|AT], Called, Tail) :- !,
+	meta_called(MT, AT, Called, Tail).
+
+
+:- html_meta
+	html(html,?,?),
+	page(html,?,?),
+	page(html,html,?,?),
+	page(html,html,?,?),
+	pagehead(html,?,?),
+	pagebody(html,?,?),
+	reply_html_page(html,html),
+	reply_html_page(+,html,html),
+	html_post(+,html,?,?).
+
+
 		 /*******************************
 		 *	PCE EMACS SUPPORT	*
 		 *******************************/
@@ -1060,33 +1178,9 @@ reply_html_page(Style, Head, Body) :-
 	emacs_prolog_colours:identify/2,
 	prolog:called_by/2.
 
-emacs_prolog_colours:goal_colours(html(HTML,_,_),
-				  built_in-[Colours, classify, classify]) :-
-	html_colours(HTML, Colours).
-emacs_prolog_colours:goal_colours(page(HTML,_,_),
-				  built_in-[Colours, classify, classify]) :-
-	html_colours(HTML, Colours).
-emacs_prolog_colours:goal_colours(page(Head, Body,_,_),
-				  built_in-[HC, BC, classify, classify]) :-
-	html_colours(Head, HC),
-	html_colours(Body, BC).
-emacs_prolog_colours:goal_colours(pagehead(HTML,_,_),
-				  built_in-[Colours, classify, classify]) :-
-	html_colours(HTML, Colours).
-emacs_prolog_colours:goal_colours(pagebody(HTML,_,_),
-				  built_in-[Colours, classify, classify]) :-
-	html_colours(HTML, Colours).
-emacs_prolog_colours:goal_colours(reply_html_page(Head, Body),
-				  built_in-[HC, BC]) :-
-	html_colours(Head, HC),
-	html_colours(Body, BC).
-emacs_prolog_colours:goal_colours(reply_html_page(_Style, Head, Body),
-				  built_in-[identifier, HC, BC]) :-
-	html_colours(Head, HC),
-	html_colours(Body, BC).
-emacs_prolog_colours:goal_colours(html_post(_Id, HTML, _, _),
-				  built_in-[classify, Colours]) :-
-	html_colours(HTML, Colours).
+emacs_prolog_colours:goal_colours(Goal, Colours) :-
+	html_meta_head(Goal, _Module, Head),
+	html_meta_colours(Head, Goal, Colours).
 
 
 					% TBD: Check with do_expand!
@@ -1212,22 +1306,9 @@ emacs_prolog_colours:identify(http_no_location_for_id(ID), Summary) :-
 %	or callable+N to indicate (DCG) arglist extension.
 
 
-prolog:called_by(html(HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(page(HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(page(Head,Body,_,_), Called) :-
-	phrase(called_by([Head,Body]), Called).
-prolog:called_by(pagehead(HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(pagebody(HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(html_post(_,HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(reply_html_page(Head,Body), Called) :-
-	phrase(called_by([Head,Body]), Called).
-prolog:called_by(reply_html_page(_Style,Head,Body), Called) :-
-	phrase(called_by([Head,Body]), Called).
+prolog:called_by(Goal, Called) :-
+	html_meta_head(Goal, _Module, Head),
+	html_meta_called(Head, Goal, Called).
 
 called_by(Term) -->
 	called_by(Term, _).
@@ -1239,7 +1320,7 @@ called_by(\G, M) --> !,
 	(   { is_list(G) }
 	->  called_by(G, M)
 	;   {atom(M)}
-	->  [M:G+2]
+	->  [(M:G)+2]
 	;   [G+2]
 	).
 called_by([], _) --> !,
