@@ -118,16 +118,38 @@ make_socket(Port, Options0, Options) :-
 	tcp_setopt(Socket, reuseaddr),
 	tcp_bind(Socket, Port),
 	tcp_listen(Socket, 5),
-	atom_concat('httpd@', Port, Queue),
+	make_addr_atom('httpd@', Port, Queue),
 	Options = [ queue(Queue),
 		    tcp_socket(Socket)
 		  | Options0
 		  ].
 
+%%	make_addr_atom(+Prefix, +Address, -Atom) is det.
+%
+%	Create an atom that identifies  the   server's  queue and thread
+%	resources.
+
+make_addr_atom(Prefix, Address, Atom) :-
+	phrase(address_parts(Address), Parts),
+	atomic_list_concat([Prefix|Parts], Atom).
+
+address_parts(Atomic) -->
+	{ atomic(Atomic) }, !,
+	[Atomic].
+address_parts(Host:Port) --> !,
+	address_parts(Host), [:], address_parts(Port).
+address_parts(ip(A,B,C,D)) --> !,
+	[ A, '.', B, '.', C, '.', D ].
+
+%%	create_server(:Goal, +Address, +Options) is det.
+%
+%	Create the main server thread that runs accept_server/2 to
+%	listen to new requests.
+
 create_server(Goal, Port, Options) :-
 	get_time(StartTime),
 	memberchk(queue(Queue), Options),
-	atom_concat('http@', Port, Alias),
+	make_addr_atom('http@', Port, Alias),
 	thread_create(accept_server(Goal, Options), _,
 		      [ alias(Alias)
 		      ]),
@@ -556,8 +578,16 @@ http_spawn(Goal, Options) :-
 				    | ThreadOptions
 				    ]),
 	      Error,
-	      spawn_error(Error, http_spawn(Goal, Options))),
-	http_spawned(Id).
+	      true),
+	(   var(Error)
+	->  http_spawned(Id)
+	;   Error = error(resource_error(threads_in_pool(_)), _)
+	->  throw(http_reply(busy))
+	;   Error = error(existence_error(thread_pool, Pool), _),
+	    create_pool(Pool)
+	->  http_spawn(Goal, Options)
+	;   throw(Error)
+	).
 http_spawn(Goal, Options) :-
 	current_output(CGI),
 	thread_create(wrap_spawned(CGI, Goal), Id,
@@ -571,13 +601,19 @@ wrap_spawned(CGI, Goal) :-
 	http_wrap_spawned(Goal, Request, Connection),
 	next(Connection, Request).
 
-spawn_error(error(resource_error(threads_in_pool(_)), _), _) :-
-	throw(http_reply(busy)).
-spawn_error(error(existence_error(thread_pool, Pool), _), Retry) :-
-	http:create_pool(Pool),
-	call(Retry).
-spawn_error(Error, _) :-
-	throw(Error).
+%%	create_pool(+Pool)
+%
+%	Lazy  creation  of  worker-pools  for   the  HTTP  server.  This
+%	predicate calls the hook http:create_pool/1.   If the hook fails
+%	it creates a default pool of size 10.   This should suffice most
+%	typical usecases.
+
+create_pool(Pool) :-
+	http:create_pool(Pool), !.
+create_pool(Pool) :-
+	print_message(informational, httpd(created_pool(Pool))),
+	thread_pool_create(Pool, 10, []).
+
 
 
 		 /*******************************
@@ -589,3 +625,9 @@ spawn_error(Error, _) :-
 
 prolog:message(httpd_stopped_worker(Self, Status)) -->
 	[ 'Stopped worker ~p: ~p'-[Self, Status] ].
+prolog:message(httpd(created_pool(Pool))) -->
+	[ 'Created thread-pool ~p of size 10'-[Pool], nl,
+	  'Create this pool at startup-time or define the hook ', nl,
+	  'http:create_pool/1 to avoid this message and create a ', nl,
+	  'pool that fits the usage-profile.'
+	].
