@@ -30,7 +30,7 @@
 #endif
 #include "lock.h"
 
-#define RDF_VERSION 20800		/* 2.8.0 */
+#define RDF_VERSION 20900		/* 2.8.0 */
 
 #define URL_subPropertyOf \
 	"http://www.w3.org/2000/01/rdf-schema#subPropertyOf"
@@ -182,14 +182,17 @@ typedef struct triple
   atom_t	graph;			/* where it comes from */
   unsigned long line;			/* graph-line number */
 					/* indexing */
-  struct triple*next[INDEX_TABLES];	/* hash-table next links */
+  union
+  { struct triple*next[INDEX_TABLES];	/* hash-table next links */
+    literal 	end;			/* end for between(X,Y) patterns */
+  } tp;					/* triple or pattern */
 					/* flags */
   unsigned	object_is_literal : 1;	/* Object is a literal */
   unsigned	resolve_pred : 1;	/* predicates needs to be resolved */
   unsigned	indexed : 4;		/* Partials: BY_* */
   unsigned	erased  : 1;		/* If TRUE, triple is erased */
   unsigned	first   : 1;		/* I'm the first on subject */
-  unsigned	match   : 3;		/* How to match literals */
+  unsigned	match   : 4;		/* How to match literals */
   unsigned	inversed : 1;		/* Partials: using inverse match */
   unsigned	is_duplicate : 1;	/* I'm a duplicate */
   unsigned	allocated : 1;		/* Triple is allocated */
@@ -206,7 +209,6 @@ typedef enum
   TR_ASSERT,				/* rdf_assert */
   TR_RETRACT,				/* rdf_retractall */
   TR_UPDATE,				/* rdf_update */
-  TR_UPDATE_SRC,			/* rdf_update */
   TR_UPDATE_MD5,			/* update md5 src */
   TR_RESET,				/* rdf_reset_db */
   TR_VOID				/* no-op */
@@ -252,14 +254,12 @@ typedef struct rdf_db
   size_t	indexed[16];		/* Count calls */
   int		rehash_count;		/* # rehashes */
   int		gc_count;		/* # garbage collections */
-  int		gc_blocked;		/* GC is blocked; */
   double	rehash_time;		/* time spent in rehash */
   double	gc_time;		/* time spent in GC */
   size_t	core;			/* core in use */
   predicate   **pred_table;		/* Hash-table of predicates */
   int		pred_table_size;	/* #entries in the table */
   int		pred_count;		/* #predicates */
-  unsigned long next_hash;		/* cloud hash keys */
   int		active_queries;		/* Calls with choicepoints */
   int		need_update;		/* We need to update */
   size_t	agenda_created;		/* #visited nodes in agenda */
