@@ -35,14 +35,16 @@
 
 :- asserta(user:file_search_path(foreign, '../sgml')).
 :- asserta(user:file_search_path(library, '../sgml')).
+:- asserta(user:file_search_path(library, '../plunit')).
 :- asserta(user:file_search_path(foreign, '../clib')).
 :- asserta(user:file_search_path(library, '../clib')).
-:- asserta(user:file_search_path(library, '../sgml/RDF')).
+:- asserta(user:file_search_path(library, '../RDF')).
 :- asserta(user:file_search_path(foreign, '.')).
 :- use_module(rdf_db).
 :- use_module(rdfs).
 :- use_module(library(xsdp_types)).
 :- use_module(library(lists)).
+:- use_module(library(plunit)).
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 RDF-DB test file.  A test is a clause of the form:
@@ -58,7 +60,10 @@ available test sets. The public goals are:
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 test_rdf_db :-
-	test.
+	test,
+	run_tests([ lang_matches,
+		    lit_ranges
+		  ]).
 
 
 		 /*******************************
@@ -98,7 +103,8 @@ save_reload :-
 	rdf_reset_db,
 	rdf_load(File,
 		 [ base_uri([]),	% do not qualify
-		   convert_typed_literal(convert_typed)
+		   convert_typed_literal(convert_typed),
+		   format(xml)
 		 ]),
 	delete_file(File).
 
@@ -108,7 +114,8 @@ save_reload(Encoding) :-
 	rdf_reset_db,
 	rdf_load(File,
 		 [ base_uri([]),	% do not qualify
-		   convert_typed_literal(convert_typed)
+		   convert_typed_literal(convert_typed),
+		   format(xml)
 		 ]),
 	delete_file(File).
 
@@ -200,6 +207,10 @@ typed(save) :-
 	;   format('TV2 = ~q~n', [TV2]),
 	    fail
 	).
+typed(match) :-
+	rdf_assert(x, a, literal(c)),
+	\+ rdf(x, a, literal(type(t, c))),
+	\+ rdf(x, a, literal(type(t, _))).
 
 
 		 /*******************************
@@ -606,6 +617,11 @@ prefix(like-1) :-
 
 rdf_retractall(nopred-1) :-
 	rdf_retractall(aap, noot, mies).
+rdf_retractall(term) :-
+	rdf_assert(a, b, literal(x)),
+	rdf_assert(a, b, literal(x(1))),
+	rdf_retractall(a, b, literal(x(_))),
+	findall(V, rdf(a,b,V), [literal(x)]).
 
 
 		 /*******************************
@@ -712,24 +728,35 @@ graph(a, p, b).
 graph(b, p, c).
 graph(c, p, d).
 graph(b, p, d).
+graph(e, p, d).
 
-graph :-
+graph(Symmetric) :-
+	rdf_set_predicate(p, symmetric(Symmetric)),
 	forall(graph(S,P,O),
 	       rdf_assert(S,P,O)).
 
 reachable(1) :-
 	rdf_reachable(a, x, a).
 reachable(2) :-
-	graph,
+	graph(false),
 	rdf_reachable(a, p, d).
 reachable(3) :-
-	graph,
+	graph(false),
 	rdf_reachable(a, p, X),
 	X == c, !.
 reachable(4) :-
-	graph,
+	graph(false),
 	findall(O, rdf_reachable(a, p, O), Os),
 	Os = [a,b,c,d].
+reachable(5) :-
+	graph(false),
+	\+ rdf_reachable(d, p, a).
+reachable(6) :-
+	graph(true),
+	rdf_reachable(d, p, a).
+reachable(6) :-
+	graph(true),
+	rdf_reachable(e, p, a).
 
 
 		 /*******************************
@@ -947,3 +974,64 @@ test_failed(R, Except) :-
 blocked(Reason) :-
 	throw(blocked(Reason)).
 
+
+		 /*******************************
+		 *	      UNIT TESTS	*
+		 *******************************/
+
+:- begin_tests(lang_matches).
+
+test(lang_matches, true) :-
+	lang_matches('EN', en).
+test(lang_matches, true) :-
+	lang_matches(en, 'EN').
+test(lang_matches, fail) :-
+	lang_matches(nl, 'EN').
+test(lang_matches, true) :-
+	lang_matches('en-GB', en).
+test(lang_matches, fail) :-
+	lang_matches('en-GB', 'en-*-x').
+test(lang_matches, true) :-
+	lang_matches('en-GB-x', 'en-*-x').
+test(lang_matches, true) :-
+	lang_matches('en-GB-x-y', 'en-*-x-*').
+test(lang_matches, true) :-
+	lang_matches('en-GB-x-y', 'en-*-y').
+
+:- end_tests(lang_matches).
+
+:- begin_tests(lit_ranges, [cleanup(rdf_reset_db)]).
+
+letters :-
+	rdf_reset_db,
+	forall(between(0'a, 0'z, X),
+	       (   char_code(C, X),
+		   rdf_assert(a,b,literal(C))
+	       )).
+
+integers :-
+	rdf_reset_db,
+	forall(between(0, 9, X),
+	       rdf_assert(a,b,literal(X))).
+
+ge(S, X) :-
+	rdf(_,b,literal(ge(S),X)).
+le(S, X) :-
+	rdf(_,b,literal(le(S),X)).
+bt(L,H,X) :-
+	rdf(_,b,literal(between(L,H),X)).
+
+test(ge, [setup(letters), cleanup(rdf_reset_db), all(X==[x,y,z])]) :-
+	ge(x, X).
+test(le, [setup(letters), cleanup(rdf_reset_db), all(X==[a,b,c,d,e])]) :-
+	le(e, X).
+test(bt, [setup(letters), cleanup(rdf_reset_db), all(X==[m,n,o,p])]) :-
+	bt(m, p, X).
+test(ge, [setup(integers), cleanup(rdf_reset_db), all(X==[4,5,6,7,8,9])]) :-
+	ge(4, X).
+test(le, [setup(integers), cleanup(rdf_reset_db), all(X==[0,1,2,3])]) :-
+	le(3, X).
+test(bt, [setup(integers), cleanup(rdf_reset_db), all(X==[6,7,8])]) :-
+	bt(6,8, X).
+
+:- end_tests(lit_ranges).
