@@ -87,7 +87,7 @@ typedef struct _parser_data
   int	      max_warnings;		/* warning limit */
   errormode   error_mode;		/* how to handle errors */
   int	      positions;		/* report file-positions */
-  int         exception;		/* pending exception from callback */
+  term_t      exception;		/* pending exception from callback */
 
   predicate_t on_begin;			/* begin element */
   predicate_t on_end;			/* end element */
@@ -256,7 +256,7 @@ get_parser(term_t parser, dtd_parser **p)
   { term_t a = PL_new_term_ref();
     void *ptr;
 
-    PL_get_arg(1, parser, a);
+    _PL_get_arg(1, parser, a);
     if ( PL_get_pointer(a, &ptr) )
     { dtd_parser *tmp = ptr;
 
@@ -292,7 +292,7 @@ get_dtd(term_t t, dtd **dtdp)
   { term_t a = PL_new_term_ref();
     void *ptr;
 
-    PL_get_arg(1, t, a);
+    _PL_get_arg(1, t, a);
     if ( PL_get_pointer(a, &ptr) )
     { dtd *tmp = ptr;
 
@@ -324,7 +324,7 @@ pl_new_sgml_parser(term_t ref, term_t options)
 
   while ( PL_get_list(tail, head, tail) )
   { if ( PL_is_functor(head, FUNCTOR_dtd1) )
-    { PL_get_arg(1, head, tmp);
+    { _PL_get_arg(1, head, tmp);
 
       if ( PL_is_variable(tmp) )	/* dtd(X) */
       { dtd = new_dtd(NULL);		/* no known doctype */
@@ -389,10 +389,10 @@ pl_free_dtd(term_t t)
 		 *	   DATA EXCHANGE	*
 		 *******************************/
 
-static void
+static int
 put_atom_wchars(term_t t, wchar_t const *s)
 { PL_put_variable(t);
-  PL_unify_wchars(t, PL_ATOM, ENDSNUL, s);
+  return PL_unify_wchars(t, PL_ATOM, ENDSNUL, s);
 }
 
 
@@ -412,7 +412,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
     wchar_t *file;
     dtd_symbol *fs;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_wchars(a, NULL, &file, CVT_ATOM|CVT_EXCEPTION) )
       return FALSE;
     fs = dtd_add_symbol(p->dtd, file);	/* symbol will be freed */
@@ -420,20 +420,20 @@ pl_set_sgml_parser(term_t parser, term_t option)
   } else if ( PL_is_functor(option, FUNCTOR_line1) )
   { term_t a = PL_new_term_ref();
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_integer(a, &p->location.line) )
       return sgml2pl_error(ERR_TYPE, "integer", a);
   } else if ( PL_is_functor(option, FUNCTOR_charpos1) )
   { term_t a = PL_new_term_ref();
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_long(a, &p->location.charpos) )
       return sgml2pl_error(ERR_TYPE, "integer", a);
   } else if ( PL_is_functor(option, FUNCTOR_dialect1) )
   { term_t a = PL_new_term_ref();
     char *s;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_atom_chars(a, &s) )
       return sgml2pl_error(ERR_TYPE, "atom", a);
 
@@ -449,7 +449,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
   { term_t a = PL_new_term_ref();
     char *s;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_atom_chars(a, &s) )
       return sgml2pl_error(ERR_TYPE, "atom", a);
 
@@ -468,7 +468,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
   { term_t a = PL_new_term_ref();
     int val;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_bool(a, &val) )
       return sgml2pl_error(ERR_TYPE, "boolean", a);
 
@@ -480,7 +480,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
   { term_t a = PL_new_term_ref();
     int val;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_bool(a, &val) )
       return sgml2pl_error(ERR_TYPE, "boolean", a);
 
@@ -492,7 +492,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
   { term_t a = PL_new_term_ref();
     int val;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_bool(a, &val) )
       return sgml2pl_error(ERR_TYPE, "boolean", a);
 
@@ -501,7 +501,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
   { term_t a = PL_new_term_ref();
     char *s;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_atom_chars(a, &s) )
       return sgml2pl_error(ERR_TYPE, "atom", a);
 
@@ -515,7 +515,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
   { term_t a = PL_new_term_ref();
     char *val;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_atom_chars(a, &val) )
       return sgml2pl_error(ERR_TYPE, "atom", a);
     if ( !xml_set_encoding(p, val) )
@@ -524,7 +524,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
   { term_t a = PL_new_term_ref();
     ichar *s;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( PL_is_variable(a) )
     { p->enforce_outer_element = NULL;
     } else
@@ -538,7 +538,7 @@ pl_set_sgml_parser(term_t parser, term_t option)
     ichar ns[1] = {0};
     ichar *uri;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_wchars(a, NULL, &uri, CVT_ATOM|CVT_EXCEPTION) )
       return FALSE;
 
@@ -547,10 +547,10 @@ pl_set_sgml_parser(term_t parser, term_t option)
   { term_t a = PL_new_term_ref();
     ichar *ns, *uri;
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( !PL_get_wchars(a, NULL, &ns, CVT_ATOM|CVT_EXCEPTION) )
       return FALSE;
-    PL_get_arg(2, option, a);
+    _PL_get_arg(2, option, a);
     if ( !PL_get_wchars(a, NULL, &uri, CVT_ATOM|CVT_EXCEPTION) )
       return FALSE;
 
@@ -581,12 +581,12 @@ pl_get_sgml_parser(term_t parser, term_t option)
   if ( PL_is_functor(option, FUNCTOR_charpos1) )
   { term_t a = PL_new_term_ref();
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     return PL_unify_integer(a, file_location(p, &p->startloc)->charpos);
   } else if ( PL_is_functor(option, FUNCTOR_line1) )
   { term_t a = PL_new_term_ref();
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     return PL_unify_integer(a, file_location(p, &p->startloc)->line);
   } else if ( PL_is_functor(option, FUNCTOR_charpos2) )
   { term_t a = PL_new_term_ref();
@@ -604,7 +604,7 @@ pl_get_sgml_parser(term_t parser, term_t option)
     if ( l->type == IN_FILE && l->name.file )
     { term_t a = PL_new_term_ref();
 
-      PL_get_arg(1, option, a);
+      _PL_get_arg(1, option, a);
       return PL_unify_wchars(a, PL_ATOM, ENDSNUL, l->name.file);
     }
   } else if ( PL_is_functor(option, FUNCTOR_source1) )
@@ -613,13 +613,13 @@ pl_get_sgml_parser(term_t parser, term_t option)
     if ( pd && pd->magic == PD_MAGIC && pd->source )
     { term_t a = PL_new_term_ref();
 
-      PL_get_arg(1, option, a);
+      _PL_get_arg(1, option, a);
       return PL_unify_stream(a, pd->source);
     }
   } else if ( PL_is_functor(option, FUNCTOR_dialect1) )
   { term_t a = PL_new_term_ref();
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     switch(p->dtd->dialect)
     { case DL_SGML:
 	return PL_unify_atom_chars(a, "sgml");
@@ -631,7 +631,7 @@ pl_get_sgml_parser(term_t parser, term_t option)
   } else if ( PL_is_functor(option, FUNCTOR_event_class1) )
   { term_t a = PL_new_term_ref();
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     switch(p->event_class)
     { case EV_EXPLICIT:
 	return PL_unify_atom_chars(a, "explicit");
@@ -645,25 +645,28 @@ pl_get_sgml_parser(term_t parser, term_t option)
   } else if ( PL_is_functor(option, FUNCTOR_dtd1) )
   { term_t a = PL_new_term_ref();
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
 
     return unify_dtd(a, p->dtd);
   } else if ( PL_is_functor(option, FUNCTOR_doctype1) )
   { term_t a = PL_new_term_ref();
 
-    PL_get_arg(1, option, a);
+    _PL_get_arg(1, option, a);
     if ( p->enforce_outer_element )
       return PL_unify_wchars(a, PL_ATOM, ENDSNUL,
 			     p->enforce_outer_element->name);
     else
       return TRUE;			/* leave variable */
   } else if ( PL_is_functor(option, FUNCTOR_allowed1) )
-  { term_t tail = PL_new_term_ref();
-    term_t head = PL_new_term_ref();
-    term_t tmp = PL_new_term_ref();
+  { term_t tail, head, tmp;
     sgml_environment *env = p->environments;
 
-    PL_get_arg(1, option, tail);
+    if ( !(tail = PL_new_term_ref()) ||
+	 !(head = PL_new_term_ref()) ||
+	 !(tmp = PL_new_term_ref()) )
+      return FALSE;
+
+    _PL_get_arg(1, option, tail);
 
     if ( env )
     { for( ; env; env = env->parent)
@@ -674,12 +677,15 @@ pl_get_sgml_parser(term_t parser, term_t option)
 	state_allows_for(env->state, buf, &n);
 
 	for(i=0; i<n; i++)
-	{ if ( buf[i] == CDATA_ELEMENT )
-	    PL_put_atom_chars(tmp, "#pcdata");
-	  else
-	    put_atom_wchars(tmp, buf[i]->name->name);
+	{ int rc;
 
-	  if ( !PL_unify_list(tail, head, tail) ||
+	  if ( buf[i] == CDATA_ELEMENT )
+	    rc = PL_put_atom_chars(tmp, "#pcdata");
+	  else
+	    rc = put_atom_wchars(tmp, buf[i]->name->name);
+
+	  if ( !rc ||
+	       !PL_unify_list(tail, head, tail) ||
 	       !PL_unify(head, tmp) )
 	    return FALSE;
 	}
@@ -703,7 +709,7 @@ pl_get_sgml_parser(term_t parser, term_t option)
     term_t tmp = PL_new_term_ref();
     sgml_environment *env = p->environments;
 
-    PL_get_arg(1, option, tail);
+    _PL_get_arg(1, option, tail);
 
     for( ; env; env = env->parent)
     { put_atom_wchars(tmp, env->element->name->name);
@@ -726,19 +732,21 @@ call_prolog(parser_data *pd, predicate_t pred, term_t av)
 { qid_t qid = PL_open_query(NULL, PL_Q_PASS_EXCEPTION, pred, av);
   int rc = PL_next_solution(qid);
 
-  if ( !rc && PL_exception(qid) )
-    pd->exception = TRUE;
-  else
-    pd->exception = FALSE;
-
   PL_close_query(qid);
 
+  if ( rc )
+  { pd->exception = FALSE;
+  } else
+  { if ( (pd->exception = PL_exception(0)) )
+      pd->stopped = TRUE;
+  }
+
   return rc;
 }
 
 
 static void
-end_frame(fid_t fid, int ex)
+end_frame(fid_t fid, term_t ex)
 { if ( ex )
     PL_close_foreign_frame(fid);
   else
@@ -782,24 +790,21 @@ reset_url_cache()
 }
 
 
-static void
+WUNUSED static int
 put_url(dtd_parser *p, term_t t, const ichar *url)
 { parser_data *pd = p->closure;
+  fid_t fid;
   int i;
 
   if ( !pd->on_urlns )
-  { put_atom_wchars(t, url);
-    return;
-  }
+    return put_atom_wchars(t, url);
 
   for(i=0; i<URL_CACHE; i++)
   { if ( cache[i].url == url )		/* cache hit */
     { if ( cache[i].canonical )		/* and a canonical value */
-	PL_put_atom(t, cache[i].canonical);
+	return PL_put_atom(t, cache[i].canonical);
       else
-	put_atom_wchars(t, url);
-
-      return;
+	return put_atom_wchars(t, url);
     }
   }
 					/* shift the cache */
@@ -811,26 +816,33 @@ put_url(dtd_parser *p, term_t t, const ichar *url)
   cache[0].url = url;
   cache[0].canonical = 0;
 
-  { fid_t fid = PL_open_foreign_frame();
+  if ( (fid = PL_open_foreign_frame()) )
+  { int rc;
     term_t av = PL_new_term_refs(3);
     atom_t a;
 
-    put_atom_wchars(av+0, url);
-    unify_parser(av+2, p);
-    if ( PL_call_predicate(NULL, PL_Q_NORMAL, pd->on_urlns, av) &&
+    rc = (put_atom_wchars(av+0, url) &&
+	  unify_parser(av+2, p));
+
+    if ( rc &&
+	 PL_call_predicate(NULL, PL_Q_NORMAL, pd->on_urlns, av) &&
 	 PL_get_atom(av+1, &a) )
     { PL_register_atom(a);
       cache[0].canonical = a;
       PL_put_atom(t, a);
-    } else
-    { put_atom_wchars(t, url);
+    } else if ( rc )
+    { rc = put_atom_wchars(t, url);
     }
     PL_discard_foreign_frame(fid);
+
+    return rc;
   }
+
+  return FALSE;
 }
 
 
-static void
+WUNUSED static int
 put_attribute_name(dtd_parser *p, term_t t, dtd_symbol *nm)
 { const ichar *url, *local;
 
@@ -838,19 +850,20 @@ put_attribute_name(dtd_parser *p, term_t t, dtd_symbol *nm)
   { xmlns_resolve_attribute(p, nm, &local, &url);
 
     if ( url )
-    { term_t av = PL_new_term_refs(2);
+    { term_t av;
 
-      put_url(p, av+0, url);
-      put_atom_wchars(av+1, local);
-      PL_cons_functor_v(t, FUNCTOR_ns2, av);
+      return ( (av=PL_new_term_refs(2)) &&
+	       put_url(p, av+0, url) &&
+	       put_atom_wchars(av+1, local) &&
+	       PL_cons_functor_v(t, FUNCTOR_ns2, av) );
     } else
-      put_atom_wchars(t, local);
+      return put_atom_wchars(t, local);
   } else
-    put_atom_wchars(t, nm->name);
+    return put_atom_wchars(t, nm->name);
 }
 
 
-static void
+WUNUSED static int
 put_element_name(dtd_parser *p, term_t t, dtd_element *e)
 { const ichar *url, *local;
 
@@ -859,15 +872,16 @@ put_element_name(dtd_parser *p, term_t t, dtd_element *e)
     xmlns_resolve_element(p, &local, &url);
 
     if ( url )
-    { term_t av = PL_new_term_refs(2);
+    { term_t av;
 
-      put_url(p, av+0, url);
-      put_atom_wchars(av+1, local);
-      PL_cons_functor_v(t, FUNCTOR_ns2, av);
+      return ( (av=PL_new_term_refs(2)) &&
+	       put_url(p, av+0, url) &&
+	       put_atom_wchars(av+1, local) &&
+	       PL_cons_functor_v(t, FUNCTOR_ns2, av) );
     } else
-      put_atom_wchars(t, local);
+      return put_atom_wchars(t, local);
   } else
-    put_atom_wchars(t, e->name->name);
+    return put_atom_wchars(t, e->name->name);
 }
 
 
@@ -909,44 +923,47 @@ static int
 put_att_text(term_t t, sgml_attribute *a)
 { if ( a->value.textW )
   { PL_put_variable(t);
-    PL_unify_wchars(t, PL_ATOM, a->value.number, a->value.textW);
-    return TRUE;
+    return PL_unify_wchars(t, PL_ATOM, a->value.number, a->value.textW);
   } else
     return FALSE;
 }
 
 
-static void
+static int
 put_attribute_value(dtd_parser *p, term_t t, sgml_attribute *a)
 { switch(a->definition->type)
   { case AT_CDATA:
-      put_att_text(t, a);
-      break;
+      return put_att_text(t, a);
     case AT_NUMBER:
     { if ( !put_att_text(t, a) )
-	PL_put_integer(t, a->value.number);
-      break;
+	return PL_put_integer(t, a->value.number);
+      return TRUE;
     }
     default:				/* multi-valued attribute */
     { if ( a->definition->islist && a->value.textW )
-      { term_t tail, head = PL_new_term_ref();
+      { term_t tail, head;
 	const ichar *val = a->value.textW;
 	const ichar *e;
 
 	PL_put_variable(t);
-	tail = PL_copy_term_ref(t);
+	if ( !(head = PL_new_term_ref()) ||
+	     !(tail = PL_copy_term_ref(t)) )
+	  return FALSE;
 
 	for(e=istrblank(val); e; val = e+1, e=istrblank(val))
 	{ if ( e == val )
 	    continue;			/* skip spaces */
-	  PL_unify_list(tail, head, tail);
-	  unify_listval(p, head, a->definition->type, e-val, val);
+	  if ( !PL_unify_list(tail, head, tail) ||
+	       !unify_listval(p, head, a->definition->type, e-val, val) )
+	    return FALSE;
 	}
-        PL_unify_list(tail, head, tail);
-	unify_listval(p, head, a->definition->type, istrlen(val), val);
-        PL_unify_nil(tail);
+
+	return ( PL_unify_list(tail, head, tail) &&
+		 unify_listval(p, head, a->definition->type,
+			       istrlen(val), val) &&
+		 PL_unify_nil(tail) );
       } else
-	put_att_text(t, a);
+	return put_att_text(t, a);
     }
   }
 }
@@ -964,13 +981,12 @@ put_tag_position(dtd_parser *p, term_t pos)
 
   if ( l->type == IN_FILE && l->name.file )
   { PL_put_variable(pos);
-    PL_unify_term(pos,
-		  PL_FUNCTOR, FUNCTOR_ns2,
-		    PL_NWCHARS, wcslen(l->name.file), l->name.file,
-		    PL_FUNCTOR, FUNCTOR_minus2,
-		      PL_LONG, l->charpos,
-		      PL_LONG, p->location.charpos);
-    return TRUE;
+    return PL_unify_term(pos,
+			 PL_FUNCTOR, FUNCTOR_ns2,
+			   PL_NWCHARS, wcslen(l->name.file), l->name.file,
+			   PL_FUNCTOR, FUNCTOR_minus2,
+			 PL_LONG, l->charpos,
+			 PL_LONG, p->location.charpos);
   }
 
   return FALSE;
@@ -988,10 +1004,10 @@ unify_attribute_list(dtd_parser *p, term_t alist,
   parser_data *pd = p->closure;
 
   for(i=0; i<argc; i++)
-  { put_attribute_name(p, a+0, argv[i].definition->name);
-    put_attribute_value(p, a+1, &argv[i]);
-    PL_cons_functor_v(a, FUNCTOR_equal2, a);
-    if ( !PL_unify_list(tail, h, tail) ||
+  { if ( !put_attribute_name(p, a+0, argv[i].definition->name) ||
+	 !put_attribute_value(p, a+1, &argv[i]) ||
+	 !PL_cons_functor_v(a, FUNCTOR_equal2, a) ||
+	 !PL_unify_list(tail, h, tail) ||
 	 !PL_unify(h, a) )
       return FALSE;
   }
@@ -999,8 +1015,8 @@ unify_attribute_list(dtd_parser *p, term_t alist,
   if ( pd->positions && put_tag_position(p, a+1) )
   { PL_put_atom(a, ATOM_position);
 
-    PL_cons_functor_v(a, FUNCTOR_equal2, a);
-    if ( !PL_unify_list(tail, h, tail) ||
+    if ( !PL_cons_functor_v(a, FUNCTOR_equal2, a) ||
+	 !PL_unify_list(tail, h, tail) ||
 	 !PL_unify(h, a) )
       return FALSE;
   }
@@ -1029,12 +1045,18 @@ on_begin(dtd_parser *p, dtd_element *e, int argc, sgml_attribute *argv)
     term_t et	   = PL_new_term_ref();	/* element structure */
     term_t h       = PL_new_term_ref();
 
-    put_element_name(p, h, e);
-    unify_attribute_list(p, alist, argc, argv);
-    PL_unify_term(et, PL_FUNCTOR, FUNCTOR_element3,
-		    PL_TERM, h,
-		    PL_TERM, alist,
-		    PL_TERM, content);
+    if ( !h ||
+	 !put_element_name(p, h, e) ||
+	 !unify_attribute_list(p, alist, argc, argv) ||
+	 !PL_unify_term(et,
+			PL_FUNCTOR, FUNCTOR_element3,
+			  PL_TERM, h,
+			  PL_TERM, alist,
+			  PL_TERM, content) )
+    { pd->exception = PL_exception(0);
+      return FALSE;
+    }
+
     if ( PL_unify_list(pd->tail, h, pd->tail) &&
 	 PL_unify(h, et) )
     { env *env = sgml_calloc(1, sizeof(*env));
@@ -1045,21 +1067,34 @@ on_begin(dtd_parser *p, dtd_element *e, int argc, sgml_attribute *argv)
 
       pd->tail = content;
       PL_reset_term_refs(alist);
+
+      return TRUE;
     }
 
-    return TRUE;
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   if ( pd->on_begin )
-  { fid_t fid = PL_open_foreign_frame();
-    term_t av = PL_new_term_refs(3);
+  { fid_t fid;
 
-    put_element_name(p, av+0, e);
-    unify_attribute_list(p, av+1, argc, argv);
-    unify_parser(av+2, p);
+    if ( (fid = PL_open_foreign_frame()) )
+    { int rc;
+      term_t av = PL_new_term_refs(3);
 
-    call_prolog(pd, pd->on_begin, av);
-    end_frame(fid, pd->exception);
+      rc = ( put_element_name(p, av+0, e) &&
+	     unify_attribute_list(p, av+1, argc, argv) &&
+	     unify_parser(av+2, p) &&
+	     call_prolog(pd, pd->on_begin, av)
+	   );
+
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
+    }
+
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   return TRUE;
@@ -1074,18 +1109,29 @@ on_end(dtd_parser *p, dtd_element *e)
     return TRUE;
 
   if ( pd->on_end )
-  { fid_t fid = PL_open_foreign_frame();
-    term_t av = PL_new_term_refs(2);
+  { fid_t fid;
 
-    put_element_name(p, av+0, e);
-    unify_parser(av+1, p);
+    if ( (fid = PL_open_foreign_frame()) )
+    { int rc;
+      term_t av = PL_new_term_refs(2);
 
-    call_prolog(pd, pd->on_end, av);
-    end_frame(fid, pd->exception);
+      rc = ( put_element_name(p, av+0, e) &&
+	     unify_parser(av+1, p) &&
+	     call_prolog(pd, pd->on_end, av)
+	   );
+
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
+    }
+
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   if ( pd->tail && !pd->stopped )
-  { PL_unify_nil(pd->tail);
+  { if ( !PL_unify_nil(pd->tail) )
+      return FALSE;
     PL_reset_term_refs(pd->tail);	/* ? */
 
     if ( pd->stack )
@@ -1115,36 +1161,55 @@ on_entity(dtd_parser *p, dtd_entity *e, int chr)
     return TRUE;
 
   if ( pd->on_entity )
-  { fid_t fid = PL_open_foreign_frame();
-    term_t av = PL_new_term_refs(2);
+  { fid_t fid;
 
-    if ( e )
-      put_atom_wchars(av+0, e->name->name);
-    else
-      PL_put_integer(av+0, chr);
+    if ( (fid=PL_open_foreign_frame()) )
+    { int rc;
+      term_t av = PL_new_term_refs(2);
 
-    unify_parser(av+1, p);
+      if ( e )
+	rc = put_atom_wchars(av+0, e->name->name);
+      else
+	rc = PL_put_integer(av+0, chr);
 
-    call_prolog(pd, pd->on_end, av);
-    end_frame(fid, pd->exception);
+      if ( rc )
+	rc = ( unify_parser(av+1, p) &&
+	       call_prolog(pd, pd->on_end, av)
+	     );
+
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
+    }
+
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   if ( pd->tail )
-  { term_t h = PL_new_term_ref();
+  { int rc;
+    term_t h = PL_new_term_ref();
 
-    if ( !PL_unify_list(pd->tail, h, pd->tail) )
+    if ( !h ||
+	 !PL_unify_list(pd->tail, h, pd->tail) )
+    { pd->exception = PL_exception(0);
       return FALSE;
+    }
 
     if ( e )
-      PL_unify_term(h,
-		    PL_FUNCTOR, FUNCTOR_entity1,
-		    PL_CHARS, e->name->name);
+      rc = PL_unify_term(h,
+			 PL_FUNCTOR, FUNCTOR_entity1,
+			   PL_CHARS, e->name->name);
     else
-      PL_unify_term(h,
-		    PL_FUNCTOR, FUNCTOR_entity1,
-		    PL_INT, chr);
+      rc = PL_unify_term(h,
+			 PL_FUNCTOR, FUNCTOR_entity1,
+			   PL_INT, chr);
 
     PL_reset_term_refs(h);
+    if ( !rc )
+      pd->exception = PL_exception(0);
+
+    return rc;
   }
 
   return TRUE;
@@ -1156,15 +1221,23 @@ on_data(dtd_parser *p, data_type type, int len, const wchar_t *data)
 { parser_data *pd = p->closure;
 
   if ( pd->on_cdata )
-  { fid_t fid = PL_open_foreign_frame();
-    term_t av = PL_new_term_refs(2);
+  { fid_t fid;
 
-    PL_unify_wchars(av+0, PL_ATOM, len, data);
+    if ( (fid=PL_open_foreign_frame()) )
+    { int rc;
+      term_t av = PL_new_term_refs(2);
 
-    unify_parser(av+1, p);
+      rc = ( PL_unify_wchars(av+0, PL_ATOM, len, data) &&
+	     unify_parser(av+1, p) &&
+	     call_prolog(pd, pd->on_cdata, av) );
 
-    call_prolog(pd, pd->on_cdata, av);
-    end_frame(fid, pd->exception);
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
+    }
+
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   if ( pd->tail && !pd->stopped )
@@ -1203,6 +1276,8 @@ on_data(dtd_parser *p, data_type type, int len, const wchar_t *data)
       if ( rval )
       { PL_reset_term_refs(h);
 	return TRUE;
+      } else
+      { pd->exception = PL_exception(0);
       }
     }
   }
@@ -1267,46 +1342,69 @@ on_error(dtd_parser *p, dtd_error *error)
   }
 
   if ( pd->on_error )			/* msg, parser */
-  { fid_t fid = PL_open_foreign_frame();
-    term_t av = PL_new_term_refs(3);
+  { fid_t fid;
 
-    PL_put_atom_chars(av+0, severity);
-    PL_unify_wchars(av+1, PL_ATOM,
-		    wcslen(error->plain_message), error->plain_message);
-    unify_parser(av+2, p);
+    if ( (fid=PL_open_foreign_frame()) )
+    { int rc;
+      term_t av = PL_new_term_refs(3);
 
-    call_prolog(pd, pd->on_error, av);
-    end_frame(fid, pd->exception);
+      rc = ( PL_put_atom_chars(av+0, severity) &&
+	     PL_unify_wchars(av+1, PL_ATOM,
+			     wcslen(error->plain_message),
+			     error->plain_message) &&
+	     unify_parser(av+2, p) &&
+	     call_prolog(pd, pd->on_error, av)
+	   );
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
+    }
+    pd->exception = PL_exception(0);
+    return FALSE;
   } else if ( pd->error_mode != EM_QUIET )
-  { fid_t fid = PL_open_foreign_frame();
-    predicate_t pred = PL_predicate("print_message", 2, "user");
-    term_t av = PL_new_term_refs(2);
-    term_t src = PL_new_term_ref();
-    term_t parser = PL_new_term_ref();
-    dtd_srcloc *l = file_location(p, &p->startloc);
+  { fid_t fid;
 
-    unify_parser(parser, p);
-    PL_put_atom_chars(av+0, severity);
+    if ( (fid=PL_open_foreign_frame()) )
+    { int rc;
+      predicate_t pred = PL_predicate("print_message", 2, "user");
+      term_t av = PL_new_term_refs(2);
+      term_t src = PL_new_term_ref();
+      term_t parser = PL_new_term_ref();
+      dtd_srcloc *l = file_location(p, &p->startloc);
 
-    if ( l->name.file )
-    { if ( l->type == IN_FILE )
-	put_atom_wchars(src, l->name.file);
-      else
-	put_atom_wchars(src, l->name.entity);
-    } else
-    { PL_put_nil(src);
+      rc = ( unify_parser(parser, p) &&
+	     PL_put_atom_chars(av+0, severity) );
+
+      if ( rc )
+      { if ( l->name.file )
+	{ if ( l->type == IN_FILE )
+	    rc = put_atom_wchars(src, l->name.file);
+	  else
+	    rc = put_atom_wchars(src, l->name.entity);
+	} else
+	{ PL_put_nil(src);
+	}
+      }
+
+      if ( rc )
+	rc = PL_unify_term(av+1,
+			   PL_FUNCTOR_CHARS, "sgml", 4,
+			     PL_TERM, parser,
+			     PL_TERM, src,
+			     PL_INT, l->line,
+			     PL_NWCHARS, wcslen(error->plain_message),
+					 error->plain_message);
+
+      if ( rc )
+	rc = PL_call_predicate(NULL, PL_Q_NODEBUG, pred, av);
+
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
     }
 
-    PL_unify_term(av+1,
-		  PL_FUNCTOR_CHARS, "sgml", 4,
-		    PL_TERM, parser,
-		    PL_TERM, src,
-		    PL_INT, l->line,
-		    PL_NWCHARS, wcslen(error->plain_message), error->plain_message);
-
-    PL_call_predicate(NULL, PL_Q_NODEBUG, pred, av);
-
-    PL_discard_foreign_frame(fid);
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   return TRUE;
@@ -1321,18 +1419,34 @@ on_xmlns(dtd_parser *p, dtd_symbol *ns, dtd_symbol *url)
     return TRUE;
 
   if ( pd->on_xmlns )
-  { fid_t fid = PL_open_foreign_frame();
-    term_t av = PL_new_term_refs(3);
+  { fid_t fid;
+    term_t av;
 
-    if ( ns )
-      put_atom_wchars(av+0, ns->name);
-    else
-      PL_put_nil(av+0);
-    put_atom_wchars(av+1, url->name);
-    unify_parser(av+2, p);
+    if ( (fid = PL_open_foreign_frame()) &&
+	 (av = PL_new_term_refs(3)) )
+    { int rc;
 
-    call_prolog(pd, pd->on_xmlns, av);
-    end_frame(fid, pd->exception);
+      if ( ns )
+      { rc = put_atom_wchars(av+0, ns->name);
+      } else
+      { PL_put_nil(av+0);
+	rc = TRUE;
+      }
+
+      if ( rc )
+      { rc = ( put_atom_wchars(av+1, url->name) &&
+	       unify_parser(av+2, p) &&
+	       call_prolog(pd, pd->on_xmlns, av)
+	     );
+      }
+      end_frame(fid, pd->exception);
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
+    }
+
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   return TRUE;
@@ -1347,25 +1461,41 @@ on_pi(dtd_parser *p, const ichar *pi)
     return TRUE;
 
   if ( pd->on_pi )
-  { fid_t fid = PL_open_foreign_frame();
-    term_t av = PL_new_term_refs(2);
+  { fid_t fid;
 
-    put_atom_wchars(av+0, pi);
-    unify_parser(av+1, p);
+    if ( (fid=PL_open_foreign_frame()) )
+    { int rc;
+      term_t av = PL_new_term_refs(2);
 
-    call_prolog(pd, pd->on_pi, av);
-    end_frame(fid, pd->exception);
+      rc = ( put_atom_wchars(av+0, pi) &&
+	     unify_parser(av+1, p) &&
+	     call_prolog(pd, pd->on_pi, av)
+	   );
+
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
+    }
+
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   if ( pd->tail )
-  { term_t h = PL_new_term_ref();
+  { term_t h;
 
-    if ( !PL_unify_list(pd->tail, h, pd->tail) )
+    if ( !(h = PL_new_term_ref()) ||
+	 !PL_unify_list(pd->tail, h, pd->tail) )
+    { pd->exception = PL_exception(0);
       return FALSE;
+    }
 
-    PL_unify_term(h,
-		  PL_FUNCTOR, FUNCTOR_pi1,
-		  PL_NWCHARS, wcslen(pi), pi);
+    if ( !PL_unify_term(h,
+			PL_FUNCTOR, FUNCTOR_pi1,
+			  PL_NWCHARS, wcslen(pi), pi) )
+    { pd->exception = PL_exception(0);
+      return FALSE;
+    }
 
     PL_reset_term_refs(h);
   }
@@ -1382,14 +1512,25 @@ on_decl(dtd_parser *p, const ichar *decl)
     return TRUE;
 
   if ( pd->on_decl )
-  { fid_t fid = PL_open_foreign_frame();
-    term_t av = PL_new_term_refs(2);
+  { fid_t fid;
+    term_t av;
 
-    put_atom_wchars(av+0, decl);
-    unify_parser(av+1, p);
+    if ( (fid = PL_open_foreign_frame()) &&
+	 (av = PL_new_term_refs(2)) )
+    { int rc;
 
-    call_prolog(pd, pd->on_decl, av);
-    end_frame(fid, pd->exception);
+      rc = ( put_atom_wchars(av+0, decl) &&
+	     unify_parser(av+1, p) &&
+	     call_prolog(pd, pd->on_decl, av)
+	   );
+      end_frame(fid, pd->exception);
+      PL_discard_foreign_frame(fid);
+      if ( rc )
+	return TRUE;
+    }
+
+    pd->exception = PL_exception(0);
+    return FALSE;
   }
 
   if ( pd->stopat == SA_DECL )
@@ -1416,7 +1557,10 @@ write_parser(void *h, char *buf, int len)
   }
 
   for(; s<e; s++)
-    putchar_dtd_parser(pd->parser, *s);
+  { putchar_dtd_parser(pd->parser, *s);
+    if ( pd->exception )
+      break;
+  }
 
   return len;
 }
@@ -1433,7 +1577,9 @@ close_parser(void *h)
   }
 
   if ( pd->tail )
-    PL_unify_nil(pd->tail);
+  { if ( !PL_unify_nil(pd->tail) )
+      return -1;			/* resource error */
+  }
 
   if ( p->dmode == DM_DTD )
     p->dtd->implicit = FALSE;		/* assume we loaded a DTD */
@@ -1496,7 +1642,7 @@ pl_open_dtd(term_t ref, term_t options, term_t stream)
     { term_t a = PL_new_term_ref();
       char *s;
 
-      PL_get_arg(1, option, a);
+      _PL_get_arg(1, option, a);
       if ( !PL_get_atom_chars(a, &s) )
 	return sgml2pl_error(ERR_TYPE, "atom", a);
 
@@ -1532,11 +1678,11 @@ set_callback_predicates(parser_data *pd, term_t option)
   int arity;
   module_t m = NULL;
 
-  PL_get_arg(2, option, a);
+  _PL_get_arg(2, option, a);
   PL_strip_module(a, &m, a);
   if ( !PL_get_atom(a, &pname) )
     return sgml2pl_error(ERR_TYPE, "atom", a);
-  PL_get_arg(1, option, a);
+  _PL_get_arg(1, option, a);
   if ( !PL_get_atom_chars(a, &fname) )
     return sgml2pl_error(ERR_TYPE, "atom", a);
 
@@ -1626,19 +1772,19 @@ pl_sgml_parse(term_t parser, term_t options)
   while ( PL_get_list(tail, head, tail) )
   { if ( PL_is_functor(head, FUNCTOR_document1) )
     { pd->list  = PL_new_term_ref();
-      PL_get_arg(1, head, pd->list);
+      _PL_get_arg(1, head, pd->list);
       pd->tail  = PL_copy_term_ref(pd->list);
       pd->stack = NULL;
     } else if ( PL_is_functor(head, FUNCTOR_source1) )
     { term_t a = PL_new_term_ref();
 
-      PL_get_arg(1, head, a);
+      _PL_get_arg(1, head, a);
       if ( !PL_get_stream_handle(a, &in) )
 	return FALSE;
     } else if ( PL_is_functor(head, FUNCTOR_content_length1) )
     { term_t a = PL_new_term_ref();
 
-      PL_get_arg(1, head, a);
+      _PL_get_arg(1, head, a);
       if ( !PL_get_int64(a, &content_length) )
 	return sgml2pl_error(ERR_TYPE, "integer", a);
       has_content_length = TRUE;
@@ -1649,7 +1795,7 @@ pl_sgml_parse(term_t parser, term_t options)
     { term_t a = PL_new_term_ref();
       char *s;
 
-      PL_get_arg(1, head, a);
+      _PL_get_arg(1, head, a);
       if ( !PL_get_atom_chars(a, &s) )
 	return sgml2pl_error(ERR_TYPE, "atom", a);
       if ( streq(s, "error") )
@@ -1662,7 +1808,7 @@ pl_sgml_parse(term_t parser, term_t options)
     { term_t a = PL_new_term_ref();
       char *s;
 
-      PL_get_arg(1, head, a);
+      _PL_get_arg(1, head, a);
       if ( !PL_get_atom_chars(a, &s) )
 	return sgml2pl_error(ERR_TYPE, "atom", a);
       if ( streq(s, "element") )
@@ -1680,14 +1826,14 @@ pl_sgml_parse(term_t parser, term_t options)
     } else if ( PL_is_functor(head, FUNCTOR_max_errors1) )
     { term_t a = PL_new_term_ref();
 
-      PL_get_arg(1, head, a);
+      _PL_get_arg(1, head, a);
       if ( !PL_get_integer(a, &pd->max_errors) )
 	return sgml2pl_error(ERR_TYPE, "integer", a);
     } else if ( PL_is_functor(head, FUNCTOR_syntax_errors1) )
     { term_t a = PL_new_term_ref();
       char *s;
 
-      PL_get_arg(1, head, a);
+      _PL_get_arg(1, head, a);
       if ( !PL_get_atom_chars(a, &s) )
 	return sgml2pl_error(ERR_TYPE, "atom", a);
 
@@ -1703,7 +1849,7 @@ pl_sgml_parse(term_t parser, term_t options)
     { term_t a = PL_new_term_ref();
       char *s;
 
-      PL_get_arg(1, head, a);
+      _PL_get_arg(1, head, a);
       if ( !PL_get_atom_chars(a, &s) )
 	return sgml2pl_error(ERR_TYPE, "atom", a);
 
@@ -1742,6 +1888,8 @@ pl_sgml_parse(term_t parser, term_t options)
     else
       p->encoded = FALSE;		/* already decoded */
 
+    pd->stopped = FALSE;
+
     if ( !recursive )
     { pd->source = in;
       begin_document_dtd_parser(p);
@@ -1809,7 +1957,9 @@ pl_sgml_parse(term_t parser, term_t options)
   out:
     reset_url_cache();
     if ( pd->tail )
-      PL_unify_nil(pd->tail);
+    { if ( !PL_unify_nil(pd->tail) )
+	return FALSE;
+    }
 
     if ( recursive )
     { p->closure = oldpd;
@@ -1833,7 +1983,7 @@ pl_sgml_parse(term_t parser, term_t options)
 		 *	  DTD PROPERTIES	*
 		 *******************************/
 
-static void	put_model(term_t t, dtd_model *m);
+static int put_model(term_t t, dtd_model *m) WUNUSED;
 
 /* doctype(DocType) */
 
@@ -1847,31 +1997,37 @@ dtd_prop_doctype(dtd *dtd, term_t prop)
 
 /* elements(ListOfElements) */
 
-static void
+WUNUSED static int
 make_model_list(term_t t, dtd_model *m, functor_t f)
 { if ( !m->next )
-  { put_model(t, m);
+  { return put_model(t, m);
   } else
-  { term_t av = PL_new_term_refs(2);
+  { term_t av;
 
-    put_model(av+0, m);
-    make_model_list(av+1, m->next, f);
-    PL_cons_functor_v(t, f, av);
-    PL_reset_term_refs(av);
+    if ( (av=PL_new_term_refs(2)) &&
+	 put_model(av+0, m) &&
+	 make_model_list(av+1, m->next, f) &&
+	 PL_cons_functor_v(t, f, av) )
+    { PL_reset_term_refs(av);
+      return TRUE;
+    }
+
+    return FALSE;
   }
 }
 
 
-static void
+WUNUSED static int
 put_model(term_t t, dtd_model *m)
-{ functor_t f;
+{ int rc = TRUE;
+  functor_t f;
 
   switch(m->type)
   { case MT_PCDATA:
-      PL_put_atom(t, ATOM_pcdata);
+      rc = PL_put_atom(t, ATOM_pcdata);
       goto card;
     case MT_ELEMENT:
-      put_atom_wchars(t, m->content.element->name->name);
+      rc = put_atom_wchars(t, m->content.element->name->name);
       goto card;
     case MT_AND:
       f = FUNCTOR_and2;
@@ -1889,46 +2045,51 @@ put_model(term_t t, dtd_model *m)
       break;
   }
 
-  if ( !m->content.group )
-    PL_put_atom(t, ATOM_empty);
-  else
-    make_model_list(t, m->content.group, f);
+  if ( rc )
+  { if ( !m->content.group )
+      rc = PL_put_atom(t, ATOM_empty);
+    else
+      rc = make_model_list(t, m->content.group, f);
+  }
 
 card:
+  if ( !rc )
+    return FALSE;
+
   switch(m->cardinality)
   { case MC_ONE:
       break;
     case MC_OPT:
-      PL_cons_functor_v(t, FUNCTOR_opt1, t);
+      rc = PL_cons_functor_v(t, FUNCTOR_opt1, t);
       break;
     case MC_REP:
-      PL_cons_functor_v(t, FUNCTOR_rep1, t);
+      rc = PL_cons_functor_v(t, FUNCTOR_rep1, t);
       break;
     case MC_PLUS:
-      PL_cons_functor_v(t, FUNCTOR_plus1, t);
+      rc = PL_cons_functor_v(t, FUNCTOR_plus1, t);
       break;
   }
+
+  return rc;
 }
 
 
-static void
+WUNUSED static int
 put_content(term_t t, dtd_edef *def)
 { switch(def->type)
   { case C_EMPTY:
-      PL_put_atom(t, ATOM_empty);
-      return;
+      return PL_put_atom(t, ATOM_empty);
     case C_CDATA:
-      PL_put_atom(t, ATOM_cdata);
-      return;
+      return PL_put_atom(t, ATOM_cdata);
     case C_RCDATA:
-      PL_put_atom(t, ATOM_rcdata);
-      return;
+      return PL_put_atom(t, ATOM_rcdata);
     case C_ANY:
-      PL_put_atom(t, ATOM_any);
-      return;
+      return PL_put_atom(t, ATOM_any);
     default:
       if ( def->content )
-	put_model(t, def->content);
+	return put_model(t, def->content);
+
+      return TRUE;
   }
 }
 
@@ -1985,8 +2146,8 @@ dtd_prop_element(dtd *dtd, term_t name, term_t omit, term_t content)
 		        PL_ATOM, def->omit_close ? ATOM_true : ATOM_false) )
     return FALSE;
 
-  put_content(model, def);
-  return PL_unify(content, model);
+  return ( put_content(model, def) &&
+	   PL_unify(content, model) );
 }
 
 
@@ -2062,21 +2223,22 @@ unify_attribute_type(term_t type, dtd_attr *a)
 
   if ( a->type == AT_NAMEOF || a->type == AT_NOTATION )
   { dtd_name_list *nl;
-    term_t tail = PL_new_term_ref();
-    term_t head = PL_new_term_ref();
-    term_t elem = PL_new_term_ref();
+    term_t tail, head, elem;
 
-    if ( !PL_unify_functor(type,
+    if ( !(tail = PL_new_term_ref()) ||
+	 !(head = PL_new_term_ref()) ||
+	 !(elem = PL_new_term_ref()) ||
+	 !PL_unify_functor(type,
 			   a->type == AT_NAMEOF ?
 			     FUNCTOR_nameof1 :
 			     FUNCTOR_notation1) )
       return FALSE;
-    PL_get_arg(1, type, tail);
+
+    _PL_get_arg(1, type, tail);
 
     for(nl = a->typeex.nameof; nl; nl = nl->next)
-    { put_atom_wchars(elem, nl->value->name);
-
-      if ( !PL_unify_list(tail, head, tail) ||
+    { if ( !put_atom_wchars(elem, nl->value->name) ||
+	   !PL_unify_list(tail, head, tail) ||
 	   !PL_unify(head, elem) )
 	return FALSE;
     }
@@ -2109,9 +2271,12 @@ unify_attribute_default(term_t defval, dtd_attr *a)
       v = PL_unify_functor(defval, FUNCTOR_fixed1);
     common:
       if ( v )
-      { term_t tmp = PL_new_term_ref();
+      { term_t tmp;
 
-	PL_get_arg(1, defval, tmp);
+	if ( !(tmp=PL_new_term_ref()) )
+	  return FALSE;
+
+	_PL_get_arg(1, defval, tmp);
 	switch( a->type )
 	{ case AT_CDATA:
 	    return PL_unify_wchars(tmp, PL_ATOM, ENDSNUL, a->att_def.cdata);
@@ -2361,7 +2526,7 @@ pl_dtd_property(term_t ref, term_t property)
       int i;
 
       for(i=0; i<p->arity; i++)
-	PL_get_arg(i+1, property, a+i);
+	_PL_get_arg(i+1, property, a+i);
 
       switch(p->arity)
       { case 1:
@@ -2443,3 +2608,8 @@ install()
 #endif
 }
 
+
+install_t
+uninstall()
+{ stop_ring();
+}
