@@ -238,7 +238,9 @@ emit_xml_encoding(Out, Options) :-
 	option(header(Hdr), Options, true),
 	Hdr == true, !,
 	stream_property(Out, encoding(Encoding)),
-	(   Encoding == utf8
+	(   (   Encoding == utf8
+	    ;	Encoding == wchar_t
+	    )
 	->  format(Out, '<?xml version="1.0" encoding="UTF-8"?>~n~n', [])
 	;   Encoding == iso_latin_1
 	->  format(Out, '<?xml version="1.0" encoding="ISO-8859-1"?>~n~n', [])
@@ -294,14 +296,20 @@ write_doctype(Out, DocType, PubId, SysId) :-
 %
 %	Emit a single element
 
+emit(Var, _, _) :-
+	var(Var), !,
+	instantiation_error(Var).
 emit([], _, _) :- !.
 emit([H|T], Out, State) :- !,
 	emit(H, Out, State),
 	emit(T, Out, State).
+emit(CDATA, Out, State) :-
+	atom(CDATA), !,
+	sgml_write_content(Out, CDATA, State).
 emit(Element, Out, State) :-
 	\+ \+ emit_element(Element, Out, State).
 
-emit_element(pi(PI), Out, State) :-
+emit_element(pi(PI), Out, State) :- !,
 	get_state(State, entity_map, EntityMap),
 	write(Out, <?),
 	write_quoted(Out, PI, "", EntityMap),
@@ -309,7 +317,9 @@ emit_element(pi(PI), Out, State) :-
 	    write(Out, ?>)
 	;   write(Out, >)
 	).
-emit_element(element(Name, Attributes, Content), Out, State) :-
+emit_element(element(Name, Attributes, Content), Out, State) :- !,
+	must_be(list, Attributes),
+	must_be(list, Content),
 	att_length(Attributes, State, Alen),
 	(   Alen > 60,
 	    get_state(State, layout, true)
@@ -331,6 +341,8 @@ emit_element(element(Name, Attributes, Content), Out, State) :-
 	;   attributes(Attributes, Sep, Out, State)
 	),
 	content(Content, Out, Name, State).
+emit_element(E, _, _) :-
+	type_error(xml_dom, E).
 
 attributes([], _, _, _).
 attributes([H|T], Sep, Out, State) :-
@@ -395,7 +407,7 @@ emit_name(URI:Name, Out, State) :-
 	->  write(Out, Name)
 	;   format(Out, '~w:~w', [NS, Name])
 	).
-emit_name(Term, Out, _) :-
+emit_name(Term, Out, _) :-		% error?
 	write(Out, Term).
 
 %%	update_nsmap(+Attributes, !State)
@@ -571,6 +583,7 @@ generate_ns(_, NS) :-
 	rdf_db:ns/2.
 
 default_ns('http://www.w3.org/2001/XMLSchema-instance', xsi).
+default_ns('http://www.w3.org/1999/xhtml', xhtml).
 default_ns(URI, NS) :-
 	rdf_db:ns(NS, URI).
 
@@ -650,7 +663,8 @@ writeq([H|T], Out, Escape, EntityMap) :-
 	(   memberchk(H, Escape)
 	->  write_entity(H, Out, EntityMap)
 	;   H >= 256
-	->  (   stream_property(Out, encoding(utf8))
+	->  (   stream_property(Out, encoding(Enc)),
+		unicode_encoding(Enc)
 	    ->	put_code(Out, H)
 	    ;	write_entity(H, Out, EntityMap)
 	    )
@@ -658,6 +672,10 @@ writeq([H|T], Out, Escape, EntityMap) :-
 	),
 	writeq(T, Out, Escape, EntityMap).
 
+unicode_encoding(utf8).
+unicode_encoding(wchar_t).
+unicode_encoding(unicode_le).
+unicode_encoding(unicode_be).
 
 write_entity(Code, Out, EntityMap) :-
 	(   get_assoc(Code, EntityMap, EntityName)
