@@ -29,14 +29,17 @@
     the GNU General Public License.
 */
 
+:- asserta(user:file_search_path(library, '.')).
+:- asserta(user:file_search_path(library, '../clib')).
 :- asserta(user:file_search_path(foreign, '.')).
+:- asserta(user:file_search_path(foreign, '../clib')).
 
-:- use_module(ssl).
+:- use_module(library(ssl)).
 :- use_module(library(debug)).
 :- use_module(library(readutil)).
 
 %:- debug(connection).
-:- debug(_).
+%:- debug(_).
 
 :- dynamic
 	option/1,			% Options to test
