@@ -26,10 +26,6 @@
 #define SSLLIBH__
 #include "../clib/nonblockio.h"
 
-#ifdef __WINDOWS__
-typedef size_t socklen_t;
-#endif
-
 #define SSL_CONFIG_MAGIC 0x539dbe3a
 
 typedef int BOOL;
@@ -47,6 +43,7 @@ typedef enum
 } PL_SSL_ROLE;
 
 #include <openssl/x509.h>
+#include <openssl/x509v3.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 
@@ -90,8 +87,8 @@ typedef struct pl_ssl {
      * Application defined handlers
      */
     BOOL                (*pl_ssl_cb_cert_verify)( struct pl_ssl *config
-                                                , const char *cert
-						, long cert_len
+                                                , X509*
+                                                , X509_STORE_CTX*
                                                 , const char *error
                                                 ) ;
     void *              pl_ssl_cb_cert_verify_data;
@@ -149,8 +146,8 @@ BOOL		ssl_set_close_parent(PL_SSL *config, int closeparent);
 BOOL            ssl_set_cb_cert_verify
                                  ( PL_SSL *config
                                  , BOOL (*callback)( PL_SSL *
-                                                   , const char *
-						   , long
+                                                   , X509*
+                                                   , X509_STORE_CTX*
                                                    , const char *
                                                    )
                                  , void *
@@ -169,4 +166,8 @@ void            ssl_err          (char *fmt, ...);
 int		ssl_set_debug	 (int level);
 void            ssl_deb          (int level, char *fmt, ...);
 
+extern BIO_METHOD bio_read_functions;
+extern BIO_METHOD bio_write_functions;
+
 #endif
+
