@@ -1,10 +1,25 @@
 /*  $Id$
 
-    Part of XPCE
-    Designed and implemented by Anjo Anjewierden and Jan Wielemaker
-    E-mail: jan@swi.psy.uva.nl
+    Part of XPCE --- The SWI-Prolog GUI toolkit
 
-    Copyright (C) 1996 University of Amsterdam. All rights reserved.
+    Author:        Jan Wielemaker and Anjo Anjewierden
+    E-mail:        jan@swi.psy.uva.nl
+    WWW:           http://www.swi.psy.uva.nl/projects/xpce/
+    Copyright (C): 1985-2002, University of Amsterdam
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
 :- module(classdoc,
@@ -43,7 +58,7 @@ do_process(In, Out) :-
 	),
 	make_diagrams('figs/class'),
 	no_summary.
-	    
+
 :- dynamic
 	diagram/3.			% LineNo, file, classes
 
@@ -142,7 +157,7 @@ class_to_tex(Name, TeXName) :-
 	atom_codes(Name, Chars),
 	member(C, Chars),
 	\+ alnum(C), !,
-	concat_atom(['{\\tt\\string', Name, '}'], TeXName).
+	atomic_list_concat(['{\\tt\\string', Name, '}'], TeXName).
 class_to_tex(Name, Name).
 
 alnum(C) :-
