@@ -35,16 +35,13 @@
 	, pce_autoload_all/0
 	]).
 
-:- use_module(pce_boot(pce_principal),
-	      [ get/3,
-		send/2
-	      ]).
+:- use_module(pce_boot(pce_principal)).
 :- use_module(pce_boot(pce_realise),
 	      [ pce_realise_class/1,
 		pce_prolog_class/1
 	      ]).
 :- require([ is_absolute_file_name/1
-	   , concat_atom/2
+	   , atomic_list_concat/2
 	   , absolute_file_name/3
 	   , get/3
 	   ]).
@@ -69,7 +66,7 @@ pce_autoload(Class, Abs) :-
 	assert(autoload(Class, Canonical)).
 pce_autoload(Class, Local) :-
 	prolog_load_context(directory, Dir),
-	concat_atom([Dir, /, Local], File),
+	atomic_list_concat([Dir, /, Local], File),
 	pce_host:property(file_extensions(Exts)),
 	absolute_file_name(File,
 			   [ extensions(Exts),
@@ -101,7 +98,7 @@ register_handler :-
 	register_handler.
 
 pce_ifhostproperty(prolog(swi),
-		   (:- '$hide'(trap_autoload, 1)),
+		   (:- '$hide'(trap_autoload/1)),
 		   (notrace(G) :- G)).
 
 trap_autoload(Class) :-
