@@ -126,6 +126,8 @@ map_term_expand(X, Y) :-
 map_term_expand(X, X).
 
 
+do_term_expand(end_of_file, _) :-
+	cleanup, !, fail.
 do_term_expand(In0, Out) :-
 	pce_expandable(In0),
 	(   do_expand(In0, Out0)
@@ -288,8 +290,8 @@ do_expand((:- pce_end_class),
 	findall(R, retract(attribute(ClassName, classvar, R)),  Resources),
 	findall(D, retract(attribute(ClassName, directive, D)), Directs),
 	(   attribute(ClassName, extending, true)
-	->  MetaClass = '-',
-	    Super = '-',
+	->  MetaClass = (-),
+	    Super = (-),
 	    expand_term((:- initialization(pce_extended_class(ClassName))),
 			RegisterDecl)
 	;   retract(attribute(ClassName, super, Super)),
@@ -941,6 +943,21 @@ pce_compiling :-
 	compiling(_, _), !.
 
 
+		 /*******************************
+		 *	      CLEANUP		*
+		 *******************************/
+
+%	cleanup
+%
+%	Cleanup the compilation data. We should  probably give a warning
+%	when not under xref and there is data left.
+
+cleanup :-
+	source_location(Path, _),
+	forall(retract(compiling(Class, Path)),
+	       retractall(attribute(Class, _, _))).
+
+
 		 /*******************************
 		 *	      CHECKS		*
 		 *******************************/
