@@ -134,7 +134,7 @@ class_variable(varmark_style, style*,
 	       style(background := honeydew,
 		     underline := @on)).
 class_variable(show_syntax_errors, {never,typing,pause},
-	       pause).
+	       typing).
 
 variable(varmark_style,    style*,       get, "How to mark variables").
 variable(has_var_marks,    bool := @off, get, "Optimise a bit").
@@ -204,6 +204,8 @@ expand_path(Term, D) :-
 	      new(regex(':-|:->|:<-|-->'))).
 :- pce_global(@prolog_full_stop,
 	      new(regex('[^-#$&*+./:<=>?@\\\\^`~]\\.($|\\s)'))).
+:- pce_global(@prolog_decl_regex,
+	      new(regex('^:-\\s*[a-z_]+'))).
 
 indent_line(E) :->
 	"Indent current line (Prolog indentation)"::
@@ -282,6 +284,7 @@ indent_clause_line(E) :->
 	get(E, caret, Caret),
 	get(E, text_buffer, TB),
 	get(E, skip_comment, Caret-1, 0, Glue),
+	debug(emacs(indent), 'Glue at ~d', [Glue]),
 	(   send(regex(\.), match, TB, Glue)		% new clause
 	->  send(E, align_line, 0)
 	;   send(regex(','), match, TB, Glue)	  	% Next subclause
@@ -292,6 +295,8 @@ indent_clause_line(E) :->
 	    )
 	;   send(@prolog_neck_regex, match, TB, Glue+1, 0) % First subclause
 	->  send(E, align_line, 8)			% (seach backward)
+	;   send(@prolog_decl_regex, match, TB, Glue+1, 0)
+	->  send(E, align_line, 8)
 	;   send(E, align_with_previous_line)
 	).
 
@@ -348,10 +353,12 @@ fill_paragraph(M, Justify:[int]) :->
 
 fill_comment(M,
 	     Start:from=int, End:to=int,
-	     Re:leading=regex, Justify:justify=[bool|int]) :->
+	     Re:leading=regex, Justify:justify=[bool|int],
+	     LeadCont:lead_continuation=[char_array]) :->
 	"Fill/justify comments"::
 	send(M, slot, var_mark_enabled, @off),
-	call_cleanup(send_super(M, fill_comment, Start, End, Re, Justify),
+	call_cleanup(send_super(M, fill_comment, Start, End, Re, Justify,
+				LeadCont),
 		     send(M, slot, var_mark_enabled, @on)),
 	(   send(M, has_send_method, colourise_comments)
 	->  send(M, colourise_comments, Start, End)
@@ -559,10 +566,12 @@ pce_define_class(M, Name:name,
 
 prolog_navigator(M) :->
 	"Open source-file browser"::
-	get(M, file, File), File \== @nil,
-	get(File, absolute_path, Path),
-	get(M, line_number, Line),
-	prolog_ide(open_navigator(source_location(Path, Line))).
+	(   get(M, file, File), File \== @nil
+	->  get(File, absolute_path, Path),
+	    get(M, line_number, Line),
+	    prolog_ide(open_navigator(source_location(Path, Line)))
+	;   prolog_ide(open_navigator)
+	).
 
 
 edit_breakpoints(_M) :->
