@@ -1,10 +1,32 @@
 /*  $Id$
 
-    Part of XPCE
-    Designed and implemented by Anjo Anjewierden and Jan Wielemaker
-    E-mail: jan@swi.psy.uva.nl
+    Part of XPCE --- The SWI-Prolog GUI toolkit
 
-    Copyright (C) 1995 University of Amsterdam. All rights reserved.
+    Author:        Jan Wielemaker and Anjo Anjewierden
+    E-mail:        J.Wielemaker@cs.vu.nl
+    WWW:           http://www.swi-prolog.org/packages/xpce/
+    Copyright (C): 2010, University of Amsterdam
+
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
+    as published by the Free Software Foundation; either version 2
+    of the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    As a special exception, if you link this library with other files,
+    compiled with a Free Software compiler, to produce an executable, this
+    library does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however
+    invalidate any other reasons why the executable file might be covered by
+    the GNU General Public License.
 */
 
 :- module(pce_password_item,
@@ -23,7 +45,7 @@ find.
 		 /*******************************
 		 *	 CLASS PASSWD_ITEM	*
 		 *******************************/
-	    
+
 :- pce_begin_class(password_item, text_item, "text-item for entering a passwd").
 
 variable(shadow,	text_item,	get, "The real (invisible) item").
@@ -65,7 +87,7 @@ update(I) :->
 	make_star_string(Size, Stars),
 	send_super(I, selection, Stars),
 	send(I, caret, Caret).
-	    
+
 
 selection(I, Passwd:string) :<-
 	get(I, shadow, Shadow),
