@@ -42,10 +42,11 @@
 :- pce_autoload(float_item,   library(pce_float_item)).
 :- pce_autoload(tick_box,     library(pce_tick_box)).
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+/** <module> XPCE-base SWI-Prolog thread monitor
+
 This library defines  the  class   prolog_thread_monitor,  a  frame that
 displays the status of threads.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+*/
 
 resource(running,   image, library('trace/icons/mini-run.xpm')).
 resource(true,	    image, image('16x16/ok.xpm')).
@@ -58,22 +59,26 @@ resource(profiling, image, image('16x16/profiler.xpm')).
 :- pce_begin_class(thread_statistics, object,
 		   "Record historic status of a thread").
 
-variable(local,	 int, get, "Local-stack usage").
-variable(global, int, get, "Global-stack usage").
-variable(trail,	 int, get, "Trail-stack usage").
-variable(cpu,	 int, get, "%CPU (0..100)").
+variable(local,	 int := 0, get, "Local-stack usage").
+variable(global, int := 0, get, "Global-stack usage").
+variable(trail,	 int := 0, get, "Trail-stack usage").
+variable(cpu,	 int := 0, get, "%CPU (0..100)").
 
 initialise(TS, Thread:any, CPU:int) :->
 	"Create for victim"::
 	send_super(TS, initialise),
+	(   catch(thread_stack_sizes(Thread, Local, Global, Trail), _, fail)
+	->  send(TS, slot, local,  Local),
+	    send(TS, slot, global, Global),
+	    send(TS, slot, trail,  Trail)
+	;   true
+	),
+	send(TS, slot, cpu, CPU).
+
+thread_stack_sizes(Thread, Local, Global, Trail) :-
 	thread_statistics(Thread, localused, Local),
 	thread_statistics(Thread, globalused, Global),
-	thread_statistics(Thread, trailused, Trail),
-
-	send(TS, slot, local,  Local),
-	send(TS, slot, global, Global),
-	send(TS, slot, trail,  Trail),
-	send(TS, slot, cpu,    CPU).
+	thread_statistics(Thread, trailused, Trail).
 
 :- pce_end_class(thread_statistics).
 
@@ -124,7 +129,12 @@ update(TS, Status:prolog, CpuH:[int]) :->
 
 cpu_percentage(TS, CPU:'0..100') :<-
 	get(TS, key, TID),
-	thread_statistics(TID, cputime, Now),
+	(   catch(thread_statistics(TID, cputime, Now), _, fail)
+	->  true
+	;   get(TS, lastcpu, Now), Now \== @nil
+	->  true
+	;   Now = 0.0
+	),
 	get_time(WallNow),
 	(   get(TS, lastcpu, Last), Last \== @nil,
 	    get(TS, lastwall, LastWall),
@@ -249,12 +259,6 @@ initialise(TB) :->
 			      message(@arg1, join),
 			      condition := not(IsRunning)),
 		    gap,
-		    menu_item(attach_console,
-			      message(@arg1, signal, attach_console),
-			      condition := IsRunning),
-		    menu_item(trace,
-			      message(@arg1, trace),
-			      condition := IsRunning),
 		    menu_item(graphical_debugger,
 			      message(@arg1, gtrace),
 			      condition := IsRunning),
@@ -265,6 +269,13 @@ initialise(TB) :->
 			      message(@arg1, signal, nodebug),
 			      condition := IsRunning),
 		    gap,
+		    menu_item(trace,
+			      message(@arg1, trace),
+			      condition := IsRunning),
+		    menu_item(attach_console,
+			      message(@arg1, signal, attach_console),
+			      condition := IsRunning),
+		    gap,
 		    menu_item(profile,
 			      message(@arg1, start_profile),
 			      condition := and(IsRunning,
@@ -600,7 +611,8 @@ initialise(TM) :->
 	send(TM, fill_tool_dialog, TD),
 	send(TM, update),
 	get(TM, update_interval, Time),
-	send(TM, update_interval, Time).
+	send(TM, update_interval, Time),
+	send(TM, selection, main).
 
 unlink(TM) :->
 	send(TM, update_interval, @nil),
@@ -654,6 +666,12 @@ update(TM) :->
 	get(TM, member, thread_browser, TB),
 	send(TB, update).
 
+selection(TM, Thread:'name|int') :->
+	"Select the given thread"::
+	get(TM, member, thread_browser, TB),
+	get(TB, member, Thread, DI),
+	send(TB, selection, DI),
+	send(TB, details, DI).
 
 recall(TM, Recall:'1..') :->
 	"#samples recalled"::
