@@ -46,8 +46,12 @@
 %
 %	Display parent environments and choicepoints.
 
+running_in_pce_thread :-
+	pce_thread(Pce),
+	thread_self(Pce).
+
 display_stack(Window, Call, Choice) :-
-	assertion(thread_self(main)),
+	assertion(running_in_pce_thread),
 	get(@pce, convert, normal, font, Font),
 	get(Font, height, FH),
 	merge_choices(Choice, Call, AllFrames),
@@ -111,7 +115,7 @@ display_levels([Y-Frames|T], Choice, Window) :-
 	SortedFrames = [frame(Frame, PC)|_],
 	prolog_frame_attribute(Window, Frame, level, Level),
 	send(Window, display, text(Level, left, normal), point(5, Y)),
-	(   PC == choice
+	(   PC = choice(_)
 	->  X0 is 30 + 150
 	;   X0 is 30
 	),
@@ -119,12 +123,12 @@ display_levels([Y-Frames|T], Choice, Window) :-
 	display_levels(T, Choice, Window).
 
 
-cmpframes(Choice, Result, frame(Fr1, choice), frame(Fr2, choice)) :- !,
-	nth1(I1, Choice, frame(Fr1, choice)),
-	nth1(I2, Choice, frame(Fr2, choice)),
+cmpframes(Choice, Result, frame(Fr1, choice(_)), frame(Fr2, choice(_))) :- !,
+	nth1(I1, Choice, frame(Fr1, choice(_))),
+	nth1(I2, Choice, frame(Fr2, choice(_))),
 	compare(Result, I1, I2).
-cmpframes(_, <, _, frame(_, choice)) :- !.
-cmpframes(_, >, frame(_, choice), _) :- !.
+cmpframes(_, <, _, frame(_, choice(_))) :- !.
+cmpframes(_, >, frame(_, choice(_)), _) :- !.
 cmpframes(_, Result, F1, F2) :-
 	compare(Result, F1, F2).
 
@@ -137,9 +141,9 @@ display_frames([F|T], Window, X, Y) :-
 	display_frames(T, Window, X2, Y).
 
 
-v_stack_frame(Window, frame(Frame, choice), V) :- !,
+v_stack_frame(Window, frame(Frame, choice(CH)), V) :- !,
 	frame_label(Window, Frame, Label),
-	new(V, prolog_stack_frame(Window, Frame, Label, choice, choicepoint)).
+	new(V, prolog_stack_frame(Window, Frame, Label, choice(CH), choicepoint)).
 v_stack_frame(Window, frame(Frame, PC), V) :-
 	frame_label(Window, Frame, Label),
 	frame_style(Window, Frame, PC, Style),
@@ -219,7 +223,7 @@ clean_level(Text) :-
 variable(members,       hash_table,  get, "Frame --> Visualiser table").
 
 initialise(B) :->
-	assertion(thread_self(main)),
+	assertion(running_in_pce_thread),
 	get(@pce, convert, normal, font, Font),
 	get(Font, ex, Ex),
 	send_super(B, initialise, size := size(20 * Ex, 100)),
@@ -235,13 +239,15 @@ clear(B) :->
 	send(B?graphicals, for_all, message(@arg1, destroy)).
 
 
-member(B, F:int, PC:[name|int], V:prolog_stack_frame) :<-
+member(B, F:int, Location:[prolog], V:prolog_stack_frame) :<-
 	"Find visualiser from frame"::
 	get(B, members, Table),
 	get(Table, member, F, Chain),
-	(   PC == @default
+	(   Location == @default
 	->  get(Chain, head, V)
-	;   get(Chain, find, @arg1?pc == PC, V)
+	;   Location = choice(CHP)
+	->  get(Chain, find, @arg1?choice == CHP, V)
+	;   get(Chain, find, @arg1?pc == Location, V)
 	).
 
 
@@ -320,7 +326,7 @@ down(B, Times:[int]) :->
 	NN is -N,
 	send(B, up, NN).
 
-selection(B, What:'int|prolog_stack_frame', PC:[int|name]) :->
+selection(B, What:'int|prolog_stack_frame', PC:location=[prolog]) :->
 	"Select the given frame"::
 	(   integer(What)
 	->  (   get(B, member, What, PC, V)
@@ -373,6 +379,7 @@ style_image(user,		'user.xpm').
 
 variable(frame_reference,  int,		get, "Reference of Prolog frame").
 variable(pc,		   'int|name',	get, "Location in the frame").
+variable(choice,	   int*,	get, "Id of choice-point").
 variable(frame_level,	   int,		get, "Nesting of the frame").
 
 :- pce_global(@prolog_stack_frame_recogniser,
@@ -380,7 +387,7 @@ variable(frame_level,	   int,		get, "Nesting of the frame").
 				message(@receiver, select)))).
 
 initialise(D, Window:window, Frame:int, Label:char_array,
-	   PC:'int|name', Style:name) :->
+	   Location:prolog, Style:name) :->
 	send_super(D, initialise),
 	send(D, border, 3),
 	send(D, shadow, 1),
@@ -390,7 +397,11 @@ initialise(D, Window:window, Frame:int, Label:char_array,
 	send(D, display, new(B, bitmap(Image))),
 	send(D, display, text(Label, left, normal), point(B?right_side, 0)),
 	send(D, slot, frame_reference, Frame),
-	send(D, slot, pc, PC),
+	(   Location = choice(CH)
+	->  send(D, slot, pc, choice),
+	    send(D, slot, choice, CH)
+	;   send(D, slot, pc, Location)
+	),
 	prolog_frame_attribute(Window, Frame, level, Level),
 	send(D, slot, frame_level, Level).
 
@@ -418,7 +429,12 @@ select(D, Show:[bool]) :->
 	->  get(D, frame_reference, Frame),
 	    get(D, pc, PC),
 	    debug('~p: select ~p at PC=~w~n', [D, Frame, PC]),
-	    send(D?frame, show_frame, Frame, PC)
+	    (	PC == choice
+	    ->	get(D, choice, CH),
+		Location = choice(CH)
+	    ;	Location = PC
+	    ),
+	    send(D?frame, show_frame, Frame, Location)
 	;   true
 	).
 
