@@ -92,7 +92,7 @@ NewClass(parbox)
   Int		line_width;		/* Max width of a line */
   Vector	content;		/* Contained hboxes */
   Name		alignment;		/* left,right,center,justify */
-  Bool		auto_crop;		/* Crop content */
+  BoolObj		auto_crop;		/* Crop content */
 End;
 
 
