@@ -596,9 +596,9 @@ test, conversion and computation macro's are provided.
 #define ON		(&BoolOn)
 #define OFF		(&BoolOff)
 
-#define isOn(val)	((Bool)(val) == ON)
-#define isOff(val)	((Bool)(val) == OFF)
-#define isBoolean(val)	((Bool)(val) == ON || (Bool)(val) == OFF)
+#define isOn(val)	((BoolObj)(val) == ON)
+#define isOff(val)	((BoolObj)(val) == OFF)
+#define isBoolean(val)	((BoolObj)(val) == ON || (BoolObj)(val) == OFF)
 
 #define isNil(o)	((Constant)(o) == NIL)
 #define notNil(o)	((Constant)(o) != NIL)
@@ -655,7 +655,7 @@ test, conversion and computation macro's are provided.
   uintptr_t	dflags;			/* Debugging flags */
 
 #define ABSTRACT_RECOGNISER \
-  Bool		active;			/* Does accept events? */
+  BoolObj		active;			/* Does accept events? */
 
 #define ABSTRACT_CODE \
   ABSTRACT_PROGRAM_OBJECT
@@ -941,7 +941,7 @@ NewClass(class)
   Sheet		features;		/* installed features */
   Int		no_created;		/* how many were created */
   Int		no_freed;		/* how many were freed */
-  Bool		solid;			/* graphicals: OFF by default */
+  BoolObj		solid;			/* graphicals: OFF by default */
   Name		selection_style;	/* graphicals: feedback selected */
   Chain		handles;		/* graphicals only: connection pts */
   Int		instance_size;		/* Instance size in bytes */
@@ -951,7 +951,7 @@ NewClass(class)
   Chain		changed_messages;	/* Trap instance changes */
   Chain		created_messages;	/* Trap instance creation */
   Chain		freed_messages;		/* Trap instance destruction */
-  Bool		un_answer;		/* Decide on slot assignment */
+  BoolObj		un_answer;		/* Decide on slot assignment */
 
   Code		make_class_message;	/* Message to build the class */
 
@@ -969,12 +969,12 @@ NewClass(class)
   HashTable	class_variable_table;	/* hash-table of class-variables */
   HashTable	instances;		/* hash-table holding the instances */
 
-  Bool		realised;		/* Class has been realised? */
+  BoolObj		realised;		/* Class has been realised? */
   Name		init_variables;		/* How to initialise slots */
 
   InstanceProto	proto;			/* Prototype instance */
-  int		tree_index;		/* Index in depth-first tree */
-  int		neighbour_index;	/* Index of my neighbour */
+  intptr_t	tree_index;		/* Index in depth-first tree */
+  intptr_t	neighbour_index;	/* Index of my neighbour */
 
   GetFunc	get_function;		/* `Get' on Code objects */
   SendFunc	send_function;		/* `Send' on Code objects */
@@ -985,7 +985,7 @@ NewClass(class)
   SendFunc	changedFunction;	/* Trap instance changes */
   SendFunc	in_event_area_function;	/* Test if event is in area */
   SendFunc	make_class_function;	/* makeClass function pointer */
-  int		boot;			/* When booting: #pce-slots; else 0 */
+  intptr_t	boot;			/* When booting: #pce-slots; else 0 */
 
   classdecl    *c_declarations;		/* Non-object declarations */
 End;
@@ -997,8 +997,8 @@ NewClass(type)
   Name		argument_name;		/* Name of the argument */
   Chain		supers;			/* Super-types */
   Any		context;		/* Context argument for functions */
-  Bool		vector;			/* Method: vector of these */
-  int		validate_function;	/* Function to check the type */
+  BoolObj	vector;			/* Method: vector of these */
+  intptr_t	validate_function;	/* Function to check the type */
   Func		translate_function;	/* Function to convert the type */
 End;
 
@@ -1093,7 +1093,7 @@ End;
 #define ABSTRACT_HASH_TABLE \
   Name		refer;			/* Maintain references */ \
   Int		size;			/* # symbols in table */ \
-  int		buckets;		/* # buckets in symbol-array */ \
+  intptr_t	buckets;		/* # buckets in symbol-array */ \
   Symbol	symbols;		/* Symbol-array */
 
 NewClass(hash_table)
@@ -1149,12 +1149,12 @@ End;
 
 NewClass(pce)
 #ifndef O_RUNTIME
-  Bool		debugging;		/* debugging? (watching spy points) */
-  Bool		trap_errors;		/* Trap tracer on errors */
+  BoolObj		debugging;		/* debugging? (watching spy points) */
+  BoolObj		trap_errors;		/* Trap tracer on errors */
 #endif
   Name		last_error;		/* Last error occured */
   Chain		catched_errors;		/* Stack of catched error-id's */
-  Bool		catch_error_signals;	/* Catch Unix signals */
+  BoolObj		catch_error_signals;	/* Catch Unix signals */
 
   Chain		exit_messages;		/* Called on exit */
   Sheet		exception_handlers;	/* exception-name --> code */
@@ -1184,7 +1184,7 @@ End;
 #define ABSTRACT_HOST \
   Name		language;		/* Prolog, Lisp, ... */ \
   Name		system;			/* host system we are connected to */ \
-  Bool		callBack;		/* if @on can be called directly */ \
+  BoolObj		callBack;		/* if @on can be called directly */ \
   Chain		messages;		/* messages waiting in queue */
 
 
@@ -1681,8 +1681,8 @@ extern char *T_report[];		/* ->report: kind, format, args... */
   { int _i; for(_i=0; _i < (n); _i++) (t)[_i] = (f)[_i]; }
 
 #define for_chain(ch, val, code) \
-  { int _i=0, _size  = valInt(ch->size); \
-    Any *_array = (Any *)alloca(_size * sizeof(Any)); \
+  { intptr_t _i=0, _size  = valInt(ch->size); \
+    Any *_array = (Any *)alloca((size_t)_size * sizeof(Any)); \
     Cell _cell = ch->head; \
 	\
     for( ; notNil(_cell); _cell = _cell->next, _i++ ) \
@@ -1699,7 +1699,7 @@ extern char *T_report[];		/* ->report: kind, format, args... */
   }
 
 #define for_vector(v, val, code) \
-  { int _iv, _sizev = valInt((v)->size); \
+  { intptr_t _iv, _sizev = valInt((v)->size); \
     for(_iv = 0; _iv < _sizev; _iv++) \
     { val = (v)->elements[_iv]; \
       code; \
@@ -1707,10 +1707,10 @@ extern char *T_report[];		/* ->report: kind, format, args... */
   }
 
 #define for_vector_i(v, val, i, code) \
-  { int _iv, _sizev = valInt((v)->size); \
-    int _offv = valInt((v)->offset)+1; \
+  { intptr_t _iv, _sizev = valInt((v)->size); \
+    intptr_t _offv = valInt((v)->offset)+1; \
     for(_iv = 0; _iv < _sizev; _iv++) \
-    { int i = _iv + _offv; \
+    { intptr_t i = _iv + _offv; \
       val = (v)->elements[_iv]; \
       code; \
     } \
@@ -1718,7 +1718,7 @@ extern char *T_report[];		/* ->report: kind, format, args... */
 
 
 #define for_hash_table(ht, var, code) \
-  { int _iht, _sizeht = (ht)->buckets; \
+  { intptr_t _iht, _sizeht = (ht)->buckets; \
     for(_iht = 0; _iht < _sizeht; _iht++) \
     { Symbol var = &(ht)->symbols[_iht]; \
       if ( var->name != NULL ) \
