@@ -1,4 +1,4 @@
-/*  $Id: stream.h,v 1.17 2006/02/10 13:06:59 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -25,11 +25,29 @@
 #ifndef _PL_STREAM_H
 #define _PL_STREAM_H
 
+/* This appears to make the wide-character support compile and work
+   on HPUX 11.23.  There really should be a cleaner way ...
+*/
+#if defined(__hpux)
+#include <sys/_mbstate_t.h>
+#endif
+
+#if defined(_MSC_VER) && !defined(__WINDOWS__)
+#define __WINDOWS__ 1
+#endif
+
 #include <stdarg.h>
 #include <wchar.h>
-#ifdef WIN32
+#include <stddef.h>
+#ifdef __WINDOWS__
 typedef __int64 int64_t;
+#if (_MSC_VER < 1300)
+typedef long intptr_t;
+typedef unsigned long uintptr_t;
+#endif
+typedef intptr_t ssize_t;		/* signed version of size_t */
 #else
+#include <unistd.h>
 #include <inttypes.h>			/* more portable than stdint.h */
 #endif
 
@@ -48,11 +66,8 @@ stuff.
 
 #ifndef _PL_EXPORT_DONE
 #define _PL_EXPORT_DONE
-#if defined(WIN32) && !defined(__WIN32__)
-#define __WIN32__
-#endif
 
-#if (defined(__WIN32__) || defined(__CYGWIN__)) && !defined(__LCC__)
+#if (defined(__WINDOWS__) || defined(__CYGWIN__)) && !defined(__LCC__)
 #define HAVE_DECLSPEC
 #endif
 
@@ -90,8 +105,9 @@ stuff.
 #define NULL ((void *)0)
 #endif
 
-#if defined(__WIN32__) && !defined(EWOULDBLOCK)
-#define EWOULDBLOCK	1000		/* Needed for socket handling */
+#if defined(__WINDOWS__) && !defined(EWOULDBLOCK)
+#define EWOULDBLOCK	140		/* Needed for socket handling */
+					/* 140 is compatible to VS2010 */
 #endif
 #define EPLEXCEPTION	1001		/* errno: pending Prolog exception */
 
@@ -100,8 +116,8 @@ stuff.
 #define SIO_MAGIC	(7212676)	/* magic number */
 #define SIO_CMAGIC	(42)		/* we are close (and thus illegal!) */
 
-typedef int   (*Sread_function)(void *handle, char *buf, int bufsize);
-typedef int   (*Swrite_function)(void *handle, char*buf, int bufsize);
+typedef ssize_t (*Sread_function)(void *handle, char *buf, size_t bufsize);
+typedef ssize_t (*Swrite_function)(void *handle, char*buf, size_t bufsize);
 typedef long  (*Sseek_function)(void *handle, long pos, int whence);
 typedef int64_t (*Sseek64_function)(void *handle, int64_t pos, int whence);
 typedef int   (*Sclose_function)(void *handle);
@@ -120,7 +136,7 @@ typedef struct io_functions
   Sseek_function	seek;		/* seek to position */
   Sclose_function	close;		/* close stream */
   Scontrol_function	control;	/* Info/control */
-  Sseek64_function	seek64;		/* seek to position (long files) */
+  Sseek64_function	seek64;		/* seek to position (intptr_t files) */
 } IOFUNCTIONS;
 
 typedef struct io_position
@@ -128,7 +144,7 @@ typedef struct io_position
   int64_t		charno;		/* character position in file */
   int			lineno;		/* lineno in file */
   int			linepos;	/* position in line */
-  long			reserved[2];	/* future extensions */
+  intptr_t		reserved[2];	/* future extensions */
 } IOPOS;
 
 					/* NOTE: check with encoding_names */
@@ -145,6 +161,10 @@ typedef enum
   ENC_WCHAR				/* pl_wchar_t */
 } IOENC;
 
+#define SIO_NL_POSIX  0			/* newline as \n */
+#define SIO_NL_DOS    1			/* newline as \r\n */
+#define SIO_NL_DETECT 3			/* detect processing mode */
+
 typedef struct io_stream
 { char		       *bufp;		/* `here' */
   char		       *limitp;		/* read/write limit */
@@ -170,7 +190,14 @@ typedef struct io_stream
   IOENC			encoding;	/* character encoding used */
   struct io_stream *	tee;		/* copy data to this stream */
   mbstate_t *		mbstate;	/* ENC_ANSI decoding */
-  long			reserved[6];	/* reserved for extension */
+  struct io_stream *	upstream;	/* stream providing our input */
+  struct io_stream *	downstream;	/* stream providing our output */
+  unsigned		newline : 2;	/* Newline mode */
+  unsigned		erased : 1;	/* Stream was erased */
+  unsigned		references : 4;	/* Reference-count */
+  int			io_errno;	/* Save errno value */
+  void *		exception;	/* pending exception (record_t) */
+  intptr_t		reserved[2];	/* reserved for extension */
 } IOSTREAM;
 
 
@@ -229,9 +256,12 @@ PL_EXPORT_DATA(IOSTREAM)    S__iob[3];		/* Libs standard streams */
 #define Sgetchar()	Sgetc(Sinput)
 #define Sputchar(c)	Sputc((c), Soutput)
 
+#define S__checkpasteeof(s,c) \
+	if ( (c)==-1 && (s)->flags & (SIO_FEOF|SIO_FERR) ) \
+	  ((s)->flags |= SIO_FEOF2)
 #define S__updatefilepos_getc(s, c) \
 	((s)->position ? S__fupdatefilepos_getc((s), (c)) \
-		       : (c))
+		       : S__fcheckpasteeof((s), (c)))
 
 #define Snpgetc(s) ((s)->bufp < (s)->limitp ? (int)(*(s)->bufp++)&0xff \
 					    : S__fillbuf(s))
@@ -241,6 +271,8 @@ PL_EXPORT_DATA(IOSTREAM)    S__iob[3];		/* Libs standard streams */
 #define SIO_GETSIZE	(1)		/* get size of underlying object */
 #define SIO_GETFILENO	(2)		/* get underlying file (if any) */
 #define SIO_SETENCODING	(3)		/* modify encoding of stream */
+#define SIO_FLUSHOUTPUT	(4)		/* flush output */
+#define SIO_LASTERROR	(5)		/* string holding last error */
 
 /* Sread_pending() */
 #define SIO_RP_BLOCK 0x1		/* wait for new input */
@@ -307,9 +339,10 @@ PL_EXPORT_DATA(IOSTREAM)    S__iob[3];		/* Libs standard streams */
 PL_EXPORT(void)		SinitStreams();
 PL_EXPORT(void)		Scleanup(void);
 PL_EXPORT(void)		Sreset(void);
-PL_EXPORT(int)		S__fupdatefilepos(IOSTREAM *s, int c);
 PL_EXPORT(int)		S__fupdatefilepos_getc(IOSTREAM *s, int c);
+PL_EXPORT(int)		S__fcheckpasteeof(IOSTREAM *s, int c);
 PL_EXPORT(int)		S__fillbuf(IOSTREAM *s);
+PL_EXPORT(int)		Sunit_size(IOSTREAM *s);
 					/* byte I/O */
 PL_EXPORT(int)		Sputc(int c, IOSTREAM *s);
 PL_EXPORT(int)		Sfgetc(IOSTREAM *s);
@@ -318,28 +351,34 @@ PL_EXPORT(int)		Sungetc(int c, IOSTREAM *s);
 PL_EXPORT(int)		Scanrepresent(int c, IOSTREAM *s);
 PL_EXPORT(int)		Sputcode(int c, IOSTREAM *s);
 PL_EXPORT(int)		Sgetcode(IOSTREAM *s);
-PL_EXPORT(int)		Sungetcode(int c, IOSTREAM *s);
+PL_EXPORT(int)		Speekcode(IOSTREAM *s);
 					/* word I/O */
 PL_EXPORT(int)		Sputw(int w, IOSTREAM *s);
 PL_EXPORT(int)		Sgetw(IOSTREAM *s);
-PL_EXPORT(int)		Sfread(void *data, int size, int elems, IOSTREAM *s);
-PL_EXPORT(int)		Sfwrite(const void *data, int size, int elems,
+PL_EXPORT(size_t)	Sfread(void *data, size_t size, size_t elems,
+			       IOSTREAM *s);
+PL_EXPORT(size_t)	Sfwrite(const void *data, size_t size, size_t elems,
 				IOSTREAM *s);
 PL_EXPORT(int)		Sfeof(IOSTREAM *s);
 PL_EXPORT(int)		Sfpasteof(IOSTREAM *s);
 PL_EXPORT(int)		Sferror(IOSTREAM *s);
 PL_EXPORT(void)		Sclearerr(IOSTREAM *s);
 PL_EXPORT(void)		Sseterr(IOSTREAM *s, int which, const char *message);
+#ifdef _FLI_H_INCLUDED
+PL_EXPORT(void)		Sset_exception(IOSTREAM *s, term_t ex);
+#else
+PL_EXPORT(void)		Sset_exception(IOSTREAM *s, intptr_t ex);
+#endif
 PL_EXPORT(int)		Ssetenc(IOSTREAM *s, IOENC new_enc, IOENC *old_enc);
 PL_EXPORT(int)		Sflush(IOSTREAM *s);
 PL_EXPORT(long)		Ssize(IOSTREAM *s);
-PL_EXPORT(long)		Sseek(IOSTREAM *s, long pos, int whence);
+PL_EXPORT(int)		Sseek(IOSTREAM *s, long pos, int whence);
 PL_EXPORT(long)		Stell(IOSTREAM *s);
 PL_EXPORT(int)		Sclose(IOSTREAM *s);
 PL_EXPORT(char *)	Sfgets(char *buf, int n, IOSTREAM *s);
 PL_EXPORT(char *)	Sgets(char *buf);
-PL_EXPORT(int)		Sread_pending(IOSTREAM *s,
-				      char *buf, int limit, int flags);
+PL_EXPORT(ssize_t)	Sread_pending(IOSTREAM *s,
+				      char *buf, size_t limit, int flags);
 PL_EXPORT(int)		Sfputs(const char *q, IOSTREAM *s);
 PL_EXPORT(int)		Sputs(const char *q);
 PL_EXPORT(int)		Sfprintf(IOSTREAM *s, const char *fm, ...);
@@ -358,13 +397,15 @@ PL_EXPORT(IOSTREAM *)	Sopen_file(const char *path, const char *how);
 PL_EXPORT(IOSTREAM *)	Sfdopen(int fd, const char *type);
 PL_EXPORT(int)	   	Sfileno(IOSTREAM *s);
 PL_EXPORT(IOSTREAM *)	Sopen_pipe(const char *command, const char *type);
-PL_EXPORT(IOSTREAM *)	Sopenmem(char **buffer, int *sizep, const char *mode);
-PL_EXPORT(IOSTREAM *)	Sopen_string(IOSTREAM *s, char *buf, int sz, const char *m);
+PL_EXPORT(IOSTREAM *)	Sopenmem(char **buffer, size_t *sizep, const char *mode);
+PL_EXPORT(IOSTREAM *)	Sopen_string(IOSTREAM *s, char *buf, size_t sz, const char *m);
 PL_EXPORT(int)		Sclosehook(void (*hook)(IOSTREAM *s));
 PL_EXPORT(void)		Sfree(void *ptr);
+PL_EXPORT(int)		Sset_filter(IOSTREAM *parent, IOSTREAM *filter);
+PL_EXPORT(void)		Ssetbuffer(IOSTREAM *s, char *buf, size_t size);
 
 PL_EXPORT(int64_t)	Stell64(IOSTREAM *s);
-PL_EXPORT(int64_t)	Sseek64(IOSTREAM *s, int64_t pos, int whence);
+PL_EXPORT(int)		Sseek64(IOSTREAM *s, int64_t pos, int whence);
 
 PL_EXPORT(int)		ScheckBOM(IOSTREAM *s);
 PL_EXPORT(int)		SwriteBOM(IOSTREAM *s);
