@@ -3,15 +3,15 @@
 /* /home/jan/src/pl/packages/xpce/src/ker/alloc.c */
 COMMON(Any)	alloc(size_t n);
 COMMON(void)	unalloc(size_t n, Any p);
-COMMON(void)	initAlloc(void);
+COMMON(void)	pceInitAlloc(void);
 COMMON(void)	allocRange(void *low, size_t size);
-COMMON(status)	listWastedCorePce(Pce pce, Bool ppcells);
+COMMON(status)	listWastedCorePce(Pce pce, BoolObj ppcells);
 COMMON(Any)	alloc(size_t n);
 COMMON(void)	unalloc(size_t n, Any p);
 COMMON(void)	initAlloc(void);
 COMMON(void)	allocRange(void *low, size_t size);
 COMMON(void)	checkFreeChains(void);
-COMMON(status)	listWastedCorePce(Pce pce, Bool ppcells);
+COMMON(status)	listWastedCorePce(Pce pce, BoolObj ppcells);
 COMMON(char *)	save_string(const char *s);
 COMMON(void)	free_string(char *s);
 
@@ -63,7 +63,7 @@ COMMON(status)	saveStyleClass(Class class, Name style);
 COMMON(status)	setCloneFunctionClass(Class class, SendFunc function);
 COMMON(status)	setRedrawFunctionClass(Class class, SendFunc function);
 COMMON(status)	setLoadStoreFunctionClass(Class class, SendFunc load, SendFunc store);
-COMMON(status)	solidClass(Class class, Bool val);
+COMMON(status)	solidClass(Class class, BoolObj val);
 COMMON(status)	sourceClass(Class class, SendFunc f, char *file, char *rcs);
 COMMON(void)	localClass(Class class, Name name, Name group, char *type, Name access, char *doc);
 COMMON(status)	declareClass(Class class, const classdecl *decls);
@@ -73,8 +73,8 @@ COMMON(Variable) getInstanceVariableClass(Class class, Any which);
 COMMON(void)	resetMessageResolve(void);
 COMMON(Any)	getResolveSendMethodClass(Class class, Name name);
 COMMON(Any)	getResolveGetMethodClass(Class class, Name name);
-COMMON(Int)	getNoCreatedClass(Class class, Bool subtoo);
-COMMON(Int)	getNoFreedClass(Class class, Bool subtoo);
+COMMON(Int)	getNoCreatedClass(Class class, BoolObj subtoo);
+COMMON(Int)	getNoFreedClass(Class class, BoolObj subtoo);
 COMMON(status)	createdClass(Class class, Any instance, Name how);
 COMMON(status)	freedClass(Class class, Any instance);
 COMMON(status)	isAClass(Class class, Class super);
@@ -89,7 +89,7 @@ COMMON(status)	toString(Any obj, String s);
 COMMON(char *)	toCharp(Any obj);
 COMMON(Int)	toInteger(Any obj);
 COMMON(Real)	toReal(Any obj);
-COMMON(Bool)	toBool(Any obj);
+COMMON(BoolObj)	toBool(Any obj);
 COMMON(Name)	toName(Any obj);
 COMMON(Type)	toType(Any obj);
 COMMON(char *)	pcePP(Any obj);
@@ -97,7 +97,7 @@ COMMON(char *)	pcePP(Any obj);
 COMMON(Any)	expandFunction(Any obj);
 
 /* /home/jan/src/pl/packages/xpce/src/ker/debug.c */
-COMMON(void)	catchErrorSignals(Bool yes);
+COMMON(void)	catchErrorSignals(BoolObj yes);
 COMMON(status)	confirmTerminal(char *question, char *def);
 
 /* /home/jan/src/pl/packages/xpce/src/ker/declarations.c */
@@ -228,9 +228,9 @@ COMMON(Int)	getReferencesObject(Any obj);
 COMMON(Name)	getFlagsObject(Any obj);
 COMMON(status)	protectObject(Any obj);
 COMMON(status)	doneObject(Any obj);
-COMMON(status)	lockObject(Any obj, Bool val);
-COMMON(status)	inspectObject(Any obj, Bool val);
-COMMON(Bool)	getInspectObject(Any obj);
+COMMON(status)	lockObject(Any obj, BoolObj val);
+COMMON(status)	inspectObject(Any obj, BoolObj val);
+COMMON(BoolObj)	getInspectObject(Any obj);
 COMMON(Name)	getClassNameObject(Any obj);
 COMMON(Class)	getClassObject(Any obj);
 COMMON(Any)	getSelfObject(Any obj);
@@ -248,11 +248,11 @@ COMMON(Tuple)	getGetMethodObject(Any obj, Name selector);
 COMMON(status)	hasSendMethodObject(Any obj, Name selector);
 COMMON(status)	hasGetMethodObject(Any obj, Name selector);
 COMMON(Any)	getCreateContextObject(Any obj, Code cond);
-COMMON(Chain)	getAllConstraintsObject(Any obj, Bool create);
-COMMON(Chain)	getAllHypersObject(Any obj, Bool create);
-COMMON(Chain)	getAllAttributesObject(Any obj, Bool create);
-COMMON(Chain)	getAllSendMethodsObject(Any obj, Bool create);
-COMMON(Chain)	getAllGetMethodsObject(Any obj, Bool create);
+COMMON(Chain)	getAllConstraintsObject(Any obj, BoolObj create);
+COMMON(Chain)	getAllHypersObject(Any obj, BoolObj create);
+COMMON(Chain)	getAllAttributesObject(Any obj, BoolObj create);
+COMMON(Chain)	getAllSendMethodsObject(Any obj, BoolObj create);
+COMMON(Chain)	getAllGetMethodsObject(Any obj, BoolObj create);
 COMMON(Any)	getCloneObject(Any obj);
 COMMON(status)	clonePceSlots(Any org, Any Clone);
 COMMON(Any)	getClone2Object(Any obj);
@@ -273,10 +273,12 @@ COMMON(Any)	getClassVariableValueObject(Any obj, Name name);
 COMMON(status)	obtainClassVariablesObject(Any obj);
 COMMON(status)	convertLoadedObjectObject(Any obj, Int oldversion, Int currentversion);
 COMMON(Any)	getConvertObject(Any ctx, Any x);
-COMMON(status)	CheckObject(Any obj, Bool recursive);
+COMMON(status)	CheckObject(Any obj, BoolObj recursive);
 COMMON(status)	errorObjectv(Any obj, Error e, int argc, Any *argv);
 COMMON(Name)	getManIdObject(Any obj);
 COMMON(status)	makeClassObject(Class class);
+COMMON(status)	printReportObject(Any obj, Name kind,
+				  CharArray fmt, int argc, Any *argv);
 
 /* /home/jan/src/pl/packages/xpce/src/ker/programobject.c */
 COMMON(status)	initialiseProgramObject(Any obj);
@@ -306,12 +308,12 @@ COMMON(status)	catchErrorPce(Pce pce, Any ids);
 COMMON(status)	catchPopPce(Pce pce);
 COMMON(status)	catchedErrorPce(Pce pce, Name id);
 COMMON(Name)	getEnvironmentVariablePce(Pce pce, Name name);
-COMMON(status)	debuggingPce(Pce pce, Bool val);
+COMMON(status)	debuggingPce(Pce pce, BoolObj val);
 COMMON(status)	exceptionPcev(Pce pce, Name name, int argc, Any *argv);
 COMMON(status)	exceptionPce(Pce pce, Name kind, ...);
 COMMON(int)	getFileDesCount(void);
 COMMON(Name)	getHostnamePce(Pce pce);
-COMMON(status)	catchErrorSignalsPce(Pce pce, Bool val);
+COMMON(status)	catchErrorSignalsPce(Pce pce, BoolObj val);
 COMMON(status)	resetPce(Pce pce);
 COMMON(Any)	getObjectFromReferencePce(Pce pce, Any ref);
 COMMON(status)	makeClassPce(Class class);
@@ -345,7 +347,7 @@ COMMON(Type)	getLookupType(Class class, Name name);
 COMMON(Type)	createType(Name name, Name kind, Any context);
 COMMON(Name)	getNameType(Type t);
 COMMON(void)	superType(Type t, Type t2);
-COMMON(status)	vectorType(Type t, Bool val);
+COMMON(status)	vectorType(Type t, BoolObj val);
 COMMON(status)	isClassType(Type t);
 COMMON(status)	specialisedType(Type t1, Type t2);
 COMMON(status)	equalType(Type t1, Type t2);
