@@ -181,7 +181,7 @@ superType(Type t, Type t2)
 
 
 status
-vectorType(Type t, Bool val)
+vectorType(Type t, BoolObj val)
 { assign(t, vector, val);
 
   succeed;
@@ -1398,7 +1398,8 @@ nameToType(Name name)
   if ( (type = getMemberHashTable(TypeTable, name)) )
     return type;
 
-  init_string(&str, &name->data);
+  if ( !init_string(&str, &name->data) )
+    fail;
 
   if ( (type = named_type(&str)) )
     return type;
