@@ -37,9 +37,7 @@
 #include "zutil.h"
 #endif
 
-static functor_t FUNCTOR_error2;	/* error(Formal, Context) */
-static functor_t FUNCTOR_type_error2;	/* type_error(Term, Expected) */
-static functor_t FUNCTOR_domain_error2;	/* domain_error(Term, Expected) */
+install_t install_zlib4pl(void);
 
 static atom_t ATOM_format;		/* format(Format) */
 static atom_t ATOM_level;		/* level(Int) */
@@ -54,75 +52,6 @@ static int debuglevel = 0;
 #define DEBUG(n, g) (void)0
 #endif
 
-		 /*******************************
-		 *	       ERRORS		*
-		 *******************************/
-
-static int
-type_error(term_t actual, const char *expected)
-{ term_t ex = PL_new_term_ref();
-
-  PL_unify_term(ex, PL_FUNCTOR, FUNCTOR_error2,
-		      PL_FUNCTOR, FUNCTOR_type_error2,
-		        PL_CHARS, expected,
-		        PL_TERM, actual,
-		      PL_VARIABLE);
-
-  return PL_raise_exception(ex);
-}
-
-
-static int
-domain_error(term_t actual, const char *domain)
-{ term_t ex = PL_new_term_ref();
-
-  PL_unify_term(ex, PL_FUNCTOR, FUNCTOR_error2,
-		      PL_FUNCTOR, FUNCTOR_domain_error2,
-		        PL_CHARS, domain,
-		        PL_TERM, actual,
-		      PL_VARIABLE);
-
-  return PL_raise_exception(ex);
-}
-
-
-static int
-instantiation_error()
-{ term_t ex = PL_new_term_ref();
-
-  PL_unify_term(ex, PL_FUNCTOR, FUNCTOR_error2,
-		      PL_CHARS, "inistantiation_error",
-		      PL_VARIABLE);
-
-  return PL_raise_exception(ex);
-}
-
-
-static int
-get_atom_ex(term_t t, atom_t *a)
-{ if ( PL_get_atom(t, a) )
-    return TRUE;
-
-  return type_error(t, "atom");
-}
-
-static int
-get_int_ex(term_t t, int *i)
-{ if ( PL_get_integer(t, i) )
-    return TRUE;
-
-  return type_error(t, "integer");
-}
-
-static int
-get_bool_ex(term_t t, int *i)
-{ if ( PL_get_bool(t, i) )
-    return TRUE;
-
-  return type_error(t, "boolean");
-}
-
-
 		 /*******************************
 		 *	       TYPES		*
 		 *******************************/
@@ -336,7 +265,8 @@ gz_skip_footer(z_context *ctx)
       Sseterr(ctx->zstream, SIO_FERR, msg);
       return -1;
     }
-    if ( size != ctx->zstate.total_out )
+					/* size is truncated to 4 bytes */
+    if ( size != (ctx->zstate.total_out & 0xffffffff) )
     { char msg[256];
 
       Ssprintf(msg, "Size mismatch (%ld != %ld)", size, ctx->zstate.total_out);
@@ -362,18 +292,23 @@ called with data in the buffer. It moves the remaining data to the start
 of the stream buffer and tries to read more data into the stream.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+static void
+sync_stream(z_context *ctx)
+{ ctx->stream->bufp   = (char*)ctx->zstate.next_in;
+}
+
 static int
 read_more(z_context *ctx)
 { int c;
 
-  ctx->stream->bufp   = (char*)ctx->zstate.next_in;
+  DEBUG(1, Sdprintf("Short input, read_more()\n"));
+
   ctx->stream->limitp =	ctx->stream->bufp + ctx->zstate.avail_in;
 
   if ( (c=S__fillbuf(ctx->stream)) != EOF )
   { Sungetc(c, ctx->stream);
     ctx->zstate.next_in  = (Bytef*)ctx->stream->bufp;
     ctx->zstate.avail_in = (long)(ctx->stream->limitp - ctx->stream->bufp);
-    ctx->stream->bufp    = ctx->stream->limitp;
 
     return 0;
   }
@@ -394,7 +329,7 @@ zread(void *handle, char *buf, size_t size)
     } else
     { ctx->zstate.next_in  = (Bytef*)ctx->stream->bufp;
       ctx->zstate.avail_in = (long)(ctx->stream->limitp - ctx->stream->bufp);
-      ctx->stream->bufp    = ctx->stream->limitp; /* empty buffer */
+      DEBUG(1, Sdprintf("Set avail_in to %d\n", ctx->zstate.avail_in));
     }
   }
 
@@ -425,9 +360,13 @@ zread(void *handle, char *buf, size_t size)
       DEBUG(1, Sdprintf("Skipped gzip header (%d bytes)\n", m));
       ctx->zstate.next_in = p;
       ctx->zstate.avail_in -= m;
+      sync_stream(ctx);
 
 					/* init without header */
-      switch(inflateInit2(&ctx->zstate, -MAX_WBITS))
+      rc = inflateInit2(&ctx->zstate, -MAX_WBITS);
+      sync_stream(ctx);
+
+      switch( rc )
       { case Z_OK:
 	  ctx->initialized = TRUE;
 	  ctx->crc = crc32(0L, Z_NULL, 0);
@@ -442,7 +381,10 @@ zread(void *handle, char *buf, size_t size)
 	  return -1;
       }
     } else
-    { switch(inflateInit(&ctx->zstate))
+    { rc = inflateInit(&ctx->zstate);
+      sync_stream(ctx);
+
+      switch( rc )
       { case Z_OK:
 	  ctx->format = F_DEFLATE;
 	  ctx->initialized = TRUE;
@@ -463,27 +405,25 @@ zread(void *handle, char *buf, size_t size)
     while( (rc=gz_skip_footer(ctx)) == -2 )
     { int rc2;
 
+      sync_stream(ctx);
       if ( (rc2=read_more(ctx)) < 0 )
 	return -1;
     }
 
+    sync_stream(ctx);
+
     if ( rc == 0 )
-    { int avail = ctx->zstate.avail_in;
-
-					/* copy back unprocessed bytes */
-      DEBUG(1, Sdprintf("GZIP footer ok; copying %d bytes back\n", avail));
-      memmove(ctx->stream->buffer, ctx->zstate.next_in, avail);
-      ctx->stream->bufp   = ctx->stream->buffer;
-      ctx->stream->limitp = ctx->stream->bufp + avail;
-
-      return 0;			/* EOF */
+    { return 0;			/* EOF */
     } else
     { DEBUG(1, Sdprintf("GZIP CRC/length error\n"));
       return -1;
     }
   }
 
-  switch((rc=inflate(&ctx->zstate, Z_NO_FLUSH)))
+  rc = inflate(&ctx->zstate, Z_NO_FLUSH);
+  sync_stream(ctx);
+
+  switch( rc )
   { case Z_OK:
     case Z_STREAM_END:
     { long n = (long)(size - ctx->zstate.avail_out);
@@ -498,6 +438,14 @@ zread(void *handle, char *buf, size_t size)
 	  ctx->format = F_GZIP_CRC;
       } else
       { DEBUG(1, Sdprintf("inflate(): Z_OK: %d bytes\n", n));
+	if (n == 0 && rc != Z_STREAM_END)
+        { /* If we get here then there was not enough data in the in buffer to decode
+             a single character, but we are not at the end of the stream, so we must read
+	     more from the parent */
+          DEBUG(1, Sdprintf("Not enough data to decode.  Retrying\n"));
+
+          return zread(handle, buf, size);
+	}
       }
 
       return n;
@@ -683,32 +631,32 @@ pl_zopen(term_t org, term_t new, term_t options)
     term_t arg = PL_new_term_ref();
 
     if ( !PL_get_name_arity(head, &name, &arity) || arity != 1 )
-      return type_error(head, "option");
-    PL_get_arg(1, head, arg);
+      return PL_type_error("option", head);
+    _PL_get_arg(1, head, arg);
 
     if ( name == ATOM_format )
     { atom_t a;
 
-      if ( !get_atom_ex(arg, &a) )
+      if ( !PL_get_atom_ex(arg, &a) )
 	return FALSE;
       if ( a == ATOM_gzip )
 	fmt = F_GZIP;
       else if ( a == ATOM_deflate )
 	fmt = F_DEFLATE;
       else
-	return domain_error(arg, "compression_format");
+	return PL_domain_error("compression_format", arg);
     } else if ( name == ATOM_level )
-    { if ( !get_int_ex(arg, &level) )
+    { if ( !PL_get_integer_ex(arg, &level) )
 	return FALSE;
       if ( level < 0 || level > 9 )
-	return domain_error(arg, "compression_level");
+	return PL_domain_error("compression_level", arg);
     } else if ( name == ATOM_close_parent )
-    { if ( !get_bool_ex(arg, &close_parent) )
+    { if ( !PL_get_bool_ex(arg, &close_parent) )
 	return FALSE;
     }
   }
-  if ( !PL_get_nil(tail) )
-    return type_error(tail, "list");
+  if ( !PL_get_nil_ex(tail) )
+    return FALSE;
 
   if ( !PL_get_stream_handle(org, &s) )
     return FALSE;			/* Error */
@@ -751,7 +699,7 @@ pl_zopen(term_t org, term_t new, term_t options)
   } else
   { ctx->close_parent = FALSE;
     Sclose(s2);
-    return instantiation_error();
+    return PL_instantiation_error(new);	/* actually, over instantiation */
   }
 }
 
@@ -770,12 +718,8 @@ zdebug(term_t level)
 #define MKFUNCTOR(name, arity) PL_new_functor(PL_new_atom(name), arity)
 
 install_t
-install_zlib4pl()
-{ FUNCTOR_error2        = MKFUNCTOR("error", 2);
-  FUNCTOR_type_error2   = MKFUNCTOR("type_error", 2);
-  FUNCTOR_domain_error2 = MKFUNCTOR("domain_error", 2);
-
-  ATOM_format       = PL_new_atom("format");
+install_zlib4pl(void)
+{ ATOM_format       = PL_new_atom("format");
   ATOM_level        = PL_new_atom("level");
   ATOM_close_parent = PL_new_atom("close_parent");
   ATOM_gzip	    = PL_new_atom("gzip");
