@@ -10,7 +10,7 @@
 %	atoms XXX<N> over the queue and   validate them at the receiving
 %	end. If atom-gc picks one of them   we will not receive a proper
 %	sequence.
-%	
+%
 %	For testing purposes we use a   rather  small number of messages
 %	and frequent atom-gc.
 
@@ -28,8 +28,9 @@ test(N) :-
 		   thread_send_message(Q, A)
 	       )),
 	thread_join(Id, Exit),
-	assertion(Exit == true).
-	
+	assertion(Exit == true),
+	message_queue_destroy(Q).
+
 eat(Q, I, N) :-
 	thread_get_message(Q, A),
 	atom_concat('XXX', NA, A),
