@@ -39,32 +39,32 @@
 :- use_module(library('unicode/unicode_data'), [unicode_property/2]).
 :- use_module(derived_core_properties, [unicode_derived_core_property/2]).
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+/** <module> Generate Prolog Unicode map
+
 Create a C structure and  access   functions  for  classification of the
 characters we need  for  realising  the   Prolog  syntax.  We  keep  the
 definition of the first 128  ASCII   characters.  Characters  above that
 needs to be classified as
 
-	# id_start (csymf)
+	* id_start (csymf)
 	May start an identifier.
 
-	# id_continue (csym)
+	* id_continue (csym)
 	May be used anywhere in identifier
 
-	# uppercase
+	* uppercase
 	We need this to be able to distinquish variables from non-variables.
 
-	# Separators
+	* Separators
 	We need this for classifying blank space
 
-	# lowercase
+	* Symbols
+	Characters that glue together to form symbols.  These extend the
+	default Prolog symbol set: #$&*+-./:<=>?@\^`~
+
+	* lowercase
 	<not needed by Prolog>
-
-
-
-Prolog symbols:
-	#$&*+-./:<=>?@\^`~
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+*/
 
 :- multifile
 	user:file_search_path/2.
@@ -75,19 +75,21 @@ user:file_search_path(unicode, '.').
 main :-
 	write_syntax_map('../pl-umap.c', []).
 
+last_unicode_page(LastPage) :-
+	LastPage is (0x10ffff + 1) // 0x100.
 
 		 /*******************************
 		 *	     C TABLES		*
 		 *******************************/
 
-%	write_unicode_collate_map(+File, +Options)
+%%	write_syntax_map(+File, +Options)
 %
 %	Options supported are:
 %
 %		# first_codepage [0]
 %		Code page to start
 %
-%		# last_codepage [255]
+%		# last_codepage [last_unicode_page/1]
 %		Code page to end.
 
 write_syntax_map(File, Options) :-
@@ -113,7 +115,7 @@ write_codepage(Out, CP, Map) :-
 	format(Out, '~N};~n~n', []).
 
 cp_name(CP, CPN) :-
-	sformat(CPN, 'ucp0x~|~`0t~16r~2+', [CP]).
+	format(atom(CPN), 'ucp0x~|~`0t~16r~2+', [CP]).
 
 map_entries([], _, _).
 map_entries([H|T], I, Out) :-
@@ -128,7 +130,8 @@ map_entries([H|T], I, Out) :-
 	map_entries(T, I2, Out).
 
 write_map(Out, Tables, Options) :-
-	option(last_codepage(Last), Options, 255),
+	last_unicode_page(DefLast),
+	option(last_codepage(Last), Options, DefLast),
 	format(Out,
 	       'static const char* const uflags_map[UNICODE_MAP_SIZE] =~n',
 	       []),
@@ -149,14 +152,15 @@ map_tables(CP, Last, Tables, Out) :-
 	(   is_list(Map)
 	->  cp_name(CP, CPN),
 	    format(Out, '~w', [CPN])
-	;   format(Out, '~|~tF(~16r)~7+', [Map])
+	;   format(Out, '~|~tF(0x~16r)~7+', [Map])
 	),
 	CP2 is CP + 1,
 	map_tables(CP2, Last, Tables, Out).
 
 
 write_header(Out, Options) :-
-	option(last_codepage(Last), Options, 255),
+	last_unicode_page(DefLast),
+	option(last_codepage(Last), Options, DefLast),
 	Size is Last+1,
 	format(Out, '/*  Generated file.  Do not edit!\n    \
 		         Generated by Unicode/prolog_syntax_map.pl\n\
@@ -180,23 +184,18 @@ write_footer(Out, _Options) :-
 		  if ( cp < UNICODE_MAP_SIZE )\n  \
 		  { const char *s = uflags_map[cp];\n    \
 		    if ( s < (const char *)256 )\n      \
-		      return (int)s;\n    \
+		      return (int)(intptr_t)s;\n    \
 		    return s[code&0xff];\n  \
 		  }\n  \
 		  return 0;\n\
 		}\n\n', []).
-%	format(Out,
-%	       'static int\n\
-%		uflagsA(int code)\n\
-%		{ return ucp0x00[code&0xff];\n\
-%		}\n\n', []).
 
 
 		 /*******************************
 		 *	       TABLES		*
 		 *******************************/
 
-%	gen_tables(-Tables, +Options)
+%%	gen_tables(-Tables, +Options)
 %
 %	Table is of  the  format  below,   where  CodePage  is  the page
 %	(0..255) for 16-bit Unicode and  ValueList   are  the values for
@@ -208,8 +207,9 @@ gen_tables(Tables, Options) :-
 	findall(table(CP,Map), table(CP, Map, Options), Tables).
 
 table(CP, Map, Options) :-
+	last_unicode_page(DefPage),
 	option(first_codepage(First), Options, 0),
-	option(last_codepage(Last), Options, 255),
+	option(last_codepage(Last), Options, DefPage),
 	between(First, Last, CP),
 	findall(M, char(CP, M, Options), Map0),
 	flat_map(Map0, Map).
@@ -229,26 +229,62 @@ or([H|T], F) :-
 	F is F0 \/ H.
 
 flag(Code, Flag) :-
-	code_flag(Code, Name),
-	flag_name(Name, Flag).
+	flag_name(Name, Flag),
+	code_flag(Code, Name).
 
 flag_name(id_start,    0x01).
 flag_name(id_continue, 0x02).
 flag_name(uppercase,   0x04).
-flag_name(lowercase,   0x08).
-flag_name(separator,   0x10).
+flag_name(separator,   0x08).
+flag_name(symbol,      0x10).
+flag_name(other,       0x20).
+flag_name(control,     0x40).
 
 code_flag(C, id_start) :-    unicode_derived_core_property(C, id_start).
 code_flag(C, id_continue) :- unicode_derived_core_property(C, id_continue).
 code_flag(C, uppercase) :-   unicode_derived_core_property(C, uppercase).
-code_flag(C, lowercase) :-   unicode_derived_core_property(C, lowercase).
 code_flag(C, separator) :-
 	unicode_property(C, general_category(Cat)),
 	sep_cat(Cat).
+code_flag(C, symbol) :-
+	unicode_property(C, general_category(Cat)),
+	symbol_cat(Cat).
+code_flag(C, other) :-
+	unicode_property(C, general_category(Cat)),
+	other_cat(Cat).
+code_flag(C, control) :-
+	unicode_property(C, general_category(Cat)),
+	control_cat(Cat).
+code_flag(C, unassigned) :-
+	\+ unicode_property(C, general_category(_)).
+
+% See http://www.unicode.org/reports/tr44/#Property_Values
+
+sep_cat('Zs').		% a space character (of various non-zero widths)
+sep_cat('Zl').		% U+2028 LINE SEPARATOR only
+sep_cat('Zp').		% U+2029 PARAGRAPH SEPARATOR only
+
+symbol_cat('Sm').	% a symbol of primarily mathematical use
+symbol_cat('Sc').	% a currency sign
+symbol_cat('Sk').	% a non-letterlike modifier symbol
+symbol_cat('So').	% a symbol of other type
+symbol_cat('Pc').	% a connecting punctuation mark, like a tie
+symbol_cat('Pd').	% a dash or hyphen punctuation mark
+symbol_cat('Ps').	% an opening punctuation mark (of a pair)
+symbol_cat('Pe').	% a closing punctuation mark (of a pair)
+symbol_cat('Pi').	% an initial quotation mark
+symbol_cat('Pf').	% a final quotation mark
+symbol_cat('Po').	% a punctuation mark of other type
+
+other_cat('No').	% a numeric character of other type
+other_cat('Me').	% an enclosing combining mark
+
+control_cat('Cc').	% a C0 or C1 control code
+control_cat('Cf').	% a format control character
+control_cat('Cs').	% a surrogate code point
+control_cat('Co').	% a private-use character
+control_cat('Cn').	% a reserved unassigned code point or a noncharacter
 
-sep_cat('Zs').
-sep_cat('Zl').
-sep_cat('Zp').
 
 flat_map(Map0, Value) :-
 	sort(Map0, [Value]), !.
