@@ -1187,92 +1187,6 @@ equalIndirectFromCode(word a, Code *PC)
 }
 
 
-		/********************************
-		*            STRINGS            *
-		*********************************/
-
-
-#ifdef O_DEBUG
-#define CHAR_INUSE 0x42
-#define CHAR_FREED 0x41
-
-char *
-store_string(const char *s)
-{ if ( s )
-  { GET_LD
-    char *copy = (char *)allocHeap(strlen(s)+2);
-
-    *copy++ = CHAR_INUSE;
-    strcpy(copy, s);
-
-    return copy;
-  } else
-  { return NULL;
-  }
-}
-
-
-void
-remove_string(char *s)
-{ if ( s )
-  { GET_LD
-    assert(s[-1] == CHAR_INUSE);
-
-    s[-1] = CHAR_FREED;
-    freeHeap(s-1, strlen(s)+2);
-  }
-}
-
-#else /*O_DEBUG*/
-
-char *
-store_string(const char *s)
-{ if ( s )
-  { GET_LD
-
-    char *copy = (char *)allocHeap(strlen(s)+1);
-
-    strcpy(copy, s);
-    return copy;
-  } else
-  { return NULL;
-  }
-}
-
-
-void
-remove_string(char *s)
-{ if ( s )
-  { GET_LD
-    freeHeap(s, strlen(s)+1);
-  }
-}
-
-#endif /*O_DEBUG*/
-
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Hash function for strings.  This function has been evaluated on Shelley,
-which defines about 5000 Prolog atoms.  It produces a very nice  uniform
-distribution over these atoms.  Note that size equals 2^n.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-int
-unboundStringHashValue(const char *t, size_t len)
-{ unsigned int value = 0;
-  unsigned int shift = 5;
-
-  while(len-- != 0)
-  { unsigned int c = *t++;
-
-    c -= 'a';
-    value ^= c << (shift & 0xf);
-    shift ^= c;
-  }
-
-  return value ^ (value >> 16);
-}
-
-
 		 /*******************************
 		 *	     GNU MALLOC		*
 		 *******************************/
