@@ -3,9 +3,10 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (C): 1985-2011, University of Amsterdam
+			      VU University Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -24,7 +25,7 @@
 
 /*#define O_DEBUG 1*/
 #include "pl-incl.h"
-#include "pl-ctype.h"
+#include "os/pl-ctype.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Implementation issues
@@ -353,10 +354,9 @@ lookupBlob(const char *s, size_t length, PL_blob_t *type, int *new)
 
   if ( !type->registered )		/* avoid deadlock */
     PL_register_blob_type(type);
-
-  startCritical;
-  LOCK();
   v0 = MurmurHashAligned2(s, length, MURMUR_SEED);
+
+  LOCK();
   v  = v0 & (atom_buckets-1);
   DEBUG(0, lookups++);
 
@@ -370,10 +370,11 @@ lookupBlob(const char *s, size_t length, PL_blob_t *type, int *new)
 	{
 #ifdef O_ATOMGC
 	  if ( indexAtom(a->atom) >= GD->atoms.builtin )
-	    a->references++;
+	  { if ( a->references++ == 0 )
+	      GD->atoms.unregistered--;
+	  }
 #endif
           UNLOCK();
-	  endCritical;
 	  *new = FALSE;
 	  return a->atom;
 	}
@@ -387,10 +388,10 @@ lookupBlob(const char *s, size_t length, PL_blob_t *type, int *new)
 	     s == a->name )
 	{
 #ifdef O_ATOMGC
-	  a->references++;
+	  if ( a->references++ == 0 )
+	    GD->atoms.unregistered--;
 #endif
           UNLOCK();
-	  endCritical;
 	  *new = FALSE;
 	  return a->atom;
 	}
@@ -424,8 +425,8 @@ lookupBlob(const char *s, size_t length, PL_blob_t *type, int *new)
 
 #ifdef O_ATOMGC
   if ( GD->atoms.margin != 0 &&
-       GD->statistics.atoms >= GD->atoms.non_garbage + GD->atoms.margin )
-  { intptr_t x = GD->statistics.atoms - (GD->atoms.non_garbage + GD->atoms.margin);
+       GD->atoms.unregistered >= GD->atoms.non_garbage + GD->atoms.margin )
+  { intptr_t x = GD->atoms.unregistered - (GD->atoms.non_garbage + GD->atoms.margin);
 
     if ( x % 128 == 0 )			/* see (*) above */
       PL_raise(SIG_ATOM_GC);
@@ -442,7 +443,6 @@ lookupBlob(const char *s, size_t length, PL_blob_t *type, int *new)
   *new = TRUE;
   if ( type->acquire )
     (*type->acquire)(a->atom);
-  endCritical;
 
   return a->atom;
 }
@@ -555,8 +555,8 @@ atoms each pass.
 
 void
 lockAtoms()
-{ GD->atoms.builtin     = GD->atoms.count;
-  GD->atoms.non_garbage = GD->atoms.builtin;
+{ GD->atoms.builtin      = GD->atoms.count;
+  GD->atoms.unregistered = 0;
 }
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -593,17 +593,17 @@ atoms that should *not* be subject to AGC.   If we find one collected we
 know we trapped a bug.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static void
+static int
 destroyAtom(Atom *ap, uintptr_t mask ARG_LD)
 { Atom a = *ap;
   Atom *ap2 = &atomTable[a->hash_value & mask];
 
   if ( a->type->release )
   { if ( !(*a->type->release)(a->atom) )
-      return;
+      return FALSE;
   } else if ( GD->atoms.gc_hook )
   { if ( !(*GD->atoms.gc_hook)(a->atom) )
-      return;				/* foreign hooks says `no' */
+      return FALSE;				/* foreign hooks says `no' */
   }
 
 #if 0
@@ -630,16 +630,15 @@ destroyAtom(Atom *ap, uintptr_t mask ARG_LD)
   }
 
   *ap = NULL;			/* delete from index array */
-  GD->atoms.collected++;
-  GD->statistics.atoms--;
-
   if ( false(a->type, PL_BLOB_NOCOPY) )
     freeHeap(a->name, a->length+1);
   freeHeap(a, sizeof(*a));
+
+  return TRUE;
 }
 
 
-static void
+static size_t
 collectAtoms(void)
 { GET_LD
   Atom *ap0 = GD->atoms.array;
@@ -647,6 +646,7 @@ collectAtoms(void)
   Atom *ep  = ap0 + GD->atoms.count;
   int hole_seen = FALSE;
   uintptr_t mask = atom_buckets-1;
+  size_t reclaimed = 0;
 
   ap--;
   while(++ap < ep)
@@ -661,15 +661,19 @@ collectAtoms(void)
     }
 
     if ( a->references == 0 )
-    { destroyAtom(ap, mask PASS_LD);
-      if ( !hole_seen && *ap == NULL )
-      { hole_seen = TRUE;
-	GD->atoms.no_hole_before = ap-ap0;
+    { if ( destroyAtom(ap, mask PASS_LD) )
+      { reclaimed++;
+	if ( !hole_seen && *ap == NULL )
+	{ hole_seen = TRUE;
+	  GD->atoms.no_hole_before = ap-ap0;
+	}
       }
     } else
     { a->references &= ~ATOM_MARKED_REFERENCE;
     }
   }
+
+  return reclaimed;
 }
 
 
@@ -687,6 +691,7 @@ pl_garbage_collect_atoms()
   int verbose;
   double t;
   sigset_t set;
+  size_t reclaimed;
 
   PL_LOCK(L_GC);
   if ( gc_status.blocked )		/* Tricky things; avoid problems. */
@@ -732,8 +737,11 @@ pl_garbage_collect_atoms()
 #endif
   oldcollected = GD->atoms.collected;
   oldheap = GD->statistics.heap;
-  collectAtoms();
-  GD->atoms.non_garbage = GD->statistics.atoms;
+  reclaimed = collectAtoms();
+  GD->atoms.collected += reclaimed;
+  GD->statistics.atoms -= reclaimed;
+  GD->atoms.unregistered -= reclaimed;
+  GD->atoms.non_garbage = GD->atoms.unregistered;
   t = CpuTime(CPU_USER) - t;
   GD->atoms.gc_time += t;
   GD->atoms.gc++;
@@ -790,7 +798,8 @@ PL_register_atom(atom_t a)
 
     LOCK();
     p = GD->atoms.array[index];
-    p->references++;
+    if ( p->references++ == 0 )
+      GD->atoms.unregistered--;
     UNLOCK();
   }
 #endif
@@ -807,7 +816,8 @@ PL_unregister_atom(atom_t a)
 
     LOCK();
     p = GD->atoms.array[index];
-    p->references--;
+    if ( --p->references == 0 )
+      GD->atoms.unregistered++;
     if ( p->references == (unsigned)-1 )
     { Sdprintf("OOPS: -1 references to '%s'\n", p->name);
       trap_gdb();
@@ -1017,6 +1027,24 @@ PRED_IMPL("current_atom", 1, current_atom, PL_FA_NONDETERMINISTIC)
   return current_blob(A1, 0, CTX_CNTRL, CTX_INT PASS_LD);
 }
 
+
+/** blob(@Term, ?Type) is semidet
+
+Type-test for a blob.
+*/
+
+static
+PRED_IMPL("blob", 2, blob, 0)
+{ PRED_LD
+  PL_blob_t *bt;
+
+  if ( PL_is_blob(A1, &bt) )
+    return PL_unify_atom(A2, bt->atom_name);
+
+  return FALSE;
+}
+
+
 static
 PRED_IMPL("$atom_references", 2, atom_references, 0)
 { PRED_LD
@@ -1094,7 +1122,7 @@ pl_complete_atom(term_t prefix, term_t common, term_t unique)
   char buf[LINESIZ];
   char cmm[LINESIZ];
 
-  if ( !PL_get_chars_ex(prefix, &p, CVT_ALL) )
+  if ( !PL_get_chars(prefix, &p, CVT_ALL|CVT_EXCEPTION) )
     fail;
   strcpy(buf, p);
 
@@ -1160,7 +1188,7 @@ pl_atom_completions(term_t prefix, term_t alternatives)
   term_t alts = PL_copy_term_ref(alternatives);
   term_t head = PL_new_term_ref();
 
-  if ( !PL_get_chars_ex(prefix, &p, CVT_ALL) )
+  if ( !PL_get_chars(prefix, &p, CVT_ALL|CVT_EXCEPTION) )
     fail;
   strcpy(buf, p);
 
@@ -1333,5 +1361,6 @@ PL_atom_generator_w(const pl_wchar_t *prefix,
 BeginPredDefs(atom)
   PRED_DEF("current_blob",  2, current_blob, PL_FA_NONDETERMINISTIC)
   PRED_DEF("current_atom", 1, current_atom, PL_FA_NONDETERMINISTIC)
+  PRED_DEF("blob", 2, blob, 0)
   PRED_DEF("$atom_references", 2, atom_references, 0)
 EndPredDefs
