@@ -61,7 +61,7 @@ for -DMD="config/win64.h"
 #endif
 
 #include "pl-mutex.h"
-#include "SWI-Stream.h"
+#include "os/SWI-Stream.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Symbols are local to shared objects  by   default  in  COFF based binary
@@ -290,7 +290,7 @@ EndPredDefs
 #define CTX_ARITY PL__ac
 
 #define BeginPredDefs(id) \
-        PL_extension PL_predicates_from_ ## id[] = {
+        const PL_extension PL_predicates_from_ ## id[] = {
 #define PRED_DEF(name, arity, fname, flags) \
         { name, arity, pl_ ## fname ## arity ## _va, (flags)|PL_FA_VARARGS },
 #define PRED_SHARE(name, arity, fname, flags) \
