@@ -111,7 +111,7 @@ definitions below. Also the tagtypeex[] array defined in pl-setup.c must
 be kept consistent.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#include "pl-buffer.h"
+#include "os/pl-buffer.h"
 
 #define LMASK_BITS	7		/* total # mask bits */
 
@@ -206,7 +206,7 @@ and while loading .wic files.  It comes at no price.
 #define deRef2(p, d)	{ (d) = (p); deRef(d); }
 #define makeRefL(p)	consPtr(p, TAG_REFERENCE|STG_LOCAL)
 #define makeRefG(p)	consPtr(p, TAG_REFERENCE|STG_GLOBAL)
-#define makeRef(p)	((p) >= (Word)lBase ? makeRefL(p) : makeRefG(p))
+#define makeRef(p)	((void*)(p) >= (void*)lBase ? makeRefL(p) : makeRefG(p))
 #ifdef O_ATTVAR
 #define needsRef(w)	(tag(w) <= TAG_ATTVAR)
 #else
