@@ -25,8 +25,9 @@
 /*#define O_SECURE 1*/
 /*#define O_DEBUG 1*/
 #include "pl-incl.h"
-#include "pl-ctype.h"
-#include "pl-utf8.h"
+#include "os/pl-ctype.h"
+#include "os/pl-utf8.h"
+#include "pl-codelist.h"
 #include <errno.h>
 
 #include <limits.h>
@@ -73,16 +74,6 @@ Prolog int) is used by the garbage collector to update the stack frames.
 #endif
 #define valHandleP(h)		valTermRef(h)
 
-static inline word
-valHandle__LD(term_t r ARG_LD)
-{ Word p = valTermRef(r);
-
-  deRef(p);
-  return *p;
-}
-
-#define valHandle(r) valHandle__LD(r PASS_LD)
-
 static int	PL_unify_int64__LD(term_t t, int64_t i, int ex ARG_LD);
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -626,6 +617,23 @@ PL_atom_wchars(atom_t a, size_t *len)
 }
 
 
+int
+charCode(word w)
+{ if ( isAtom(w) )
+  { Atom a = atomValue(w);
+
+    if ( a->length == 1 && true(a->type, PL_BLOB_TEXT) )
+      return a->name[0] & 0xff;
+    if ( a->length == sizeof(pl_wchar_t) && a->type == &ucs_atom )
+    { pl_wchar_t *p = (pl_wchar_t*)a->name;
+
+      return p[0];
+    }
+  }
+
+  return -1;
+}
+
 
 		 /*******************************
 		 *    QUINTUS/SICSTUS WRAPPER   *
@@ -783,7 +791,7 @@ PL_cvt_o_address(void *address, term_t p)
 		 *	      COMPARE		*
 		 *******************************/
 
-int
+int					/* TBD: how to report error? */
 PL_compare(term_t t1, term_t t2)
 { GET_LD
   Word p1 = valHandleP(t1);
@@ -1103,154 +1111,13 @@ PL_get_string(term_t t, char **s, size_t *len)
 }
 #endif
 
-#define discardable_buffer 	(LD->fli._discardable_buffer)
-#define buffer_ring		(LD->fli._buffer_ring)
-#define current_buffer_id	(LD->fli._current_buffer_id)
-
-Buffer
-findBuffer(int flags)
-{ GET_LD
-  Buffer b;
-
-  if ( flags & BUF_RING )
-  { if ( ++current_buffer_id == BUFFER_RING_SIZE )
-      current_buffer_id = 0;
-    b = &buffer_ring[current_buffer_id];
-  } else
-    b = &discardable_buffer;
-
-  if ( !b->base )
-    initBuffer(b);
-
-  emptyBuffer(b);
-  return b;
-}
-
-
-char *
-buffer_string(const char *s, int flags)
-{ Buffer b = findBuffer(flags);
-  size_t l = strlen(s) + 1;
-
-  addMultipleBuffer(b, s, l, char);
-
-  return baseBuffer(b, char);
-}
-
-
-int
-unfindBuffer(int flags)
-{ GET_LD
-  if ( flags & BUF_RING )
-  { if ( --current_buffer_id <= 0 )
-      current_buffer_id = BUFFER_RING_SIZE-1;
-  }
-
-  fail;
-}
-
-
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-codes_or_chars_to_buffer(term_t l, unsigned flags, int wide)
-
-If l represents a list of codes   or characters, return a buffer holding
-the characters. If wide == TRUE  the   buffer  contains  objects of type
-pl_wchar_t. Otherwise it contains traditional characters.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-static int
-charCode(word w)
-{ if ( isAtom(w) )
-  { Atom a = atomValue(w);
-
-    if ( a->length == 1 && true(a->type, PL_BLOB_TEXT) )
-      return a->name[0] & 0xff;
-    if ( a->length == sizeof(pl_wchar_t) && a->type == &ucs_atom )
-    { pl_wchar_t *p = (pl_wchar_t*)a->name;
-
-      return p[0];
-    }
-  }
-
-  return -1;
-}
-
-
-Buffer
-codes_or_chars_to_buffer(term_t l, unsigned int flags, int wide)
-{ GET_LD
-  Buffer b;
-  word list = valHandle(l);
-  Word arg, tail;
-  enum { CHARS, CODES } type;
-
-  if ( isList(list) )
-  { arg = argTermP(list, 0);
-    deRef(arg);
-    if ( isTaggedInt(*arg) )
-    { intptr_t i = valInt(*arg);
-      if ( i >= 0 && (wide || i < 256) )
-      { type = CODES;
-	goto ok;
-      }
-    } else if ( charCode(*arg) >= 0 )
-    { type = CHARS;
-      goto ok;
-    }
-  } else if ( isNil(list) )
-  { return findBuffer(flags);
-  }
-
-  fail;
-
-ok:
-  b = findBuffer(flags);
-
-  while( isList(list) )
-  { intptr_t c = -1;
-
-    arg = argTermP(list, 0);
-    deRef(arg);
-
-    switch(type)
-    { case CODES:
-	if ( isTaggedInt(*arg) )
-	{ c = valInt(*arg);
-	}
-        break;
-      case CHARS:
-	c = charCode(*arg);
-        break;
-    }
-
-    if ( c < 0 || (!wide && c > 0xff) )
-    { unfindBuffer(flags);		/* TBD: check unicode range */
-      return NULL;
-    }
-
-    if ( wide )
-      addBuffer(b, (pl_wchar_t)c, pl_wchar_t);
-    else
-      addBuffer(b, (unsigned char)c, unsigned char);
-
-    tail = argTermP(list, 1);
-    deRef(tail);
-    list = *tail;
-  }
-  if ( !isNil(list) )
-  { unfindBuffer(flags);
-    return NULL;
-  }
-
-  return b;
-}
-
 
 int
 PL_get_list_nchars(term_t l, size_t *length, char **s, unsigned int flags)
 { Buffer b;
+  CVT_result result;
 
-  if ( (b = codes_or_chars_to_buffer(l, flags, FALSE)) )
+  if ( (b = codes_or_chars_to_buffer(l, flags, FALSE, &result)) )
   { char *r;
     size_t len = entriesBuffer(b, char);
 
@@ -2022,6 +1889,15 @@ PL_is_list(term_t t)
 }
 
 
+int
+PL_is_pair(term_t t)
+{ GET_LD
+  word w = valHandle(t);
+
+  return isList(w) ? TRUE : FALSE;
+}
+
+
 int
 PL_is_atomic__LD(term_t t ARG_LD)
 { word w = valHandle(t);
@@ -3472,9 +3348,12 @@ PL_predicate_info(predicate_t pred, atom_t *name, int *arity, module_t *m)
 { if ( pred->type == PROCEDURE_TYPE )
   { Definition def = pred->definition;
 
-    *name  = def->functor->name;
-    *arity = def->functor->arity;
-    *m     = def->module;
+    if ( name )
+      *name  = def->functor->name;
+    if ( arity )
+      *arity = def->functor->arity;
+    if ( m )
+      *m     = def->module;
 
     succeed;
   }
@@ -4031,6 +3910,9 @@ PL_dispatch_hook(PL_dispatch_hook_t hook)
 
 
 #if defined(HAVE_SELECT) && !defined(__WINDOWS__)
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 
 #ifdef __WINDOWS__
 #include <winsock2.h>
