@@ -300,7 +300,8 @@ addMPZToBuffer(Buffer b, mpz_t mpz)
   long hdrsize;
   size_t count;
 
-  growBuffer(b, size+4);
+  if ( !growBuffer(b, size+4) )
+    outOfCore();
   if ( mpz_sgn(mpz) < 0 )
     hdrsize = -(long)size;
   else
@@ -843,7 +844,7 @@ PL_get_number(term_t t, Number n)
 		 *	     PROMOTION		*
 		 *******************************/
 
-void
+int
 promoteToFloatNumber(Number n)
 { switch(n->type)
   { case V_INTEGER:
@@ -854,6 +855,9 @@ promoteToFloatNumber(Number n)
     case V_MPZ:
     { double val = mpz_get_d(n->value.mpz);
 
+      if ( !check_float(val) )
+	return FALSE;
+
       clearNumber(n);
       n->value.f = val;
       n->type = V_FLOAT;
@@ -862,6 +866,9 @@ promoteToFloatNumber(Number n)
     case V_MPQ:
     { double val = mpq_get_d(n->value.mpq);
 
+      if ( !check_float(val) )
+	return FALSE;
+
       clearNumber(n);
       n->value.f = val;
       n->type = V_FLOAT;
@@ -871,6 +878,8 @@ promoteToFloatNumber(Number n)
     case V_FLOAT:
       break;
   }
+
+  return TRUE;
 }
 
 
