@@ -320,6 +320,12 @@ A common basis for C keywords.
 #define NORETURN
 #endif
 
+#if defined(__GNUC__) && !defined(MAY_ALIAS)
+#define MAY_ALIAS __attribute__ ((__may_alias__))
+#else
+#define MAY_ALIAS
+#endif
+
 #ifdef HAVE___BUILTIN_EXPECT
 #define likely(x)       __builtin_expect((x), 1)
 #define unlikely(x)     __builtin_expect((x), 0)
@@ -370,8 +376,8 @@ typedef void *			caddress;
 #endif
 				/* n is 2^m !!! */
 #define ROUND(p, n)		((((p) + (n) - 1) & ~((n) - 1)))
-#define addPointer(p, n)	((void *) ((char *)(p) + (intptr_t)(n)))
-#define diffPointers(p1, p2)	((char *)(p1) - (char *)(p2))
+#define addPointer(p, n)	((void *) ((intptr_t)(p) + (intptr_t)(n)))
+#define diffPointers(p1, p2)	((intptr_t)(p1) - (intptr_t)(p2))
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 			     LIMITS
@@ -390,12 +396,6 @@ them.  Descriptions:
 	use malloc() for allocating the stack as the local and global
 	stack need to be apart by this amount.  Also, an interrupt
 	skips this amount of stack.
-
-	* MAXVARIABLES
-	Maximum number of variables in a clause.  May be increased
-	further, but the global local stacks need to be separated
-	by this amount for the segmentation-fault based stack expansion
-	version (just costs virtual memory).
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define BUFFER_RING_SIZE 	16	/* foreign buffer ring (pl-fli.c) */
@@ -403,7 +403,6 @@ them.  Descriptions:
 #define MAXARITY		1024	/* arity of predicate */
 #define MINFOREIGNSIZE		32	/* Minimum term_t in foreign frame */
 #define MAXSYMBOLLEN		256	/* max size of foreign symbols */
-#define MAXVARIABLES		65536	/* number of variables/clause */
 #define OP_MAXPRIORITY		1200	/* maximum operator priority */
 #define SMALLSTACK		32 * 1024 /* GC policy */
 
@@ -426,6 +425,11 @@ them.  Descriptions:
 				 ((n)&~PLMAXTAGGEDINT) == ~PLMAXTAGGEDINT)
 #define PLMININT		(((int64_t)-1<<(INT64BITSIZE-1)))
 #define PLMAXINT		(-(PLMININT+1))
+#if SIZEOF_WCHAR_T == 2
+#define PLMAXWCHAR		(0xffff)
+#else
+#define PLMAXWCHAR		(0x10ffff)
+#endif
 
 #if vax
 #define MAXREAL			(1.701411834604692293e+38)
@@ -448,7 +452,7 @@ sizes  of  the  hash  tables are defined.  Note that these should all be
 #define FLAGHASHSIZE		16	/* global flag/3 table */
 #define ARITHHASHSIZE		64	/* arithmetic function table */
 
-#include "pl-table.h"
+#include "os/pl-table.h"
 
 /* Definition->indexPattern is set to NEED_REINDEX if the definition's index
    pattern needs to be recomputed */
@@ -483,7 +487,6 @@ Operator types.  NOTE: if you change OP_*, check operatorTypeToAtom()!
 #define OP_XFX	(0x50|OP_INFIX)
 #define OP_XFY	(0x60|OP_INFIX)
 #define OP_YFX	(0x70|OP_INFIX)
-#define	OP_YFY	(0x80|OP_INFIX)
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Type fields.  These codes are  included  in  a  number  of  the  runtime
@@ -686,6 +689,16 @@ typedef struct
 #endif
 #define floatNumber(n)	((n)->type >= V_FLOAT)
 
+typedef enum
+{ NUM_ERROR = FALSE,			/* Syntax error */
+  NUM_OK    = TRUE,			/* Ok */
+  NUM_FUNDERFLOW = -1,			/* Float underflow */
+  NUM_FOVERFLOW = -2,			/* Float overflow */
+  NUM_IOVERFLOW = -3			/* Integer overflow */
+} strnumstat;
+
+
+
 		 /*******************************
 		 *	   GET-PROCEDURE	*
 		 *******************************/
@@ -790,6 +803,7 @@ with one operation, it turns out to be faster as well.
 #define P_ISO			(0x08000000L) /* predicate */
 #define P_META			(0x10000000L) /* predicate */
 #define P_MFCONTEXT		(0x20000000L) /* predicate */
+#define P_PUBLIC		(0x40000000L) /* predicate */
 
 #define ERASED			(0x0001) /* clause, record */
 					 /* clause flags */
@@ -1616,6 +1630,18 @@ typedef struct
 #define PL_EV_THREADFINISHED 8		/* A thread has finished */
 
 
+		 /*******************************
+		 *	       COMPARE		*
+		 *******************************/
+
+/* Results from comparison operations.  Mostly used by compareStandard() */
+
+#define CMP_ERROR  -2			/* Error (out of memory) */
+#define CMP_LESS   -1			/* < */
+#define CMP_EQUAL   0			/* == */
+#define CMP_GREATER 1			/* > */
+#define CMP_NOTEQ   2			/* \== */
+
 		/********************************
 		*             STACKS            *
 		*********************************/
@@ -1873,6 +1899,25 @@ typedef struct
 
 extern const code_info codeTable[]; /* Instruction info (read-only) */
 
+		 /*******************************
+		 *	  TEXT PROCESSING	*
+		 *******************************/
+
+typedef enum
+{ CVT_ok = 0,				/* Conversion ok */
+  CVT_wide,				/* Conversion needs wide characters */
+  CVT_partial,				/* Input list is partial */
+  CVT_nolist,				/* Input list is not a list */
+  CVT_nocode,				/* List contains a non-code */
+  CVT_nochar				/* List contains a non-char */
+} CVT_status;
+
+typedef struct
+{ CVT_status status;
+  word culprit;				/* for CVT_nocode/CVT_nochar */
+} CVT_result;
+
+
 		/********************************
 		*            DEBUGGER           *
 		*********************************/
@@ -1899,7 +1944,6 @@ Tracer communication declarations.
 #define EXCEPTION_PORT	0x100
 #define CUT_PORT	(CUT_CALL_PORT|CUT_EXIT_PORT)
 #define PORT_MASK	0x1ff
-#define VERY_DEEP	1000000000L	/* deep skiplevel */
 
 #define LONGATOM_CHECK	    0x01	/* read/1: error on intptr_t atoms */
 #define SINGLETON_CHECK	    0x02	/* read/1: check singleton vars */
@@ -1911,7 +1955,7 @@ Tracer communication declarations.
 #define SYSTEM_MODE	    (debugstatus.styleCheck & DOLLAR_STYLE)
 
 typedef struct debuginfo
-{ uintptr_t	skiplevel;		/* current skip level */
+{ size_t	skiplevel;		/* current skip level */
   bool		tracing;		/* are we tracing? */
   debug_type	debugging;		/* are we debugging? */
   int		leashing;		/* ports we are leashing */
@@ -2006,11 +2050,11 @@ decrease).
 
 #define REL(a)		((Word)(a) - (Word)(lBase))
 
-#ifdef _DEBUG
+#if defined(_DEBUG) && !defined(O_MAINTENANCE)
 #define O_MAINTENANCE
 #endif
 
-#include "pl-os.h"			/* OS dependencies */
+#include "os/pl-os.h"			/* OS dependencies */
 
 #ifdef SYSLIB_H
 #include SYSLIB_H
@@ -2032,10 +2076,11 @@ decrease).
 #include "pl-ldpass.h"			/* Wrap __LD functions */
 #include "pl-inline.h"			/* Inline facilities */
 #include "pl-privitf.h"			/* private foreign interface */
-#include "pl-text.h"			/* text manipulation */
+#include "os/pl-text.h"			/* text manipulation */
 #include "pl-hash.h"			/* Murmurhash function */
-#include "pl-option.h"			/* Option processing */
-#include "pl-files.h"			/* File management */
+#include "os/pl-option.h"		/* Option processing */
+#include "os/pl-files.h"		/* File management */
+#include "os/pl-string.h"		/* Basic string functions */
 
 #ifdef __DECC				/* Dec C-compiler: avoid conflicts */
 #undef leave
