@@ -31,7 +31,7 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 
 #include "rc/rc.h"
 #include "pl-incl.h"
-#include "pl-ctype.h"
+#include "os/pl-ctype.h"
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -230,7 +230,8 @@ memarea_limit(const char *s)
 { number n;
   unsigned char *q;
 
-  if ( str_number((unsigned char *)s, &q, &n, FALSE) && intNumber(&n) )
+  if ( str_number((unsigned char *)s, &q, &n, FALSE) == NUM_OK &&
+       intNumber(&n) )
   { switch((int)*q)
     { case 'k':
       case 'K':
