@@ -79,7 +79,7 @@ lookupProcedure(functor_t f, Module m)
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Add (import) a defintion to a  module.   Used  by  loadImport(). Must be
+Add (import) a definition to a  module.   Used  by loadImport(). Must be
 merged with pl_import().
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
@@ -303,7 +303,7 @@ get_functor(term_t descr, functor_t *fdef, Module *m, term_t h, int how)
   PL_strip_module(descr, m, head);
 
   if ( PL_is_functor(head, FUNCTOR_divide2) ||
-       (dcgpi=PL_is_functor(head, FUNCTOR_div2)) )
+       (dcgpi=PL_is_functor(head, FUNCTOR_gdiv2)) )
   { term_t a = PL_new_term_ref();
     atom_t name;
     int arity = 0;
@@ -581,7 +581,7 @@ pl_current_predicate1(term_t spec, control_t ctx)
     if ( PL_is_functor(pi, FUNCTOR_divide2) )
     { _PL_get_arg(1, pi, nt);
       _PL_get_arg(2, pi, at);
-    } else if ( PL_is_functor(pi, FUNCTOR_div2) )
+    } else if ( PL_is_functor(pi, FUNCTOR_gdiv2) )
     { _PL_get_arg(1, pi, nt);
       _PL_get_arg(2, pi, at);
       aextra = 2;
@@ -2009,7 +2009,7 @@ pl_retractall(term_t head)
   int allvars = TRUE;
   fid_t fid;
 
-  if ( !get_procedure(head, &proc, thehead, GP_FINDHERE) )
+  if ( !get_procedure(head, &proc, thehead, GP_CREATE) )
     succeed;
 
   def = getProcDefinition(proc);
@@ -2160,6 +2160,7 @@ attribute_mask(atom_t key)
   if (key == ATOM_thread_local)  return P_THREAD_LOCAL;
   if (key == ATOM_noprofile)     return P_NOPROFILE;
   if (key == ATOM_iso)      	 return P_ISO;
+  if (key == ATOM_public)      	 return P_PUBLIC;
 
   return 0;
 }
@@ -2303,8 +2304,7 @@ setDynamicProcedure(Procedure proc, bool isdyn)
   LOCKDEF(def);
 
   if ( isdyn )				/* static --> dynamic */
-  { GET_LD
-    char *msg;
+  { char *msg;
 
     if ( def->definition.clauses )
     { UNLOCKDEF(def);
@@ -2330,8 +2330,6 @@ setDynamicProcedure(Procedure proc, bool isdyn)
   ok:
     freeCodesDefinition(def);		/* reset to S_VIRGIN */
     set(def, DYNAMIC);
-    if ( SYSTEM_MODE )
-      set(def, SYSTEM|HIDE_CHILDS);
 
     UNLOCKDEF(def);
   } else				/* dynamic --> static */
