@@ -5,7 +5,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2009, University of Amsterdam
+    Copyright (C): 1985-2010, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -25,8 +25,9 @@
 #include <math.h>
 /*#define O_DEBUG 1*/
 #include "pl-incl.h"
-#include "pl-ctype.h"
-#include "pl-utf8.h"
+#include "os/pl-ctype.h"
+#include "os/pl-utf8.h"
+#include "os/pl-dtoa.h"
 #include "pl-umap.c"			/* Unicode map */
 
 typedef const unsigned char * cucharp;
@@ -37,7 +38,9 @@ typedef       unsigned char * ucharp;
 #undef LD
 #define LD LOCAL_LD
 
-static void	addUTF8Buffer(Buffer b, int c);
+static void	  addUTF8Buffer(Buffer b, int c);
+static strnumstat scan_decimal(cucharp *sp, Number n);
+
 
 		 /*******************************
 		 *     UNICODE CLASSIFIERS	*
@@ -45,22 +48,47 @@ static void	addUTF8Buffer(Buffer b, int c);
 
 #define CharTypeW(c, t, w) \
 	((unsigned)(c) <= 0xff ? (_PL_char_types[(unsigned)(c)] t) \
-			       : (uflagsW(c) & w))
+			       : (uflagsW(c) & (w)))
 
-#define PlBlankW(c)	CharTypeW(c, <= SP, U_SEPARATOR)
+#define PlBlankW(c)	CharTypeW(c, == SP, U_SEPARATOR)
 #define PlUpperW(c)	CharTypeW(c, == UC, U_UPPERCASE)
 #define PlIdStartW(c)	(c <= 0xff ? (isLower(c)||isUpper(c)||c=='_') \
 				   : uflagsW(c) & U_ID_START)
 #define PlIdContW(c)	CharTypeW(c, >= UC, U_ID_CONTINUE)
-#define PlSymbolW(c)	CharTypeW(c, == SY, 0)
+#define PlSymbolW(c)	CharTypeW(c, == SY, U_SYMBOL)
 #define PlPunctW(c)	CharTypeW(c, == PU, 0)
-#define PlSoloW(c)	CharTypeW(c, == SO, 0)
+#define PlSoloW(c)	CharTypeW(c, == SO, U_OTHER)
+#define PlInvalidW(c)   (uflagsW(c) == 0)
 
 int
 unicode_separator(pl_wchar_t c)
 { return PlBlankW(c);
 }
 
+/* unquoted_atomW() returns TRUE if text can be written to s as unquoted atom
+*/
+
+int
+unquoted_atomW(const pl_wchar_t *s, size_t len, IOSTREAM *fd)
+{ const pl_wchar_t *e = &s[len];
+
+  if ( len == 0 )
+    return FALSE;
+
+  if ( !PlIdStartW(*s) || PlUpperW(*s) )
+    return FALSE;
+
+  for(s++; s<e; )
+  { int c = *s++;
+
+    if ( !(PlIdContW(c) && (!fd || Scanrepresent(c, fd) == 0)) )
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
+
 
 		 /*******************************
 		 *	   CHAR-CONVERSION	*
@@ -248,7 +276,7 @@ typedef struct
   term_t	singles;		/* Report singleton variables */
   term_t	subtpos;		/* Report Subterm positions */
   term_t	comments;		/* Report comments */
-  short		strictness;		/* Strictness level */
+  int		strictness;		/* Strictness level */
 
   atom_t	locked;			/* atom that must be unlocked */
   struct var_table vt;			/* Data about variables */
@@ -352,6 +380,20 @@ Error term:
 
 #define syntaxError(what, rd) { errorWarning(what, 0, rd); fail; }
 
+const char *
+str_number_error(strnumstat rc)
+{ switch(rc)
+  { case NUM_ERROR:      return "illegal_number";
+    case NUM_OK:	 return "no_error";
+    case NUM_FUNDERFLOW: return "float_underflow";
+    case NUM_FOVERFLOW:  return "float_overflow";
+    case NUM_IOVERFLOW:  return "integer_overflow";
+  }
+
+  return NULL;
+}
+
+
 extern IOFUNCTIONS Sstringfunctions;
 
 static bool
@@ -471,6 +513,12 @@ errorWarning(const char *id_str, term_t id_term, ReadData _PL_rd)
 }
 
 
+static int
+numberError(strnumstat rc, ReadData _PL_rd)
+{ return errorWarning(str_number_error(rc), 0, _PL_rd);
+}
+
+
 static int
 singletonWarning(const char *which, const char **vars, int nvars)
 { GET_LD
@@ -698,6 +746,8 @@ raw_read_quoted(int q, ReadData _PL_rd)
 	    if ( c == EOF )
 	      goto eofinstr;
 	    addToBuffer(c, _PL_rd);
+	    if ( c == q )
+	      return TRUE;
 	  }
 	  continue;			/* \symbolic-control-char */
       }
@@ -770,6 +820,7 @@ raw_read2(ReadData _PL_rd ARG_LD)
   IOPOS *pos;
 
   clearBuffer(_PL_rd);				/* clear input buffer */
+  _PL_rd->strictness = truePrologFlag(PLFLAG_ISO);
   source_line_no = -1;
 
   for(;;)
@@ -865,7 +916,8 @@ raw_read2(ReadData _PL_rd ARG_LD)
 			  level++;
 			break;
 		      case '/':
-			if ( last == '*' && --level == 0 )
+			if ( last == '*' &&
+			     (--level == 0 || _PL_rd->strictness) )
 			{ if ( cbuf )
 			  { addUTF8Buffer(cbuf, EOS);
 			    if ( !add_comment(cbuf, pos, _PL_rd PASS_LD) )
@@ -921,25 +973,19 @@ raw_read2(ReadData _PL_rd ARG_LD)
 			addToBuffer(' ', _PL_rd);
 		    }
 		    if ( c == '\n' )
-		    { IOPOS p = {0};	/* silence the compiler */
-		      IOPOS *pp;
-		      if ( (pp=rb.stream->position) )
-			p = *pp;
+		    { int c2 = Speekcode(rb.stream);
 
-		      c = getchr();
-		      if ( c == '%' )
+		      if ( c2 == '%' )
 		      { if ( something_read )
-			{ addToBuffer('\n', _PL_rd);
+			{ addToBuffer(c, _PL_rd);
 			  addToBuffer(' ', _PL_rd);
 			}
-			addUTF8Buffer(cbuf, '\n');
-			addUTF8Buffer(cbuf, '%');
+			addUTF8Buffer(cbuf, c);
+			c = Sgetcode(rb.stream);
+			assert(c==c2);
+			addUTF8Buffer(cbuf, c);
 			continue;
 		      }
-		      Sungetcode(c, rb.stream); /* unsafe: see Sungetcode() */
-		      if ( pp )
-			*pp = p;
-		      c = '\n';
 		    }
 		    break;
 		  }
@@ -957,18 +1003,60 @@ raw_read2(ReadData _PL_rd ARG_LD)
 		}
 		goto handle_c;		/* is the newline */
      case '\'': if ( rb.here > rb.base && isDigit(rb.here[-1]) )
-		{ addToBuffer(c, _PL_rd); 		/* <n>' */
-		  if ( rb.here[-2] == '0' )		/* 0'<c> */
-		  { if ( (c=getchr()) != EOF )
-		    { addToBuffer(c, _PL_rd);
-		      break;
+		{ cucharp bs = &rb.here[-1];
+
+		  if ( bs > rb.base && isDigit(bs[-1]) )
+		    bs--;
+		  if ( bs > rb.base && isSign(bs[-1]) )
+		    bs--;
+
+		  if ( bs == rb.base || !PlIdContW(bs[-1]) )
+		  { int base;
+
+		    if ( isSign(bs[0]) )
+		      bs++;
+		    rb.here[0] = EOS;
+		    base = atoi((char*)bs);
+		    if ( base <= 36 )
+		    { if ( base == 0 )			/* 0'<c> */
+		      { addToBuffer(c, _PL_rd);
+			{ if ( (c=getchr()) != EOF )
+			  { addToBuffer(c, _PL_rd);
+			    if ( c == '\\' ) 		/* 0'\<c> */
+			    { if ( (c=getchr()) != EOF )
+				addToBuffer(c, _PL_rd);
+			    } else if ( c == '\'' ) 	/* 0'' */
+			    { if ( (c=getchr()) != EOF )
+			      { if ( c == '\'' )
+				  addToBuffer(c, _PL_rd);
+				else
+				  goto handle_c;
+			      }
+			    }
+			    break;
+			  }
+			  rawSyntaxError("end_of_file");
+			}
+		      } else
+		      { int c2 = Speekcode(rb.stream);
+
+			if ( c2 != EOF )
+			{ if ( digitValue(base, c2) >= 0 )
+			  { addToBuffer(c, _PL_rd);
+			    c = Sgetcode(rb.stream);
+			    addToBuffer(c, _PL_rd);
+			    dotseen = FALSE;
+			    break;
+			  }
+			  goto sqatom;
+			}
+			rawSyntaxError("end_of_file");
+		      }
 		    }
-		    rawSyntaxError("end_of_file");
 		  }
-		  dotseen = FALSE;
-		  break;
 		}
 
+	      sqatom:
      		set_start_line;
      		if ( !raw_read_quoted(c, _PL_rd) )
 		  fail;
@@ -1003,7 +1091,6 @@ raw_read2(ReadData _PL_rd ARG_LD)
       default:	if ( c < 0xff )
 		{ switch(_PL_char_types[c])
 		  { case SP:
-		    case CT:
 		    blank:
 		      if ( dotseen )
 		      { if ( rb.here - rb.base == 1 )
@@ -1332,7 +1419,7 @@ SkipSymbol(unsigned char *in, ReadData _PL_rd)
   for( ; *in; in=s)
   { s = (unsigned char*)utf8_get_char((char*)in, &chr);
 
-    if ( !isSymbolW(chr) )
+    if ( !PlSymbolW(chr) )
       return in;
     if ( chr == '`' && _PL_rd->backquoted_string )
       return in;
@@ -1361,14 +1448,17 @@ uint64_to_double(uint64_t i)
 #endif
 
 
-static int
+static strnumstat
 scan_decimal(cucharp *sp, Number n)
 { uint64_t maxi = PLMAXINT/10;
   uint64_t t = 0;
   cucharp s = *sp;
-  int c;
+  int c = *s;
 
-  for(c = *s; isDigit(c); c = *++s)
+  if ( !isDigit(c) )
+    return NUM_ERROR;
+
+  for(; isDigit(c); c = *++s)
   { if ( t > maxi || t * 10 + c - '0' > PLMAXINT )
     {
 #ifdef O_GMP
@@ -1382,20 +1472,22 @@ scan_decimal(cucharp *sp, Number n)
       }
       *sp = s;
 
-      succeed;
+      return NUM_OK;
 #else
-      double maxf = MAXREAL / (double) 10 - (double) 10;
-      double tf = uint64_to_double(t);
+      char *end;
 
-      for(c = *s; isDigit(c); c = *++s)
-      { if ( tf > maxf )
-	  fail;				/* number too large */
-        tf = tf * (double)10 + (double)(c - '0');
-      }
+      while(isDigit(*s))
+	s++;
+      errno = 0;
+      n->value.f = strtod(*sp, &end);
+      if ( s != end )
+	return NUM_ERROR;
+      if ( errno == ERANGE )
+	return NUM_FOVERFLOW;
       *sp = s;
-      n->value.f = tf;
+
       n->type = V_FLOAT;
-      succeed;
+      return NUM_OK;
 #endif
     } else
       t = t * 10 + c - '0';
@@ -1405,11 +1497,11 @@ scan_decimal(cucharp *sp, Number n)
 
   n->value.i = t;
   n->type = V_INTEGER;
-  succeed;
+  return NUM_OK;
 }
 
 
-static int
+static strnumstat
 scan_number(cucharp *s, int b, Number n)
 { int d;
   uint64_t maxi = PLMAXINT/b;		/* cache? */
@@ -1417,7 +1509,7 @@ scan_number(cucharp *s, int b, Number n)
   cucharp q = *s;
 
   if ( (d = digitValue(b, *q)) < 0 )
-    fail;				/* syntax error */
+    return NUM_ERROR;			/* syntax error */
   t = d;
   q++;
 
@@ -1436,7 +1528,7 @@ scan_number(cucharp *s, int b, Number n)
       }
       *s = q;
 
-      succeed;
+      return NUM_OK;
 #else
       double maxf = MAXREAL / (double) b - (double) b;
       double tf = uint64_to_double(t);
@@ -1451,7 +1543,7 @@ scan_number(cucharp *s, int b, Number n)
       n->value.f = tf;
       n->type = V_FLOAT;
       *s = q;
-      succeed;
+      return NUM_OK;
 #endif
     } else
     { q++;
@@ -1462,16 +1554,20 @@ scan_number(cucharp *s, int b, Number n)
   n->value.i = t;
   n->type = V_INTEGER;
   *s = q;
-  succeed;
+  return NUM_OK;
 }
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 escape_char() decodes a \<chr> specification that can appear either in a
 quoted atom/string or as a  character   specification  such as 0'\n. The
-return value is EOF and an exception is pushed on illegal sequences.
+return value is either a valid character   code,  ESC_EOS if there is no
+more data or ESC_ERROR if there is an error.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#define ESC_EOS	  (-1)
+#define ESC_ERROR (-2)
+
 static int
 escape_char(cucharp in, ucharp *end, int quote, ReadData _PL_rd)
 { int base;
@@ -1479,7 +1575,7 @@ escape_char(cucharp in, ucharp *end, int quote, ReadData _PL_rd)
   int c;
   cucharp e;
 
-#define OK(v) if (1) {chr = (v); goto ok;} else (void)0
+#define OK(v) do { chr = (v); goto ok; } while(0)
 
 again:
   in = utf8_get_uchar(in, &c);
@@ -1491,22 +1587,20 @@ again:
     case 'c':				/* skip \c<blank>* */
       if ( quote )
       { in = skipSpaces(in);
-      skip_cont:
 	e = utf8_get_uchar(in, &c);
-	if ( c == '\\' )
-	{ in = e;
-	  goto again;
-	}
-	if ( c == quote )		/* \c ' --> no output */
-	{ OK(EOF);
-	}
+      skip_cont:
 	in = e;
+	if ( c == '\\' )
+	  goto again;
+	if ( c == quote )		/* \c ' --> no output */
+	  OK(ESC_EOS);
 	OK(c);
       }
       OK('c');
     case '\n':				/* \LF<blank>* */
       if ( quote )
-      { for( ; *in; in=e )
+      { e = in;
+	for( ; *in; in=e )
 	{ e = utf8_get_uchar(in, &c);
 	  if ( c == '\n' || !PlBlankW(c) )
 	    break;
@@ -1514,6 +1608,8 @@ again:
 	goto skip_cont;
       }
       OK('\n');
+    case 'e':
+      OK(27);				/* 27 is ESC (\e is a gcc extension) */
     case 'f':
       OK('\f');
     case '\\':
@@ -1529,6 +1625,7 @@ again:
     case 'u':				/* \uXXXX */
     case 'U':				/* \UXXXXXXXX */
     { int digits = (c == 'u' ? 4 : 8);
+      cucharp errpos = in-1;
       chr = 0;
 
       while(digits-- > 0)
@@ -1538,10 +1635,20 @@ again:
 	if ( (dv=digitValue(16, c)) >= 0 )
 	{ chr = (chr<<4)+dv;
 	} else
-	{ errorWarning("Illegal \\u or \\U sequence", 0, _PL_rd);
-	  return EOF;
+	{ if ( _PL_rd )
+	  { last_token_start = (unsigned char*)errpos;
+	    errorWarning("Illegal \\u or \\U sequence", 0, _PL_rd);
+	  }
+	  return ESC_ERROR;
 	}
       }
+      if ( chr > PLMAXWCHAR )
+      { if ( _PL_rd )
+	{ last_token_start = (unsigned char*)errpos;
+	  errorWarning("Illegal character code", 0, _PL_rd);
+	}
+	return ESC_ERROR;
+      }
       OK(chr);
     }
     case 'x':
@@ -1556,15 +1663,24 @@ again:
     default:
       if ( c >= '0' && c <= '7' )	/* octal number */
       { int dv;
+	cucharp errpos;
 
 	base = 8;
 
       numchar:
+	errpos = in-1;
 	chr = digitValue(base, c);
 	c = *in++;
 	while( (dv = digitValue(base, c)) >= 0 )
 	{ chr = chr * base + dv;
 	  c = *in++;
+	  if ( chr > PLMAXWCHAR )
+	  { if ( _PL_rd )
+	    { last_token_start = (unsigned char*)errpos;
+	      errorWarning("Illegal character code", 0, _PL_rd);
+	    }
+	    return ESC_ERROR;
+	  }
 	}
 	if ( c != '\\' )
 	  in--;
@@ -1622,11 +1738,15 @@ get_string(unsigned char *in, unsigned char *ein, unsigned char **end, Buffer bu
 	break;
     } else if ( c == '\\' && DO_CHARESCAPE )
     { c = escape_char(in, &in, quote, _PL_rd);
-      if ( c == EOF )
-	return FALSE;
-      addUTF8Buffer(buf, c);
+      if ( c >= 0 )
+      { addUTF8Buffer(buf, c);
 
-      continue;
+	continue;
+      } else if ( c == ESC_ERROR )
+      { return FALSE;
+      } else
+      { break;
+      }
     } else if ( c >= 0x80 )		/* copy UTF-8 sequence */
     { do
       { addBuffer(buf, c, char);
@@ -1679,10 +1799,11 @@ neg_number(Number n)
 }
 
 
-int
+strnumstat
 str_number(cucharp in, ucharp *end, Number value, int escape)
 { int negative = FALSE;
-  unsigned int c;
+  cucharp start = in;
+  strnumstat rc;
 
   if ( *in == '-' )			/* skip optional sign */
   { negative = TRUE;
@@ -1699,8 +1820,12 @@ str_number(cucharp in, ucharp *end, Number value, int escape)
 
 	if ( escape && in[2] == '\\' )	/* 0'\n, etc */
 	{ chr = escape_char(in+3, end, 0, NULL);
+	  if ( chr < 0 )
+	    return NUM_ERROR;
 	} else
 	{ *end = utf8_get_uchar(in+2, &chr);
+	  if ( chr == '\'' && **end == '\'' ) /* handle 0''' as 0'' */
+	    (*end)++;
 	}
 
 	value->value.i = (int64_t)chr;
@@ -1722,88 +1847,80 @@ str_number(cucharp in, ucharp *end, Number value, int escape)
     }
 
     if ( base )				/* 0b<binary>, 0x<hex>, 0o<oct> */
-    { int rval;
-      in += 2;
-      rval = scan_number(&in, base, value);
+    { in += 2;
+
+      if ( (rc = scan_number(&in, base, value)) != NUM_OK )
+	return rc;
       *end = (ucharp)in;
       if ( negative )
 	neg_number(value);
-      return rval;
+
+      return NUM_OK;
     }
   }
 
-  if ( !isDigit(*in) || !scan_decimal(&in, value) )
-    fail;				/* too large? */
+  if ( (rc=scan_decimal(&in, value)) != NUM_OK )
+    return rc;				/* too large? */
 
 					/* base'value number */
-  if ( *in == '\'' )
+  if ( *in == '\'' &&
+       value->type == V_INTEGER &&
+       value->value.i <= 36 &&
+       value->value.i > 1 &&
+       digitValue(value->value.i, in[1]) >= 0 )
   { in++;
 
-    if ( value->type != V_INTEGER ||
-	 value->value.i > 36 ||
-	 value->value.i <= 1 )
-      fail;				/* illegal base */
-    if ( !scan_number(&in, (int)value->value.i, value) )
-      fail;				/* number too large */
-
-    if ( isAlpha(*in) )
-      fail;				/* illegal number */
+    if ( !(rc=scan_number(&in, (int)value->value.i, value)) )
+      return rc;			/* number too large */
 
     if ( negative )
       neg_number(value);
 
     *end = (ucharp)in;
-    succeed;
+
+    return NUM_OK;
   }
 					/* floating point numbers */
-					/* we should use strtod_l() here */
   if ( *in == '.' && isDigit(in[1]) )
-  { double n;
-
-    promoteToFloatNumber(value);
-    n = 10.0, in++;
-    while( isDigit(c = *in) )
-    { in++;
-      value->value.f += (double)(c-'0') / n;
-      n *= 10.0;
-    }
-  }
-
-  if ( *in == 'e' || *in == 'E' )
-  { number exponent;
-    bool neg_exponent;
+  { value->type = V_FLOAT;
 
     in++;
-    DEBUG(9, Sdprintf("Exponent\n"));
-    switch(*in)
-    { case '-':
-	in++;
-        neg_exponent = TRUE;
-	break;
-      case '+':
-	in++;
-      default:
-	neg_exponent = FALSE;
-        break;
-    }
+    while( isDigit(*in) )
+      in++;
+  }
 
-    if ( !isDigit(*in) ||
-	 !scan_decimal(&in, &exponent) ||
-	 exponent.type != V_INTEGER )
-      fail;				/* too large exponent */
+  if ( (*in == 'e' || *in == 'E') &&
+       ((isSign(in[1]) && isDigit(in[2])) || isDigit(in[1])) )
+  { value->type = V_FLOAT;
 
-    promoteToFloatNumber(value);
+    in++;
+    if ( isSign(*in) )
+      in++;
+    while( isDigit(*in) )
+      in++;
+  }
 
-    value->value.f *= pow((double)10.0,
-			  neg_exponent ? -(double)exponent.value.i
-			               : (double)exponent.value.i);
+  if ( value->type == V_FLOAT )
+  { char *e;
+
+    errno = 0;
+    value->value.f = strtod((char*)start, &e);
+    if ( e != (char*)in )
+      return NUM_ERROR;
+    if ( errno == ERANGE && abs(value->value.f) > 1.0 )
+      return NUM_FOVERFLOW;
+
+    *end = (ucharp)in;
+
+    return NUM_OK;
   }
 
   if ( negative )
     neg_number(value);
 
   *end = (ucharp)in;
-  succeed;
+
+  return NUM_OK;
 }
 
 
@@ -1867,7 +1984,11 @@ get_token__LD(bool must_be_op, ReadData _PL_rd ARG_LD)
 	goto upper;
       goto lower;
     }
-    goto case_solo;			/* TBD: Need foreign symbols */
+    if ( PlSymbolW(c) )
+      goto case_symbol;
+    if ( PlInvalidW(c) )
+      syntaxError("illegal_character", _PL_rd);
+    goto case_solo;
   }
 
   switch(_PL_char_types[c])
@@ -1921,17 +2042,17 @@ get_token__LD(bool must_be_op, ReadData _PL_rd ARG_LD)
 		}
     case_digit:
     case DI:	{ number value;
-		  int echr;
+		  strnumstat rc;
 
-		  if ( str_number(&rdhere[-1],
-				  &rdhere, &value, DO_CHARESCAPE) &&
-		       utf8_get_uchar(rdhere, &echr) &&
-		       !PlIdContW(echr) )
+		  if ( (rc=str_number(&rdhere[-1],
+				      &rdhere, &value, DO_CHARESCAPE)) == NUM_OK )
 		  { cur_token.value.number = value;
 		    cur_token.type = T_NUMBER;
 		    break;
 		  } else
-		    syntaxError("illegal_number", _PL_rd);
+		  { numberError(rc, _PL_rd);
+		    return NULL;
+		  }
 		}
     case_solo:
     case SO:	{ cur_token.value.atom = codeToAtom(c);		/* not registered */
@@ -1942,6 +2063,7 @@ get_token__LD(bool must_be_op, ReadData _PL_rd ARG_LD)
 
 		  break;
 		}
+    case_symbol:
     case SY:	if ( c == '`' && _PL_rd->backquoted_string )
 		  goto case_bq;
 
@@ -1952,7 +2074,7 @@ get_token__LD(bool must_be_op, ReadData _PL_rd ARG_LD)
 		       isDigit(*rdhere) )
 		  { goto case_digit;
 		  }
-		  if ( c == '.' && isBlank(*rdhere) )	/* .<blank> */
+		  if ( c == '.' && PlBlankW(*rdhere) )	/* .<blank> */
 		  { cur_token.type = T_FULLSTOP;
 		    break;
 		  }
@@ -2057,6 +2179,8 @@ get_token__LD(bool must_be_op, ReadData _PL_rd ARG_LD)
 		  break;
 		}
 #endif
+    case CT:
+		syntaxError("illegal_character", _PL_rd);
     default:
     		{ sysError("read/1: tokeniser internal error");
     		  break;		/* make lint happy */
@@ -2135,7 +2259,7 @@ build_term(term_t term, atom_t atom, int arity, term_t *argv,
   if ( !hasGlobalSpace(arity+1) &&
        (rc=ensureGlobalSpace(arity+1, ALLOW_GC|ALLOW_SHIFT)) != TRUE )
     return rc;
-  if ( ensureSpaceForTermRefs(arity PASS_LD) != TRUE )
+  if ( (rc=ensureSpaceForTermRefs(arity PASS_LD)) != TRUE )
     return rc;
 
   DEBUG(9, Sdprintf("Building term %s/%d ... ", stringAtom(atom), arity));
@@ -2214,7 +2338,6 @@ isOp(atom_t atom, int kind, op_entry *e, ReadData _PL_rd)
     case OP_XFX:	e->left_pri = pri-1; e->right_pri = pri-1; break;
     case OP_XFY:	e->left_pri = pri-1; e->right_pri = pri;   break;
     case OP_YFX:	e->left_pri = pri;   e->right_pri = pri-1; break;
-    case OP_YFY:	e->left_pri = pri;   e->right_pri = pri;   break;
   }
 
   succeed;
@@ -2462,8 +2585,8 @@ complex_term(const char *stop, short maxpri, term_t term, term_t positions,
   term_t pin;
   int thestop;				/* encountered stop-character */
 
-  if (_PL_rd -> strictness == 0)
-      maxpri = OP_MAXPRIORITY+1;
+  if ( _PL_rd->strictness == 0 )
+    maxpri = OP_MAXPRIORITY+1;
 
   in_op.left_pri = 0;
   in_op.right_pri = 0;
@@ -3367,6 +3490,71 @@ PL_chars_to_term(const char *s, term_t t)
   return rval;
 }
 
+		 /*******************************
+		 *	     CODE TYPE		*
+		 *******************************/
+
+/* '$code_class'(+Code, +Category) is semidet.
+
+True if Code is a member of Category.   This predicate is added to allow
+for inspection of the chararacter categories   used for parsing. Defined
+categories are:
+
+    * layout
+    * graphic
+    These are the glueing symbol characters
+    * solo
+    These are the non-glueing symbol characters
+    * punct
+    These are the Prolog characters with reserved meaning
+    * id_start
+    Start of an unquoted atom or variable
+    * id_continue
+    Continue an unquoted atom or variable
+    * upper
+    If the id_start char fits, this is a variable.
+    * invalid
+    Character may not appear outside comments or quoted strings
+*/
+
+static
+PRED_IMPL("$code_class", 2, code_class, 0)
+{ PRED_LD
+  int code, rc;
+  atom_t class;
+  const char *c;
+
+  if ( !PL_get_char_ex(A1, &code, FALSE) ||
+       !PL_get_atom_ex(A2, &class) )
+    return FALSE;
+
+  if ( code > 0x10ffff )
+    PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_character, A1);
+
+  c = PL_atom_chars(class);
+  if ( streq(c, "layout") )
+    rc = PlBlankW(code);
+  else if ( streq(c, "graphic") )
+    rc = PlSymbolW(code);
+  else if ( streq(c, "solo") )
+    rc = PlSoloW(code);
+  else if ( streq(c, "punct") )
+    rc = PlPunctW(code);
+  else if ( streq(c, "upper") )
+    rc = PlUpperW(code);
+  else if ( streq(c, "id_start") )
+    rc = PlIdStartW(code);
+  else if ( streq(c, "id_continue") )
+    rc = PlIdContW(code);
+  else if ( streq(c, "invalid") )
+    rc = PlInvalidW(code);
+  else
+    return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_category, A2);
+
+  return rc ? TRUE : FALSE;
+}
+
+
 		 /*******************************
 		 *      PUBLISH PREDICATES	*
 		 *******************************/
@@ -3376,4 +3564,5 @@ BeginPredDefs(read)
   PRED_DEF("read_clause", 2, read_clause, 0)
   PRED_DEF("atom_to_term", 3, atom_to_term, 0)
   PRED_DEF("term_to_atom", 2, term_to_atom, 0)
+  PRED_DEF("$code_class", 2, code_class, 0)
 EndPredDefs
