@@ -143,7 +143,6 @@ initPrologLocalData(ARG1_LD)
   depth_limit   = (uintptr_t)DEPTH_NO_LIMIT;
 #endif
 
-  LD->float_format = "%g";
   LD->prolog_flag.write_attributes = PL_WRT_ATTVAR_IGNORE;
   updateAlerted(LD);
 }
@@ -1281,9 +1280,7 @@ resetStacks()
 
 int
 trim_stack(Stack s)
-{ s->gced_size = usedStackP(s);
-
-  if ( s->spare < s->def_spare )
+{ if ( s->spare < s->def_spare )
   { ssize_t reduce = s->def_spare - s->spare;
     ssize_t room = roomStackP(s);
 
@@ -1445,7 +1442,8 @@ freePrologLocalData(PL_local_data_t *ld)
 #endif
 
 #ifdef O_CYCLIC
-  clearSegStack(&ld->cycle.stack);
+  clearSegStack(&ld->cycle.lstack);
+  clearSegStack(&ld->cycle.vstack);
 #endif
 
   freeArithLocalData(ld);
