@@ -3,9 +3,10 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        J.Wielemaker@uva.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2009, University of Amsterdam
+    Copyright (C): 1985-2011, University of Amsterdam
+			      VU University Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -209,7 +210,7 @@ ac_pushTermAgenda__LD(ac_term_agenda *a, word w, functor_t *fp ARG_LD)
 #endif /*AC_TERM_WALK*/
 
 
-#if 0
+#if AC_TERM_WALK_LR
 
 		 /*******************************
 		 *    OPERATIONS ON TWO TERMS	*
@@ -219,7 +220,7 @@ typedef struct aNodeLR
 { Word		left;			/* left term */
   Word		right;			/* right term */
   size_t 	size;
-} aNode;
+} aNodeLR;
 
 typedef struct term_agendaLR
 { aNodeLR	work;			/* current work */
@@ -230,7 +231,7 @@ typedef struct term_agendaLR
 
 static void
 initTermAgendaLR(term_agendaLR *a, Word left, Word right)
-{ initSegStack(&a->stack, sizeof(aNode),
+{ initSegStack(&a->stack, sizeof(aNodeLR),
 	       sizeof(a->first_chunk), a->first_chunk);
   a->work.left  = left;
   a->work.right = right;
@@ -244,8 +245,11 @@ clearTermAgendaLR(term_agendaLR *a)
 }
 
 
+#define nextTermAgendaLR(a, lp, rp) \
+	nextTermAgendaLR__LD(a, lp, rp PASS_LD)
+
 static int
-nextTermAgendaLR(term_agendaLR *a, Word *lp, Word *rp)
+nextTermAgendaLR__LD(term_agendaLR *a, Word *lp, Word *rp ARG_LD)
 { Word p;
 
   while ( a->work.size == 0 )
@@ -254,8 +258,8 @@ nextTermAgendaLR(term_agendaLR *a, Word *lp, Word *rp)
   }
   a->work.size--;
 
-  *lp = a->work.left++;
-  *rp = a->work.right++;
+  deRef2(a->work.left++, p); *lp = p;
+  deRef2(a->work.right++,p); *rp = p;
 
   return TRUE;
 }
@@ -274,4 +278,172 @@ pushWorkAgendaLR(term_agendaLR *a, size_t amount, Word left, Word right)
   return TRUE;
 }
 
-#endif
+#endif /*AC_TERM_WALK_LR*/
+
+#if AC_TERM_WALK_LRS
+
+		 /*******************************
+		 *       TWO TERMS WITH POP	*
+		 *******************************/
+
+typedef struct aNodeLRS
+{ Functor	left;			/* left term */
+  Functor	right;			/* right term */
+  int		arg;
+  int	 	arity;
+  void	       *data;
+} aNodeLRS;
+
+typedef void (*popLRS)(Functor left, Functor right, void *data);
+
+typedef struct term_agendaLRS
+{ aNodeLRS	work;			/* current work */
+  popLRS 	pop;
+  segstack	stack;
+  char		first_chunk[sizeof(aNodeLRS)*25];
+} term_agendaLRS;
+
+
+static void
+initTermAgendaLRS(term_agendaLRS *a,
+		  Functor left, Functor right,
+		  popLRS pop, void *data)
+{ initSegStack(&a->stack, sizeof(aNodeLRS),
+	       sizeof(a->first_chunk), a->first_chunk);
+  a->pop	= pop;
+  a->work.data  = data;
+  a->work.left  = left;
+  a->work.right = right;
+  a->work.arg   = 0;
+  a->work.arity = arityFunctor(left->definition);
+}
+
+
+static void
+clearTermAgendaLRS(term_agendaLRS *a)
+{ do
+  { if ( a->work.arg != -1 )
+      (*a->pop)(a->work.left, a->work.right, a->work.data);
+  } while(popSegStack(&a->stack, &a->work));
+}
+
+
+#define nextTermAgendaLRS(a, lp, rp) \
+	nextTermAgendaLRS__LD(a, lp, rp PASS_LD)
+
+static int
+nextTermAgendaLRS__LD(term_agendaLRS *a, Word *lp, Word *rp ARG_LD)
+{ Word p;
+
+  while ( a->work.arg == a->work.arity )
+  { (*a->pop)(a->work.left, a->work.right, a->work.data);
+    a->work.arg = -1;
+    if ( !popSegStack(&a->stack, &a->work) )
+      return FALSE;
+  }
+
+  deRef2(&a->work.left->arguments[a->work.arg], p); *lp = p;
+  deRef2(&a->work.right->arguments[a->work.arg],p); *rp = p;
+  a->work.arg++;
+
+  return TRUE;
+}
+
+
+static int
+pushWorkAgendaLRS(term_agendaLRS *a, Functor left, Functor right, void *data)
+{ if ( !pushSegStack(&a->stack, &a->work) )
+    return FALSE;
+
+  a->work.data  = data;
+  a->work.left  = left;
+  a->work.right = right;
+  a->work.arg   = 0;
+  a->work.arity = arityFunctor(left->definition);
+
+  return TRUE;
+}
+
+#endif /*AC_TERM_WALK_LRS*/
+
+#if AC_TERM_WALK_VARIANT
+
+		 /*******************************
+		 *    WALKING FOR VARIANT/2	*
+		 *******************************/
+
+typedef struct aNodeVARIANT
+{ Word		left;			/* left term */
+  Word		right;			/* right term */
+  Word		c_left;			/* cyclic left term */
+  Word		c_right;		/* cyclic right term */
+  size_t 	size;
+} aNodeVARIANT;
+
+typedef struct term_agendaVARIANT
+{ aNodeVARIANT	work;			/* current work */
+  segstack	stack;
+  char		first_chunk[256];
+} term_agendaVARIANT;
+
+
+static void
+initTermAgendaVARIANT(term_agendaVARIANT *a, Word left, Word right)
+{ initSegStack(&a->stack, sizeof(aNodeVARIANT),
+	       sizeof(a->first_chunk), a->first_chunk);
+  a->work.left    = left;
+  a->work.right   = right;
+  a->work.c_left  = left;
+  a->work.c_right = right;
+  a->work.size    = 1;
+}
+
+
+static void
+clearTermAgendaVARIANT(term_agendaVARIANT *a)
+{ clearSegStack(&a->stack);
+}
+
+
+#define nextTermAgendaVARIANT(a, lp, rp, clp, crp) \
+	nextTermAgendaVARIANT__LD(a, lp, rp, clp, crp PASS_LD)
+
+static int
+nextTermAgendaVARIANT__LD(term_agendaVARIANT *a,
+			  Word *lp, Word *rp,
+			  Word *clp, Word *crp
+			  ARG_LD)
+{ Word p;
+
+  while ( a->work.size == 0 )
+  { if ( !popSegStack(&a->stack, &a->work) )
+      return FALSE;
+  }
+  a->work.size--;
+
+  deRef2(a->work.left++,    p); *lp = p;
+  deRef2(a->work.right++,   p); *rp = p;
+  deRef2(a->work.c_left++,  p); *clp = p;
+  deRef2(a->work.c_right++, p); *crp = p;
+
+  return TRUE;
+}
+
+
+static int
+pushWorkAgendaVARIANT(term_agendaVARIANT *a, size_t amount,
+		      Word left, Word right, Word c_left, Word c_right)
+{ if ( a->work.size > 0 )
+  { if ( !pushSegStack(&a->stack, &a->work) )
+      return FALSE;
+  }
+  a->work.left    = left;
+  a->work.right   = right;
+  a->work.c_left  = c_left;
+  a->work.c_right = c_right;
+  a->work.size    = amount;
+
+  return TRUE;
+}
+
+#endif /*AC_TERM_WALK_VARIANT*/
