@@ -5,7 +5,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2009, University of Amsterdam
+    Copyright (C): 1985-2011, University of Amsterdam
+			      VU University Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -23,10 +24,13 @@
 */
 
 #include "pl-incl.h"
-#include "pl-ctype.h"
+#include "os/pl-ctype.h"
 #include "pl-inline.h"
 #include "pl-dbref.h"
 
+#define SKIP_VERY_DEEP	  1000000000L	/* deep skiplevel */
+#define SKIP_REDO_IN_SKIP (SKIP_VERY_DEEP-1)
+
 #define WFG_TRACE	0x01000
 #define WFG_TRACING	0x02000
 #define WFG_BACKTRACE	0x04000
@@ -131,6 +135,16 @@ PL_unify_choice(term_t t, Choice ch)
 }
 
 
+static inline int
+valid_choice(Choice ch ARG_LD)
+{ if ( (int)ch->type >= 0 && (int)ch->type <= CHP_DEBUG &&
+       onStack(local, ch->frame) )
+    return TRUE;
+
+  return FALSE;
+}
+
+
 static int
 PL_get_choice(term_t r, Choice *chp)
 { GET_LD
@@ -139,7 +153,9 @@ PL_get_choice(term_t r, Choice *chp)
   if ( PL_get_long(r, &i) )
   { Choice ch = ((Choice)((Word)lBase + i));
 
-    assert(ch >= (Choice)lBase && ch < (Choice)lTop);
+    if ( !(ch >= (Choice)lBase && ch < (Choice)lTop) ||
+	 !valid_choice(ch PASS_LD) )
+      return PL_error(NULL, 0, NULL, ERR_EXISTENCE, ATOM_choice, r);
     *chp = ch;
 
     succeed;
@@ -171,7 +187,7 @@ isDebugFrame(LocalFrame FR)
     {					/* not last-call optimized */
       if ( false(parent->predicate, HIDE_CHILDS) )
 	return TRUE;			/* user calls system */
-      return FALSE;			/* system cals system */
+      return FALSE;			/* system calls system */
     } else
     { if ( false(parent, FR_HIDE_CHILDS) )
 	return TRUE;
@@ -403,14 +419,16 @@ Give a trace on the skipped goal for a redo.
 
   { Code pc2 = NULL;
 
-    if ( port == REDO_PORT && debugstatus.skiplevel == VERY_DEEP &&
+    if ( port == REDO_PORT && debugstatus.skiplevel == SKIP_VERY_DEEP &&
 	 (fr = redoFrame(frame, &pc2)) != NULL )
     { int rc;
 
-      debugstatus.skiplevel--;				   /* avoid a loop */
+      debugstatus.skiplevel = SKIP_REDO_IN_SKIP;
       SAVE_PTRS();
-      rc = tracePort(fr, bfr, REDO_PORT, pc2 PASS_LD);
+      rc = tracePort(frame, bfr, REDO_PORT, pc2 PASS_LD);
       RESTORE_PTRS();
+      debugstatus.skiplevel = levelFrame(fr);
+      set(fr, FR_SKIPPED);		/* cleared by "case 'c'" */
 
       switch( rc )
       { case ACTION_CONTINUE:
@@ -474,7 +492,7 @@ again:
   if (debugstatus.leashing & port)
   { char buf[LINESIZ];
 
-    debugstatus.skiplevel = VERY_DEEP;
+    debugstatus.skiplevel = SKIP_VERY_DEEP;
     debugstatus.tracing   = TRUE;
 
     Sfputs(" ? ", Sdout);
@@ -671,7 +689,6 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
     case '\n':
     case '\r':
     case 'c':	FeedBack("creep\n");
-		clear(frame, FR_SKIPPED);
 		if ( !(port & EXIT_PORT) )
 		  clear(frame, FR_SKIPPED);
 		return ACTION_CONTINUE;
@@ -969,7 +986,9 @@ alternatives(Choice ch)
 { GET_LD
 
   for(; ch; ch = ch->parent)
-  { if ( (isDebugFrame(ch->frame) || SYSTEM_MODE) )
+  { if ( ch->type == CHP_DEBUG )
+      continue;
+    if ( (isDebugFrame(ch->frame) || SYSTEM_MODE) )
       writeFrameGoal(ch->frame, NULL, WFG_CHOICE);
   }
 }
@@ -1223,6 +1242,56 @@ traceInterception(LocalFrame frame, Choice bfr, int port, Code PC)
   return rval;
 }
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Stores up to len bytes of a representation of the frame referenced by ref
+into buf. If ref is NULL, then the currently executing frame is used nextref
+gets the value of the parent of the frame
+Return value is 0: if there are no more frames (in which case nextref
+                   will be NULL)
+               >0: The number of bytes which are required to completely
+                   protray the frame
+
+Note that the text is returned as UTF-8, regardless of locale settings.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+int
+PL_walk_prolog_stack(void* ref, char* buf, size_t len, void** nextref)
+{ GET_LD
+  LocalFrame fr;
+
+  if (ref == NULL) /* Start of the chain - use current frame */
+  { LocalFrame topfr = environment_frame;
+
+    if ( topfr->predicate->definition.function == pl_prolog_current_frame )
+      fr = parentFrame(topfr);
+    else
+      fr = topfr;
+  } else
+  { fr = (LocalFrame)ref;
+  }
+
+  if ( (*nextref = parentFrame(fr)) == NULL)
+    return 0;
+
+  if ( fr->programPointer &&
+       false(fr->predicate, FOREIGN) &&
+       fr->parent &&
+       fr->parent->clause &&
+       fr->parent->predicate != PROCEDURE_dcall1->definition &&
+       fr->clause &&
+       fr->predicate != PROCEDURE_dc_call_prolog->definition)
+  { intptr_t pc = fr->programPointer - fr->parent->clause->clause->codes;
+
+    return snprintf(buf, len, "%s [PC=%ld] [Clause %d]",
+		    predicateName(fr->predicate),
+		    (long)pc,
+		    clauseNo(fr->predicate, fr->clause->clause));
+  } else
+  { return snprintf(buf, len, "%s <foreign>", predicateName(fr->predicate));
+  }
+}
+
+
 #endif /*O_DEBUGGER*/
 
 #ifndef offset
@@ -1517,7 +1586,7 @@ tracemode(int doit, int *old)
 		   PL_ATOM, doit ? ATOM_on : ATOM_off);
   }
   if ( doit )				/* make sure trace works inside skip */
-  { debugstatus.skiplevel = VERY_DEEP;
+  { debugstatus.skiplevel = SKIP_VERY_DEEP;
     if ( LD->trace.find )
       LD->trace.find->searching = FALSE;
   }
@@ -1588,7 +1657,7 @@ debugmode(debug_type doit, debug_type *old)
 
   if ( debugstatus.debugging != doit )
   { if ( doit )
-    { debugstatus.skiplevel = VERY_DEEP;
+    { debugstatus.skiplevel = SKIP_VERY_DEEP;
       clearPrologFlagMask(PLFLAG_LASTCALL);
       if ( doit == DBG_ALL )
       { LocalFrame fr = environment_frame;
@@ -1653,30 +1722,56 @@ pl_tracing()
   return debugstatus.tracing;
 }
 
-word
-pl_skip_level(term_t old, term_t new)
+static
+PRED_IMPL("prolog_skip_level", 2, prolog_skip_level, PL_FA_NOTRACE)
 { GET_LD
+  term_t old = A1;
+  term_t new = A2;
   atom_t a;
-  long sl;
+  size_t sl;
 
-  if ( debugstatus.skiplevel == VERY_DEEP )
+  if ( debugstatus.skiplevel == SKIP_VERY_DEEP )
   { TRY(PL_unify_atom(old, ATOM_very_deep));
+  } else if ( debugstatus.skiplevel == SKIP_REDO_IN_SKIP )
+  { TRY(PL_unify_atom(old, ATOM_redo_in_skip));
   } else
   { TRY(PL_unify_integer(old, debugstatus.skiplevel));
   }
 
-  if ( PL_get_long(new, &sl) )
-  { debugstatus.skiplevel = sl;
-    succeed;
+  if ( PL_get_atom(new, &a) )
+  { if ( a == ATOM_very_deep )
+    { debugstatus.skiplevel = SKIP_VERY_DEEP;
+      succeed;
+    } else if ( a == ATOM_redo_in_skip )
+    { debugstatus.skiplevel = SKIP_REDO_IN_SKIP;
+      succeed;
+    }
   }
-  if ( PL_get_atom(new, &a) && a == ATOM_very_deep)
-  { debugstatus.skiplevel = VERY_DEEP;
+
+  if ( PL_get_size_ex(new, &sl) )
+  { debugstatus.skiplevel = sl;
     succeed;
   }
 
   fail;
 }
 
+
+static
+PRED_IMPL("prolog_skip_frame", 1, prolog_skip_frame, PL_FA_NOTRACE)
+{ PRED_LD
+  LocalFrame fr;
+
+  if ( !PL_get_frame(A1, &fr) || !fr )
+    return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_frame_reference, A1);
+
+  debugstatus.skiplevel = levelFrame(fr);
+  set(fr, FR_SKIPPED);
+
+  return TRUE;
+}
+
+
 word
 pl_spy(term_t p)
 { GET_LD
@@ -1773,15 +1868,15 @@ prolog_frame_attribute(term_t frame, term_t what,
 
   if ( key == ATOM_argument && arity == 1 )
   { term_t arg = PL_new_term_ref();
-    int argn;
+    size_t argn;
 
-    if ( !PL_get_arg_ex(1, what, arg) || !PL_get_integer_ex(arg, &argn) )
+    if ( !PL_get_arg_ex(1, what, arg) || !PL_get_size_ex(arg, &argn) )
       fail;
     if ( argn < 1 )
       return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_natural, arg);
 
     if ( true(fr->predicate, FOREIGN) || !fr->clause )
-    { if ( argn > (int)fr->predicate->functor->arity )
+    { if ( argn > fr->predicate->functor->arity )
 	fail;
     } else
     { if ( argn > fr->clause->clause->prolog_vars )
@@ -2020,6 +2115,10 @@ PRED_IMPL("prolog_choice_attribute", 3, prolog_choice_attribute, 0)
     };
 
     return PL_unify_atom(A3, types[ch->type]);
+  } else if ( key == ATOM_pc )
+  { if ( ch->type == CHP_JUMP && ch->frame->clause )
+      return PL_unify_int64(A3, ch->value.PC - ch->frame->clause->clause->codes);
+    return FALSE;
   } else
     return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_key, A2);
 
@@ -2156,4 +2255,6 @@ callEventHook(int ev, ...)
 BeginPredDefs(trace)
   PRED_DEF("prolog_frame_attribute", 3, prolog_frame_attribute, 0)
   PRED_DEF("prolog_choice_attribute", 3, prolog_choice_attribute, 0)
+  PRED_DEF("prolog_skip_frame", 1, prolog_skip_frame, PL_FA_NOTRACE)
+  PRED_DEF("prolog_skip_level", 2, prolog_skip_level, PL_FA_NOTRACE)
 EndPredDefs
