@@ -349,8 +349,8 @@ open_foreign_frame(ARG1_LD)
 }
 
 
-static void
-close_foreign_frame(fid_t id ARG_LD)
+void
+PL_close_foreign_frame__LD(fid_t id ARG_LD)
 { FliFrame fr = (FliFrame) valTermRef(id);
 
   if ( !id || fr->magic != FLI_MAGIC )
@@ -363,9 +363,8 @@ close_foreign_frame(fid_t id ARG_LD)
 
 
 fid_t
-PL_open_foreign_frame()
-{ GET_LD
-  size_t lneeded = sizeof(struct fliFrame) + MINFOREIGNSIZE*sizeof(word);
+PL_open_foreign_frame__LD(ARG1_LD)
+{ size_t lneeded = sizeof(struct fliFrame) + MINFOREIGNSIZE*sizeof(word);
 
   if ( (char*)lTop + lneeded > (char*)lMax )
   { int rc;
@@ -380,6 +379,17 @@ PL_open_foreign_frame()
 }
 
 
+#undef PL_open_foreign_frame
+fid_t
+PL_open_foreign_frame(void)
+{ GET_LD
+
+  return PL_open_foreign_frame__LD(PASS_LD1);
+}
+				/* This local definition was here before */
+#define PL_open_foreign_frame() open_foreign_frame(PASS_LD1)
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Open a foreign frame to handle a signal.  We must skip MAXARITY words to
 deal with the fact that the WAM write-mode   writes above the top of the
@@ -422,15 +432,15 @@ PL_open_signal_foreign_frame(int sync)
 }
 
 
+#undef PL_close_foreign_frame
 void
 PL_close_foreign_frame(fid_t id)
 { GET_LD
 
-  close_foreign_frame(id PASS_LD);
+  PL_close_foreign_frame__LD(id PASS_LD);
 }
+#define PL_close_foreign_frame(id) PL_close_foreign_frame__LD(id PASS_LD)
 
-#define PL_open_foreign_frame()    open_foreign_frame(PASS_LD1)
-#define PL_close_foreign_frame(id) close_foreign_frame((id) PASS_LD)
 
 void
 PL_rewind_foreign_frame(fid_t id)
@@ -596,7 +606,7 @@ callCleanupHandler(LocalFrame fr, enum finished reason ARG_LD)
     set(fr, FR_CATCHED);
     if ( unify_finished(catcher, reason) )
     { term_t clean;
-      term_t ex;
+      term_t ex = 0;
       int rval;
       int set_env = (reason == FINISH_CUT ||
 		     reason == FINISH_EXCEPT ||
