@@ -3,9 +3,10 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (C): 1985-2010, University of Amsterdam
+			      Vu University Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -24,7 +25,8 @@
 
 #include <math.h>
 #include "pl-incl.h"
-#include "pl-ctype.h"
+#include "os/pl-dtoa.h"
+#include "os/pl-ctype.h"
 #include <stdio.h>			/* sprintf() */
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -55,9 +57,12 @@ typedef struct
   visited *visited;			/* visited (attributed-) variables */
 } write_options;
 
-static bool	writeTerm2(term_t term, int prec, write_options *options, bool arg);
-static bool	writeTerm(term_t t, int prec, write_options *options);
-static bool	writeArgTerm(term_t t, int prec, write_options *options, bool arg);
+static bool	writeTerm2(term_t term, int prec,
+			   write_options *options, bool arg) WUNUSED;
+static bool	writeTerm(term_t t, int prec,
+			  write_options *options) WUNUSED;
+static bool	writeArgTerm(term_t t, int prec,
+			     write_options *options, bool arg) WUNUSED;
 
 static Word
 address_of(term_t t)
@@ -208,6 +213,8 @@ PutOpenToken() inserts a space in the output stream if the last-written
 and given character require a space to ensure a token-break.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#define isquote(c) ((c) == '\'' || (c) == '"')
+
 static bool
 needSpace(int c, IOSTREAM *s)
 { if ( c == EOF )
@@ -217,7 +224,8 @@ needSpace(int c, IOSTREAM *s)
 	      ((isAlphaW(s->lastc) && isAlphaW(c)) ||
 	       (isSymbolW(s->lastc) && isSymbolW(c)) ||
 	       (s->lastc != '(' && !isBlank(s->lastc) && c == '(') ||
-	       (c == '\'' && isDigit(s->lastc))) )
+	       (c == '\'' && (isDigit(s->lastc))) ||
+	       (isquote(c) && s->lastc == c)) )
   { return TRUE;
   }
 
@@ -294,7 +302,8 @@ PutCloseBrace(IOSTREAM *s)
 static bool
 putQuoted(int c, int quote, int flags, IOSTREAM *stream)
 { if ( (flags & PL_WRT_CHARESCAPES) )
-  { if ( !(c < 0xff && isControl(c)) && c != quote && c != '\\' )
+  { if ( c == ' ' ||
+	 (!(c < 0xff && !isGraph(c)) && c != quote && c != '\\') )
     { TRY(Putc(c, stream));
     } else
     { char esc[8];
@@ -341,10 +350,11 @@ putQuoted(int c, int quote, int flags, IOSTREAM *stream)
 	fail;
     }
   } else
-  { if ( c == quote )
-    { TRY(Putc(c, stream) && Putc(c, stream));
-    } else
-    { return Putc(c, stream);
+  { if ( !Putc(c, stream) )
+      fail;
+    if ( c == quote || c == '\\' )	/* write '' or \\ */
+    { if ( !Putc(c, stream) )
+	fail;
     }
   }
 
@@ -452,6 +462,34 @@ static int				/* FALSE, TRUE or TRUE_WITH_SPACE */
 writeAtom(atom_t a, write_options *options)
 { Atom atom = atomValue(a);
 
+  if ( (options->flags & PL_WRT_BLOB_PORTRAY) &&
+       false(atom->type, PL_BLOB_TEXT) &&
+       GD->cleaning <= CLN_PROLOG )
+  { GET_LD
+    int rc;
+    fid_t fid;
+    predicate_t pred;
+    IOSTREAM *old;
+    term_t av;
+
+    if ( !(fid = PL_open_foreign_frame()) )
+      return FALSE;
+    av = PL_new_term_ref();
+    PL_put_atom(av, a);
+
+    pred = _PL_predicate("portray", 1, "user",
+			 &GD->procedures.portray);
+
+    old = Scurout;
+    Scurout = options->out;
+    rc = PL_call_predicate(NULL, PL_Q_NODEBUG, pred, av);
+    Scurout = old;
+
+    PL_discard_foreign_frame(fid);
+    if ( rc == TRUE )
+      return TRUE;
+  }
+
   if ( atom->type->write )
     return (*atom->type->write)(options->out, a, options->flags);
   if ( false(atom->type, PL_BLOB_TEXT) )
@@ -501,33 +539,19 @@ writeUCSAtom(IOSTREAM *fd, atom_t atom, int flags)
   size_t len = a->length/sizeof(pl_wchar_t);
   pl_wchar_t *e = &s[len];
 
-  if ( flags & PL_WRT_QUOTED )
+  if ( (flags&PL_WRT_QUOTED) && !unquoted_atomW(s, len, fd) )
   { pl_wchar_t quote = L'\'';
-    int rc;
 
-    if ( isLowerW(*s) )
-    { pl_wchar_t *q;
-
-      for(q=s; q<e; q++)
-      { if ( !isAlphaW(*q) || Scanrepresent(*q, fd) < 0 )
-	  break;
-      }
-      if ( q == e )
-        goto unquoted;
-    }
-
-    TRY(Putc(quote, fd));
+    TRY(PutOpenToken(quote, fd) &&
+	Putc(quote, fd));
 
     while(s < e)
     { TRY(putQuoted(*s++, quote, flags, fd));
     }
 
-    rc = Putc(quote, fd);
-
-    return rc;
+    return Putc(quote, fd);
   }
 
-unquoted:
   if ( s < e && !PutOpenToken(s[0], fd) )
     return FALSE;
   for( ; s<e; s++)
@@ -597,48 +621,85 @@ writeString(term_t t, write_options *options)
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Formatting a float. This is very  complicated   as  we must write floats
-such that it can be read as a float. This means using the conventions of
-the C locale and if the float happens to be integer as <int>.0.
+Formatting a float. This used  to  use   sprintf(),  but  there  are two
+problems with this. First of all, this uses the current locale, which is
+complicated to avoid. Second, it does not provide a mode that guarantees
+reliable read-back. Using %g gets closest,   but %.15g doesn't guarantee
+read-back and %.17g does, but prints 0.1 as 0.100..001, etc.
 
-Switching the locale is no option as  locale handling is not thread-safe
-and may have unwanted  consequences  for   embedding.  There  is  a intptr_t
-discussion on the very same topic on  the Python mailinglist. Many hacks
-are proposed, none is very satisfactory.   Richard  O'Keefe suggested to
-use ecvt(), fcvt() and gcvt(). These  are   not  thread-safe.  The GNU C
-library provides *_r() variations that  can   do  the  trick. An earlier
-patch used localeconv() to find the  decimal   point,  but  this is both
-complicated and not thread-safe.
+This uses dtoa.c. See pl-dtoa.c for how this is packed into SWI-Prolog.
 
-Finally, with help of Richard we decided  to replace the first character
-that is not a digit nor [eE], as this must be the decimal point.
+TBD: The number of cases are large. We should see whether it is possible
+to clean this up a bit. The 5 cases   as  such are real: there is no way
+around these.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 char *
-format_float(double f, char *buf, const char *format)
-{ char *q;
+format_float(double f, char *buf)
+{ char *end, *o=buf;
+  int decpt, sign;
+  char *s = dtoa(f, 0, 30, &decpt, &sign, &end);
 
-  sprintf(buf, format, f);
+  DEBUG(2, Sdprintf("decpt=%d, sign=%d, len = %d, '%s'\n",
+		    decpt, sign, end-s, s));
 
-  q = buf;
-  if ( *q == '-' )			/* skip -?[0-9]* */
-    q++;
-  while(*q && (isDigit(*q) || *q <= ' '))
-    q++;
+  if ( sign )
+    *o++ = '-';
 
-  switch( *q )
-  { case '\0':
-      *q++ = '.';
-      *q++ = '0';
-      *q = EOS;
-      break;
-    case 'e':
-    case 'E':
-      break;
-    default:
-      *q = '.';
+  if ( decpt <= 0 )			/* decimal dot before */
+  { if ( decpt <= -4 )
+    { *o++ = s[0];
+      *o++ = '.';
+      if ( end-s > 1 )
+      { memcpy(o, s+1, end-s-1);
+	o += end-s-1;
+      } else
+	*o++ = '0';
+      sprintf(o, "e%d", decpt-1);
+    } else
+    { int i;
+
+      *o++ = '0';
+      *o++ = '.';
+      for(i=0; i < -decpt; i++)
+	*o++ = '0';
+      memcpy(o, s, end-s);
+      o[end-s] = 0;
+    }
+  } else if ( end-s > decpt )		/* decimal dot inside */
+  { memcpy(o, s, decpt);
+    o += decpt;
+    *o++ = '.';
+    memcpy(o, s+decpt, end-s-decpt);
+    o[end-s-decpt] = 0;
+  } else				/* decimal dot after */
+  { int i;
+    int trailing = decpt-(int)(end-s);
+
+    if ( decpt > 15 )			/* over precision: use eE */
+    { *o++ = s[0];
+      *o++ = '.';
+      if ( end-s > 1 )
+      { trailing += (int)(end-s)-1;
+	memcpy(o, s+1, end-s-1);
+	o += end-s-1;
+      } else
+	*o++ = '0';
+      sprintf(o, "e+%d", trailing);
+    } else				/* within precision trail with .0 */
+    { memcpy(o, s, end-s);
+      o += end-s;
+
+      for(i=(int)(end-s); i<decpt; i++)
+	*o++ = '0';
+      *o++ = '.';
+      *o++ = '0';
+      *o = 0;
+    }
   }
 
+  freedtoa(s);
+
   return buf;
 }
 
@@ -781,7 +842,7 @@ writePrimitive(term_t t, write_options *options)
     } else
     { char buf[100];
 
-      format_float(f, buf, LD->float_format);
+      format_float(f, buf);
 
       return PutToken(buf, out);
     }
@@ -818,7 +879,7 @@ pl_nl()
 Call user:portray/1 if defined.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static bool
+static int
 callPortray(term_t arg, write_options *options)
 { predicate_t portray;
 
@@ -836,7 +897,10 @@ callPortray(term_t arg, write_options *options)
     if ( !saveWakeup(&wstate, TRUE PASS_LD) )
       return FALSE;
     Scurout = options->out;
-    rval = PL_call_predicate(NULL, PL_Q_NODEBUG, portray, arg);
+    rval = PL_call_predicate(NULL, PL_Q_NODEBUG|PL_Q_PASS_EXCEPTION,
+			     portray, arg);
+    if ( !rval && PL_exception(0) )
+      rval = -1;
     Scurout = old;
     restoreWakeup(&wstate PASS_LD);
 
@@ -849,7 +913,8 @@ callPortray(term_t arg, write_options *options)
 
 static bool
 writeArgTerm(term_t t, int prec, write_options *options, bool arg)
-{ int rval;
+{ GET_LD
+  int rval;
   int levelSave = options->depth;
   fid_t fid;
 
@@ -959,9 +1024,16 @@ writeTerm2(term_t t, int prec, write_options *options, bool arg)
   IOSTREAM *out = options->out;
 
   if ( !PL_is_variable(t) &&
-       true(options, PL_WRT_PORTRAY) &&
-       callPortray(t, options) )
-    succeed;
+       true(options, PL_WRT_PORTRAY) )
+  { switch( callPortray(t, options) )
+    { case TRUE:
+	return TRUE;
+      case FALSE:
+	break;
+      default:
+	return FALSE;
+    }
+  }
 
   if ( PL_get_atom(t, &a) )
   { if ( !arg && prec < 1200 && priorityOperator(NULL, a) > 0 )
@@ -1140,6 +1212,17 @@ writeAttributeMask(atom_t a)
 }
 
 
+static int
+writeBlobMask(atom_t a)
+{ if ( a == ATOM_default )
+  { return 0;
+  } else if ( a == ATOM_portray )
+  { return PL_WRT_BLOB_PORTRAY;
+  } else
+    return -1;
+}
+
+
 static const opt_spec write_term_options[] =
 { { ATOM_quoted,	    OPT_BOOL },
   { ATOM_ignore_ops,	    OPT_BOOL },
@@ -1153,6 +1236,7 @@ static const opt_spec write_term_options[] =
   { ATOM_priority,	    OPT_INT },
   { ATOM_partial,	    OPT_BOOL },
   { ATOM_spacing,	    OPT_ATOM },
+  { ATOM_blobs,		    OPT_ATOM },
   { NULL_ATOM,	     	    0 }
 };
 
@@ -1167,10 +1251,12 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   bool charescape = -1;			/* not set */
   atom_t mname    = ATOM_user;
   atom_t attr     = ATOM_nil;
+  atom_t blobs    = ATOM_nil;
   int  priority   = 1200;
   bool partial    = FALSE;
   IOSTREAM *s;
   write_options options;
+  int rc;
 
   memset(&options, 0, sizeof(options));
   options.spacing = ATOM_standard;
@@ -1178,7 +1264,8 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   if ( !scan_options(opts, 0, ATOM_write_option, write_term_options,
 		     &quoted, &ignore_ops, &numbervars, &portray,
 		     &charescape, &options.max_depth, &mname,
-		     &bqstring, &attr, &priority, &partial, &options.spacing) )
+		     &bqstring, &attr, &priority, &partial, &options.spacing,
+		     &blobs) )
     fail;
 
   if ( attr == ATOM_nil )
@@ -1191,6 +1278,14 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
 
     options.flags |= mask;
   }
+  if ( blobs != ATOM_nil )
+  { int mask = writeBlobMask(blobs);
+
+    if ( mask < 0 )
+      return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_write_option, opts);
+
+    options.flags |= mask;
+  }
   if ( priority < 0 || priority > OP_MAXPRIORITY )
   { term_t t = PL_new_term_ref();
     PL_put_integer(t, priority);
@@ -1230,13 +1325,13 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
     PutOpenToken(EOF, s);		/* reset this */
   if ( (options.flags & PL_WRT_QUOTED) && !(s->flags&SIO_REPPL) )
   { s->flags |= SIO_REPPL;
-    writeTerm(term, priority, &options);
+    rc = writeTerm(term, priority, &options);
     s->flags &= ~SIO_REPPL;
   } else
-  { writeTerm(term, priority, &options);
+  { rc = writeTerm(term, priority, &options);
   }
 
-  return streamStatus(s);
+  return streamStatus(s) && rc;
 }
 
 
@@ -1267,6 +1362,7 @@ do_write2(term_t stream, term_t term, int flags)
 
   if ( getOutputStream(stream, &s) )
   { write_options options;
+    int rc;
 
     memset(&options, 0, sizeof(options));
     options.flags     = flags;
@@ -1278,12 +1374,12 @@ do_write2(term_t stream, term_t term, int flags)
       options.flags |= PL_WRT_BACKQUOTED_STRING;
 
     PutOpenToken(EOF, s);		/* reset this */
-    writeTerm(term, 1200, &options);
+    rc = writeTerm(term, 1200, &options);
 
-    return streamStatus(s);
+    return streamStatus(s) && rc;
   }
 
-  fail;
+  return FALSE;
 }
 
 
