@@ -98,6 +98,8 @@ syntax(char-2) :-
 syntax(char-3) :-
 	"\\" =:= 0'\\.
 
+:- op(100, yf, af).
+
 syntax(string-1) :-
 	'\c ' == ''.
 syntax(string-2) :-
@@ -110,17 +112,36 @@ syntax(quote-3) :-
 	'\x61\' == a.
 syntax(quote-4) :-
 	char_code('\'', 39).
+syntax(quote-5) :-
+	0'\' == 0''.
+syntax(quote-6) :-
+	0'\' == 0'''.
+syntax(quote-7) :-
+	atom_to_term('\'\\\n\'', T, _),
+	T == ''.
+syntax(base-1) :-
+	1'+'1 == 1+1.
+syntax(base-2) :-
+	16'af == 175.
+syntax(base-3) :-
+	10'af' == af(10).
+syntax(base-4) :-
+	_0 = 1, B is _0'+'1, B == 2.
+syntax(base-5) :-
+	A is 1.0e+0'+'1, A == 2.0.
 syntax(number-2) :-
 	catch(atom_to_term('2\'', _, _), E, true),
-	E = error(syntax_error(illegal_number), _).
+	E = error(syntax_error(end_of_file), _).
 syntax(zero-1) :-
-	term_to_atom(T, 'hello(\000\"\000\x")'),
+	term_to_atom(T, 'hello("\000\x")'),
 	T == hello([0, 120]).
 syntax(latin-1) :-
 	atom_codes(A, [247]),
 	atom_to_term(A, T, []),
 	atom_codes(T, [247]).
 
+:- op(0, yf, af).
+
 
 		 /*******************************
 		 *	       WRITE		*
@@ -326,9 +347,9 @@ arithmetic(arith-7) :-
 arithmetic(arith-8) :-
 	-6 is min(-6, -5.5).
 arithmetic(arith-9) :-
-	4000 =:= integer(10000 * float_fractional_part(1e10 + 0.4)).
+	4000 =:= integer(10000 * float_fractional_part(1.0e10 + 0.4)).
 arithmetic(arith-10) :-
-	-4000 =:= integer(10000 * float_fractional_part(-1e10 - 0.4)).
+	-4000 =:= integer(10000 * float_fractional_part(-1.0e10 - 0.4)).
 arithmetic(float_fractional_part-1) :-
 	0 is float_fractional_part(4).
 arithmetic(arith-11) :-
@@ -379,7 +400,7 @@ arithmetic(integer-2) :-
 arithmetic(int-1) :-
 	A is 1<<31, integer(A).
 arithmetic(cmp-1) :-
-	A is 100e6, 67 < A.
+	A is 100.0e6, 67 < A.
 
 
 		 /*******************************
@@ -531,17 +552,17 @@ gmp(sign-1) :-
 	-1 =:= sign(-X),
 	1 =:= sign(X).
 gmp(floor-1) :-
-	A is floor(1e20),
+	A is floor(1.0e20),
 	integer(A),
-	1e20 =:= float(A).
+	1.0e20 =:= float(A).
 gmp(floor-2) :-
 	0 is floor(9 rdiv 10),
 	-1 is floor(-1 rdiv 10),
 	-1 is floor(-9 rdiv 10).
 gmp(ceil-1) :-
-	A is ceil(1e20),
+	A is ceil(1.0e20),
 	integer(A),
-	1e20 =:= float(A).
+	1.0e20 =:= float(A).
 gmp(ceil-2) :-
 	1 is ceil(9 rdiv 10),
 	0 is ceil(-1 rdiv 10),
@@ -1102,9 +1123,9 @@ atom_handling(name-1) :-
 atom_handling(name-2) :-
 	name(V, "5"), V == 5.
 atom_handling(name-3) :-
-	name(V, "5e4"), V =:= 50000.
+	name(V, "5.0e4"), V =:= 50000.
 atom_handling(name-4) :-
-	name(V, "5e4a"), V == '5e4a'.
+	name(V, "5.0e4a"), V == '5.0e4a'.
 atom_handling(name-5) :-
 	name(V, ""), V == ''.
 
@@ -1590,7 +1611,7 @@ exception(context-2) :-
 	error_context(E, undef/0).
 exception(catch-gc) :-
 	catch(tcatch, E, true),
-	subsumes_chk(ok(error(_,_)), E).
+	subsumes_term(ok(error(_,_)), E).
 
 
 		 /*******************************
@@ -1621,17 +1642,18 @@ gc(gc-6) :-
 	catch(1.25,_,garbage_collect).
 gc(agc-1) :-
 	garbage_collect_atoms.
-gc(agc-2) :-				% not if concurrent: this is too simple.  There
-					% are enough tests for AGC in the rest of the suite.
+gc(agc-2) :-		% not if concurrent: this is too simple.  There
+			% are enough tests for AGC in the rest of the suite.
 	(   current_prolog_flag(agc_margin, Margin),
 	    Margin > 0,
 	    \+ current_prolog_flag(test_concurrent, true)
-	->  UpTo is Margin*2,
+	->  garbage_collect_atoms,
+	    UpTo is Margin*2+10,
 	    statistics(agc_gained, Gained0),
 	    forall(between(0, UpTo, X), atom_concat(foobar, X, _)),
 	    statistics(agc_gained, Gained1),
 	    Gained is Gained1 - Gained0,
-	    Gained > UpTo - 10		% might be some junk
+	    Gained > UpTo - 20		% might be some junk
 	;   true			% no atom-gc
 	).
 
@@ -1970,19 +1992,19 @@ term_hash(simple-5) :-
 	term_hash(S, 13985775).
 term_hash(compound-1) :-
 	term_hash(hello(world), X),
-	memberchk(X, [ 12599352,	% little endian
-		       13811310		% big endian
+	memberchk(X, [ 2512014,		% little endian
+		       13811310		% big endian	FIXME: Incorrect
 		     ]).
 term_hash(compound-2) :-
 	A = x(a),
 	term_hash(hello(A, A), X),
-	memberchk(X, [ 5826661,		% little endian
-		       13137004		% big endian
+	memberchk(X, [ 6171734,		% little endian
+		       13137004		% big endian	FIXME: Incorrect
 		     ]).
 term_hash(compound-3) :-
 	term_hash(hello(x(a), x(a)), X),
-	memberchk(X, [ 5826661,		% little endian
-		       13137004		% big endian
+	memberchk(X, [ 6171734,		% little endian
+		       13137004		% big endian	FIXME: Incorrect
 		     ]).
 
 
@@ -2455,7 +2477,7 @@ mutex(unlock-1) :-
 	mutex_lock(Mutex),
 	mutex_unlock(Mutex),
 	catch(mutex_unlock(Mutex), E, true),
-	E == error(permission_error(mutex, unlock, Mutex),
+	E == error(permission_error(unlock, mutex, Mutex),
 		   context(mutex_unlock/1, 'not locked')),
 	mutex_destroy(Mutex).
 mutex(destroy-1) :-
@@ -2792,7 +2814,7 @@ blocked(Reason) :-
 %	2nd context argument.
 
 error(error(Ex, _Ctx), Expected) :-
-	subsumes_chk(Expected, Ex), !.
+	subsumes_term(Expected, Ex), !.
 error(error(Ex, _Ctx), Expected) :-
 	format('~NWrong exception: ~p (expected ~p)~n', [Ex, Expected]),
 	fail.
